@@ -4,13 +4,13 @@ You need to edit your config file to use sessions. Here's an example::
 
     [/]
     tools.sessions.on = True
-    tools.sessions.storage_type = "file"
+    tools.sessions.storage_class = cherrypy.lib.sessions.FileSession
     tools.sessions.storage_path = "/home/site/sessions"
     tools.sessions.timeout = 60
 
 This sets the session to be stored in files in the directory
 /home/site/sessions, and the session timeout to 60 minutes. If you omit
-``storage_type`` the sessions will be saved in RAM.
+``storage_class``, the sessions will be saved in RAM.
 ``tools.sessions.on`` is the only required line for working sessions,
 the rest are optional.
 
@@ -95,8 +95,6 @@ import os
 import time
 import threading
 
-import six
-
 import cherrypy
 from cherrypy._cpcompat import copyitems, pickle, random20
 from cherrypy.lib import httputil
@@ -123,10 +121,10 @@ class Session(object):
         self._id = value
         for o in self.id_observers:
             o(value)
-    id = property(_get_id, _set_id, doc="The current session ID.")
+    id = property(_get_id, _set_id, doc='The current session ID.')
 
     timeout = 60
-    "Number of minutes after which to delete session data."
+    'Number of minutes after which to delete session data.'
 
     locked = False
     """
@@ -139,16 +137,16 @@ class Session(object):
     automatically on the first attempt to access session data."""
 
     clean_thread = None
-    "Class-level Monitor which calls self.clean_up."
+    'Class-level Monitor which calls self.clean_up.'
 
     clean_freq = 5
-    "The poll rate for expired session cleanup in minutes."
+    'The poll rate for expired session cleanup in minutes.'
 
     originalid = None
-    "The session id passed by the client. May be missing or unsafe."
+    'The session id passed by the client. May be missing or unsafe.'
 
     missing = False
-    "True if the session requested by the client did not exist."
+    'True if the session requested by the client did not exist.'
 
     regenerated = False
     """
@@ -156,7 +154,7 @@ class Session(object):
     internal calls to regenerate the session id."""
 
     debug = False
-    "If True, log debug information."
+    'If True, log debug information.'
 
     # --------------------- Session management methods --------------------- #
 
@@ -472,9 +470,10 @@ class FileSession(Session):
         if isinstance(self.lock_timeout, (int, float)):
             self.lock_timeout = datetime.timedelta(seconds=self.lock_timeout)
         if not isinstance(self.lock_timeout, (datetime.timedelta, type(None))):
-            raise ValueError("Lock timeout must be numeric seconds or "
-                "a timedelta instance.")
+            raise ValueError('Lock timeout must be numeric seconds or '
+                'a timedelta instance.')
 
+    @classmethod
     def setup(cls, **kwargs):
         """Set up the storage system for file-based sessions.
 
@@ -486,12 +485,11 @@ class FileSession(Session):
 
         for k, v in kwargs.items():
             setattr(cls, k, v)
-    setup = classmethod(setup)
 
     def _get_file_path(self):
         f = os.path.join(self.storage_path, self.SESSION_PREFIX + self.id)
         if not os.path.abspath(f).startswith(self.storage_path):
-            raise cherrypy.HTTPError(400, "Invalid session id in cookie.")
+            raise cherrypy.HTTPError(400, 'Invalid session id in cookie.')
         return f
 
     def _exists(self):
@@ -499,12 +497,12 @@ class FileSession(Session):
         return os.path.exists(path)
 
     def _load(self, path=None):
-        assert self.locked, ("The session load without being locked.  "
+        assert self.locked, ('The session load without being locked.  '
                              "Check your tools' priority levels.")
         if path is None:
             path = self._get_file_path()
         try:
-            f = open(path, "rb")
+            f = open(path, 'rb')
             try:
                 return pickle.load(f)
             finally:
@@ -512,21 +510,21 @@ class FileSession(Session):
         except (IOError, EOFError):
             e = sys.exc_info()[1]
             if self.debug:
-                cherrypy.log("Error loading the session pickle: %s" %
+                cherrypy.log('Error loading the session pickle: %s' %
                              e, 'TOOLS.SESSIONS')
             return None
 
     def _save(self, expiration_time):
-        assert self.locked, ("The session was saved without being locked.  "
+        assert self.locked, ('The session was saved without being locked.  '
                              "Check your tools' priority levels.")
-        f = open(self._get_file_path(), "wb")
+        f = open(self._get_file_path(), 'wb')
         try:
             pickle.dump((self._data, expiration_time), f, self.pickle_protocol)
         finally:
             f.close()
 
     def _delete(self):
-        assert self.locked, ("The session deletion without being locked.  "
+        assert self.locked, ('The session deletion without being locked.  '
                              "Check your tools' priority levels.")
         try:
             os.unlink(self._get_file_path())
@@ -598,11 +596,12 @@ class MemcachedSession(Session):
     # Wrap all .get and .set operations in a single lock.
     mc_lock = threading.RLock()
 
-    # This is a seperate set of locks per session id.
+    # This is a separate set of locks per session id.
     locks = {}
 
     servers = ['127.0.0.1:11211']
 
+    @classmethod
     def setup(cls, **kwargs):
         """Set up the storage system for memcached-based sessions.
 
@@ -614,21 +613,6 @@ class MemcachedSession(Session):
 
         import memcache
         cls.cache = memcache.Client(cls.servers)
-    setup = classmethod(setup)
-
-    def _get_id(self):
-        return self._id
-
-    def _set_id(self, value):
-        # This encode() call is where we differ from the superclass.
-        # Memcache keys MUST be byte strings, not unicode.
-        if isinstance(value, six.text_type):
-            value = value.encode('utf-8')
-
-        self._id = value
-        for o in self.id_observers:
-            o(value)
-    id = property(_get_id, _set_id, doc="The current session ID.")
 
     def _exists(self):
         self.mc_lock.acquire()
@@ -651,7 +635,7 @@ class MemcachedSession(Session):
         try:
             if not self.cache.set(self.id, (self._data, expiration_time), td):
                 raise AssertionError(
-                    "Session data for id %r not set." % self.id)
+                    'Session data for id %r not set.' % self.id)
         finally:
             self.mc_lock.release()
 
@@ -680,13 +664,13 @@ class MemcachedSession(Session):
 def save():
     """Save any changed session data."""
 
-    if not hasattr(cherrypy.serving, "session"):
+    if not hasattr(cherrypy.serving, 'session'):
         return
     request = cherrypy.serving.request
     response = cherrypy.serving.response
 
     # Guard against running twice
-    if hasattr(request, "_sessionsaved"):
+    if hasattr(request, '_sessionsaved'):
         return
     request._sessionsaved = True
 
@@ -705,8 +689,8 @@ save.failsafe = True
 
 def close():
     """Close the session object for this request."""
-    sess = getattr(cherrypy.serving, "session", None)
-    if getattr(sess, "locked", False):
+    sess = getattr(cherrypy.serving, 'session', None)
+    if getattr(sess, 'locked', False):
         # If the session is still locked we release the lock
         sess.release_lock()
         if sess.debug:
@@ -715,12 +699,19 @@ close.failsafe = True
 close.priority = 90
 
 
-def init(storage_type='ram', path=None, path_header=None, name='session_id',
+def init(storage_type=None, path=None, path_header=None, name='session_id',
          timeout=60, domain=None, secure=False, clean_freq=5,
-         persistent=True, httponly=False, debug=False, **kwargs):
+         persistent=True, httponly=False, debug=False,
+         # Py27 compat
+         # *, storage_class=RamSession,
+         **kwargs):
     """Initialize session object (using cookies).
 
+    storage_class
+        The Session subclass to use. Defaults to RamSession.
+
     storage_type
+        (deprecated)
         One of 'ram', 'file', memcached'. This will be
         used to look up the corresponding class in cherrypy.lib.sessions
         globals. For example, 'file' will use the FileSession class.
@@ -765,10 +756,13 @@ def init(storage_type='ram', path=None, path_header=None, name='session_id',
     you're using for more information.
     """
 
+    # Py27 compat
+    storage_class = kwargs.pop('storage_class', RamSession)
+
     request = cherrypy.serving.request
 
     # Guard against running twice
-    if hasattr(request, "_session_init_flag"):
+    if hasattr(request, '_session_init_flag'):
         return
     request._session_init_flag = True
 
@@ -780,11 +774,18 @@ def init(storage_type='ram', path=None, path_header=None, name='session_id',
             cherrypy.log('ID obtained from request.cookie: %r' % id,
                          'TOOLS.SESSIONS')
 
-    # Find the storage class and call setup (first time only).
-    storage_class = storage_type.title() + 'Session'
-    storage_class = globals()[storage_class]
-    if not hasattr(cherrypy, "session"):
-        if hasattr(storage_class, "setup"):
+    first_time = not hasattr(cherrypy, 'session')
+
+    if storage_type:
+        if first_time:
+            msg = 'storage_type is deprecated. Supply storage_class instead'
+            cherrypy.log(msg)
+        storage_class = storage_type.title() + 'Session'
+        storage_class = globals()[storage_class]
+
+    # call setup first time only
+    if first_time:
+        if hasattr(storage_class, 'setup'):
             storage_class.setup(**kwargs)
 
     # Create and attach a new Session instance to cherrypy.serving.
@@ -801,7 +802,7 @@ def init(storage_type='ram', path=None, path_header=None, name='session_id',
     sess.id_observers.append(update_cookie)
 
     # Create cherrypy.session which will proxy to cherrypy.serving.session
-    if not hasattr(cherrypy, "session"):
+    if not hasattr(cherrypy, 'session'):
         cherrypy.session = cherrypy._ThreadLocalProxy('session')
 
     if persistent:
@@ -869,7 +870,7 @@ def set_response_cookie(path=None, path_header=None, name='session_id',
         cookie[name]['secure'] = 1
     if httponly:
         if not cookie[name].isReservedKey('httponly'):
-            raise ValueError("The httponly cookie token is not supported.")
+            raise ValueError('The httponly cookie token is not supported.')
         cookie[name]['httponly'] = 1
 
 
