@@ -12,7 +12,7 @@ function ViewModel() {
     self.isRestarting = ko.observable(false);
     self.useGlobalOptions = ko.observable(true).extend({ persist: 'useGlobalOptions' });
     self.refreshRate = ko.observable(1).extend({ persist: 'pageRefreshRate' });
-    self.dateFormat = ko.observable('DD/MM/YYYY HH:mm').extend({ persist: 'pageDateFormat' });
+    self.dateFormat = ko.observable('fromNow').extend({ persist: 'pageDateFormat' });
     self.displayTabbed = ko.observable().extend({ persist: 'displayTabbed' });
     self.displayCompact = ko.observable(false).extend({ persist: 'displayCompact' });
     self.confirmDeleteQueue = ko.observable(true).extend({ persist: 'confirmDeleteQueue' });
@@ -40,6 +40,8 @@ function ViewModel() {
     self.quotaLimit = ko.observable();
     self.quotaLimitLeft = ko.observable();
     self.systemLoad = ko.observable();
+    self.cacheSize = ko.observable();
+    self.cacheArticles = ko.observable();
     self.nrWarnings = ko.observable(0);
     self.allWarnings = ko.observableArray([]);
     self.allMessages = ko.observableArray([]);
@@ -48,7 +50,7 @@ function ViewModel() {
 
     // Statusinfo container
     self.hasStatusInfo = ko.observable(false);
-    self.hasDiskStatusInfo = ko.observable(false);
+    self.hasPerformanceInfo = ko.observable(false);
     self.statusInfo = {};
     self.statusInfo.folders = ko.observableArray([]);
     self.statusInfo.servers = ko.observableArray([]);
@@ -59,8 +61,6 @@ function ViewModel() {
     self.statusInfo.pystone = ko.observable();
     self.statusInfo.cpumodel = ko.observable();
     self.statusInfo.loglevel = ko.observable();
-    self.statusInfo.cache_size = ko.observable();
-    self.statusInfo.cache_art = ko.observable();
     self.statusInfo.downloaddir = ko.observable();
     self.statusInfo.downloaddirspeed = ko.observable();
     self.statusInfo.completedir = ko.observable();
@@ -74,16 +74,16 @@ function ViewModel() {
     self.speedLimitText = ko.pureComputed(function() {
         // Set?
         if(!self.bandwithLimit()) return;
-        
-        // The text 
+
+        // The text
         var bandwithLimitText = self.bandwithLimit().replace(/[^a-zA-Z]+/g, '');
-        
+
         // Only the number
         var speedLimitNumberFull = (parseFloat(self.bandwithLimit()) * (self.speedLimit() / 100));
 
         // Trick to only get decimal-point when needed
         var speedLimitNumber = Math.round(speedLimitNumberFull*10)/10;
-        
+
         // Fix it for lower than 1MB/s
         if(bandwithLimitText == 'M' && speedLimitNumber < 1) {
             bandwithLimitText = 'K';
@@ -146,7 +146,7 @@ function ViewModel() {
             Possible login failure?
         ***/
         if(response.hasOwnProperty('error') && response.error == 'Missing authentication') {
-            // Restart 
+            // Restart
             document.location = document.location;
         }
 
@@ -171,10 +171,10 @@ function ViewModel() {
         } else {
             self.diskSpaceLeft2('')
         }
-        
+
         // Did we exceed the space?
-        self.diskSpaceExceeded1(parseInt(response.queue.mbleft)/1024 > parseInt(response.queue.diskspace1))
-        self.diskSpaceExceeded2(parseInt(response.queue.mbleft)/1024 > parseInt(response.queue.diskspace2))
+        self.diskSpaceExceeded1(parseInt(response.queue.mbleft)/1024 > parseFloat(response.queue.diskspace1))
+        self.diskSpaceExceeded2(parseInt(response.queue.mbleft)/1024 > parseFloat(response.queue.diskspace2))
 
         // Quota
         self.quotaLimit(response.queue.quota)
@@ -183,6 +183,10 @@ function ViewModel() {
         // System load
         self.systemLoad(response.queue.loadavg)
 
+        // Cache
+        self.cacheSize(response.queue.cache_size)
+        self.cacheArticles(response.queue.cache_art)
+
         // Warnings (new warnings will trigger an update of allMessages)
         self.nrWarnings(response.queue.have_warnings)
 
@@ -220,7 +224,7 @@ function ViewModel() {
                     // Update internally
                     self.speedHistory = sabSpeedHistory;
                 }
-                
+
                 // Create
                 $('.sparkline').peity("line", {
                     width: 275,
@@ -229,7 +233,7 @@ function ViewModel() {
                     stroke: '#AAFFAA',
                     values: sabSpeedHistory
                 })
-                
+
             } else {
                 // Update
                 $('.sparkline').text(self.speedHistory.join(",")).change()
@@ -270,9 +274,15 @@ function ViewModel() {
                 var hours = Math.floor(seconds / 3600);
                 var minutes = Math.floor((seconds -= hours * 3600) / 60);
                 seconds -= minutes * 60;
+
+                // Add leading zeros
+                if(minutes < 10) minutes = '0' + minutes;
+                if(seconds < 10) seconds = '0' + seconds;
+
+                // Final formating
                 timeString = glitterTranslate.paused + ' (' + rewriteTime(hours + ":" + minutes + ":" + seconds) + ')';
             }
-            
+
             // Add info about amount of download (if actually downloading)
             if(response.queue.noofslots > 0 && parseInt(self.queueDataLeft()) > 0) {
                 self.title(timeString + ' - ' + self.queueDataLeft() + ' ' + glitterTranslate.left + ' - SABnzbd')
@@ -310,17 +320,24 @@ function ViewModel() {
     self.refresh = function(forceFullHistory) {
         // Clear previous timeout to prevent double-calls
         clearTimeout(self.interval);
-        
+
         /**
             Limited refresh
         **/
         // Only update the title when page not visible
         if(!pageIsVisible) {
-            // Request new title 
+            // Request new title
             callSpecialAPI('./queue/', { limit: 1, start: 0 }).done(function(data) {
                 // Split title & speed
                 var dataSplit = data.split('|||');
 
+                // Maybe the result is actually the login page?
+                if(dataSplit[0].substring(0, 11) === '<html lang=') {
+                    // Redirect
+                    document.location = document.location
+                    return
+                }
+
                 // Set title
                 self.title(dataSplit[0]);
 
@@ -392,7 +409,7 @@ function ViewModel() {
             limit: parseInt(self.history.paginationLimit()),
             last_history_update: self.history.lastUpdate
         }).done(self.updateHistory);
-        
+
         // We are now done with any loading
         // But we wait a few ms so Knockout has time to update
         setTimeout(function() {
@@ -434,12 +451,12 @@ function ViewModel() {
         }
         // Show modal
         $('#modal_custom_pause').modal('show')
-        
+
         // Focus on the input field
         $('#modal_custom_pause').on('shown.bs.modal', function () {
             $('#customPauseInput').focus()
         })
-        
+
         // Reset on modal close
         $('#modal_custom_pause').on('hide.bs.modal', function () {
             self.pauseCustom('');
@@ -460,10 +477,16 @@ function ViewModel() {
             $('#modal_custom_pause .btn-default').addClass('disabled')
             return;
         }
-        
+
+        // Fix DateJS bug it has some strange problem with the current day-of-month + 1
+        // Removing the space makes DateJS work properly
+        newValue = newValue.replace(/\s*h|\s*m|\s*d/g, function(match) {
+            return match.trim()
+        });
+
         // Parse
         var pauseParsed = Date.parse(newValue);
-        
+
         // Did we get it?
         if(pauseParsed) {
             // Is it just now?
@@ -471,7 +494,7 @@ function ViewModel() {
                 // Try again with the '+' in front, the parser doesn't get 100min
                 pauseParsed = Date.parse('+' + newValue);
             }
-            
+
             // Calculate difference in minutes and save
             var pauseDuration = Math.round((pauseParsed - Date.parse('now'))/1000/60);
             $('#customPauseOutput').html('<span class="glyphicon glyphicon-pause"></span> ' +glitterTranslate.pauseFor + ' ' + pauseDuration + ' ' + glitterTranslate.minutes)
@@ -488,7 +511,7 @@ function ViewModel() {
     self.saveCustomPause = function() {
         // Get duration
         var pauseDuration = $('#customPauseOutput').data('time');
-        
+
         // If in the future
         if(pauseDuration > 0) {
             callAPI({
@@ -508,33 +531,30 @@ function ViewModel() {
     self.nrWarnings.subscribe(function(newValue) {
         // Really any change?
         if(newValue == self.allWarnings().length) return;
-        
+
         // Get all warnings
         callAPI({
             mode: 'warnings'
         }).then(function(response) {
+
             // Reset it all
             self.allWarnings.removeAll();
             if(response) {
                 // Newest first
                 response.warnings.reverse()
-                
+
                 // Go over all warnings and add
                 $.each(response.warnings, function(index, warning) {
-                    // Split warning into parts
-                    var warningSplit = warning.split(/\n/);
-                    
                     // Reformat CSS label and date
+                    // Replaces spaces by non-breakable spaces and newlines with br's
                     var warningData = {
                         index: index,
-                        type: glitterTranslate.status[warningSplit[1]].slice(0, -1),
-                        text: warningSplit.slice(2).join('<br/>').replace(/ /g, '\u00A0'), // Recombine if multiple lines
-                        date: displayDateTime(warningSplit[0], self.dateFormat(), 'YYYY-MM-DD HH:mm'),
-                        timestamp: moment(warningSplit[0], 'YYYY-MM-DD HH:mm').unix(),
-                        css: (warningSplit[1] == "ERROR" ? "danger" : warningSplit[1] == "WARNING" ? "warning" : "info"),
+                        type: glitterTranslate.status[warning.type].slice(0, -1),
+                        text: convertHTMLtoText(warning.text).replace(/ /g, '\u00A0').replace(/(?:\r\n|\r|\n)/g, '<br />'),
+                        timestamp: warning.time,
+                        css: (warning.type == "ERROR" ? "danger" : warning.type == "WARNING" ? "warning" : "info"),
                         clear: self.clearWarnings
                     };
-
                     self.allWarnings.push(warningData)
                 })
             }
@@ -559,7 +579,7 @@ function ViewModel() {
     self.speedLimit.subscribe(function(newValue) {
         // Only on new load
         if(!self.speedLimitInt()) return;
-        
+
         // Update
         if(self.speedLimitInt() != newValue) {
             callAPI({
@@ -583,17 +603,20 @@ function ViewModel() {
     }
 
     // Shutdown options
-    self.onQueueFinish.subscribe(function(newValue) {
+    self.setOnQueueFinish = function(model, event) {
         // Ignore updates before the page is done
         if(!self.hasStatusInfo()) return;
-        
+
         // Something changes
         callAPI({
             mode: 'queue',
             name: 'change_complete_action',
-            value: newValue
+            value: $(event.target).val()
         })
-    })
+
+        // Top stop blinking while the API is calling
+        self.onQueueFinish($(event.target).val())
+    }
 
     // Use global settings or device-specific?
     self.useGlobalOptions.subscribe(function(newValue) {
@@ -605,7 +628,7 @@ function ViewModel() {
     self.refreshRate.subscribe(function(newValue) {
         // Refresh now
         self.refresh();
-        
+
         // Save in config if global-settings
         if(self.useGlobalOptions()) {
             callAPI({
@@ -652,16 +675,22 @@ function ViewModel() {
             return false;
         }
 
-        // Add 
-        callAPI({
+        // Build request
+        var theCall = {
             mode: "addurl",
             name: $(form.nzbURL).val(),
             nzbname: $('#nzbname').val(),
-            cat: $('#modal-add-nzb select[name="Category"]').val() == '' ? 'Default' : $('#modal-add-nzb select[name="Category"]').val(),
-            script: $('#modal-add-nzb select[name="Post-processing"]').val() == '' ? 'Default' : $('#modal-add-nzb select[name="Post-processing"]').val(),
-            priority: $('#modal-add-nzb select[name="Priority"]').val() == '' ? -100 : $('#modal-add-nzb select[name="Priority"]').val(),
-            pp: $('#modal-add-nzb select[name="Processing"]').val() == '' ? -1 : $('#modal-add-nzb select[name="Processing"]').val()
-        }).then(function(r) {
+            script: $('#modal-add-nzb select[name="Post-processing"]').val(),
+            priority: $('#modal-add-nzb select[name="Priority"]').val(),
+            pp: $('#modal-add-nzb select[name="Processing"]').val()
+        }
+
+        // Optional, otherwise they get mis-labeled if left empty
+        if($('#modal-add-nzb select[name="Category"]').val() != '*') theCall.cat = $('#modal-add-nzb select[name="Category"]').val()
+        if($('#modal-add-nzb select[name="Processing"]').val()) theCall.pp = $('#modal-add-nzb select[name="Category"]').val()
+
+        // Add
+        callAPI(theCall).then(function(r) {
             // Hide and reset/refresh
             self.refresh()
             $("#modal-add-nzb").modal("hide");
@@ -679,6 +708,9 @@ function ViewModel() {
         var file = files[fileindex]
         fileindex++
 
+        // Check if it's maybe a folder, we can't handle those
+        if(!file.type && file.size % 4096 == 0) return;
+
         // Add notification
         showNotification('.main-notification-box-uploading', 0, fileindex)
 
@@ -687,12 +719,14 @@ function ViewModel() {
         data.append("name", file);
         data.append("mode", "addfile");
         data.append("nzbname", $('#nzbname').val());
-        data.append("cat", $('#modal-add-nzb select[name="Category"]').val() == '' ? 'Default' : $('#modal-add-nzb select[name="Category"]').val()); // Default category
-        data.append("script", $('#modal-add-nzb select[name="Post-processing"]').val() == '' ? 'Default' : $('#modal-add-nzb select[name="Post-processing"]').val()); // Default script
-        data.append("priority", $('#modal-add-nzb select[name="Priority"]').val() == '' ? -100 : $('#modal-add-nzb select[name="Priority"]').val()); // Default priority
-        data.append("pp", $('#modal-add-nzb select[name="Processing"]').val() == '' ? -1 : $('#modal-add-nzb select[name="Processing"]').val()); // Default post-processing options
+        data.append("script", $('#modal-add-nzb select[name="Post-processing"]').val())
+        data.append("priority", $('#modal-add-nzb select[name="Priority"]').val())
         data.append("apikey", apiKey);
-        
+
+        // Optional, otherwise they get mis-labeled if left empty
+        if($('#modal-add-nzb select[name="Category"]').val() != '*') data.append("cat", $('#modal-add-nzb select[name="Category"]').val());
+        if($('#modal-add-nzb select[name="Processing"]').val()) data.append("pp", $('#modal-add-nzb select[name="Processing"]').val());
+
         // Add this one
         $.ajax({
             url: "./tapi",
@@ -729,11 +763,9 @@ function ViewModel() {
         self.hasStatusInfo(false)
 
         // Load the custom status info
-        callAPI({ mode: 'fullstatus', skip_dashboard: (!statusFullRefresh)*1 }).then(function(data) {                
+        callAPI({ mode: 'fullstatus', skip_dashboard: (!statusFullRefresh)*1 }).then(function(data) {
             // Update basic
             self.statusInfo.loglevel(data.status.loglevel)
-            self.statusInfo.cache_art(data.status.cache_art)
-            self.statusInfo.cache_size(data.status.cache_size)
             self.statusInfo.folders(data.status.folders)
 
             // Update the full set
@@ -749,46 +781,59 @@ function ViewModel() {
                 self.statusInfo.publicipv4(data.status.publicipv4)
                 self.statusInfo.ipv6(data.status.ipv6 || glitterTranslate.noneText)
                 // Loaded disk info
-                self.hasDiskStatusInfo(true)
+                self.hasPerformanceInfo(true)
             }
-            
+
             // Update the servers
-            if(self.statusInfo.servers().length == 0) {
+            if(self.statusInfo.servers().length != data.status.servers.length) {
+                // Only now we can subscribe to the log-level-changes! (only at start)
+                if(self.statusInfo.servers().length == 0) {
+                    self.statusInfo.loglevel.subscribe(function(newValue) {
+                        // Update log-level
+                        callSpecialAPI('./status/change_loglevel/', {
+                            loglevel: newValue
+                        });
+                    })
+                }
+
+                // Empty them, in case of update
+                self.statusInfo.servers([])
+
                 // Initial add
                 $.each(data.status.servers, function() {
                     self.statusInfo.servers.push({
-                        'servername': this.servername,
-                        'serveroptional': this.serveroptional,
-                        'serverpriority': this.serverpriority,
-                        'servertotalconn': this.servertotalconn,
+                        'servername': ko.observable(this.servername),
+                        'serveroptional': ko.observable(this.serveroptional),
+                        'serverpriority': ko.observable(this.serverpriority),
+                        'servertotalconn': ko.observable(this.servertotalconn),
+                        'serverssl': ko.observable(this.serverssl),
+                        'serversslinfo': ko.observable(this.serversslinfo),
                         'serveractiveconn': ko.observable(this.serveractiveconn),
                         'servererror': ko.observable(this.servererror),
                         'serveractive': ko.observable(this.serveractive),
                         'serverconnections': ko.observableArray(this.serverconnections)
                     })
                 })
-
-                // Only now we can subscribe to the log-level-changes!
-                self.statusInfo.loglevel.subscribe(function(newValue) {
-                    // Update log-level
-                    callSpecialAPI('./status/change_loglevel/', {
-                        loglevel: newValue
-                    });
-                })
             } else {
                 // Update
                 $.each(data.status.servers, function(index) {
                     var activeServer = self.statusInfo.servers()[index];
-                    activeServer.serveractiveconn(this.serveractiveconn)
-                    activeServer.servererror(this.servererror)
-                    activeServer.serveractive(this.serveractive)
+                    activeServer.servername(this.servername),
+                    activeServer.serveroptional(this.serveroptional),
+                    activeServer.serverpriority(this.serverpriority),
+                    activeServer.servertotalconn(this.servertotalconn),
+                    activeServer.serverssl(this.serverssl),
+                    activeServer.serversslinfo(this.serversslinfo),
+                    activeServer.serveractiveconn(this.serveractiveconn),
+                    activeServer.servererror(this.servererror),
+                    activeServer.serveractive(this.serveractive),
                     activeServer.serverconnections(this.serverconnections)
                 })
             }
 
             // Add tooltips to possible new items
             if(!isMobile) $('#modal-options [data-tooltip="true"]').tooltip({ trigger: 'hover', container: 'body' })
-        
+
             // Stop it spin
             self.hasStatusInfo(true)
         });
@@ -796,7 +841,7 @@ function ViewModel() {
 
     // Do a disk-speedtest
     self.testDiskSpeed = function(item, event) {
-        self.hasDiskStatusInfo(false)
+        self.hasPerformanceInfo(false)
 
         // Run it and then display it
         callSpecialAPI('./status/dashrefresh/').then(function() {
@@ -897,7 +942,7 @@ function ViewModel() {
                 hideNotification(true)
                 self.loadStatusInfo(true, true)
             })
-        }     
+        }
     }
 
     // Orphaned folder adding of all
@@ -911,7 +956,7 @@ function ViewModel() {
                 hideNotification(true)
                 self.loadStatusInfo(true, true)
             })
-        }     
+        }
     }
 
     // Toggle Glitter's compact layout dynamically
@@ -993,7 +1038,7 @@ function ViewModel() {
     if(localStorageGetItem('displayTabbed') === 'true') {
         $('body').addClass('container-tabbed')
     }
-    
+
     // Get the speed-limit, refresh rate and server names
     callAPI({
         mode: 'get_config'
@@ -1003,21 +1048,21 @@ function ViewModel() {
             // Set refreshrate (defaults to 1/s)
             if(!response.config.misc.refresh_rate) response.config.misc.refresh_rate = 1;
             self.refreshRate(response.config.misc.refresh_rate.toString());
-            
+
             // Set history limit
             self.history.paginationLimit(response.config.misc.history_limit.toString())
-            
+
             // Set queue limit
             self.queue.paginationLimit(response.config.misc.queue_limit.toString())
         }
-        
+
         // Set bandwidth limit
         if(!response.config.misc.bandwidth_max) response.config.misc.bandwidth_max = false;
         self.bandwithLimit(response.config.misc.bandwidth_max);
-        
+
         // Save servers (for reporting functionality of OZnzb)
         self.servers = response.config.servers;
-        
+
         // Update message
         if(newRelease) {
             self.allMessages.push({
@@ -1046,7 +1091,7 @@ function ViewModel() {
                 type: glitterTranslate.status['INFO'],
                 text: glitterTranslate.glitterTips + ' <a class="queue-update-sab" href="https://sabnzbd.org/wiki/extra/glitter-tips-and-tricks" target="_blank">Glitter Tips and Tricks <span class="glyphicon glyphicon-new-window"></span></a>',
                 css: 'info',
-                clear: function() { 
+                clear: function() {
                     // Update the config to not show again
                     callAPI({
                         mode: 'set_config',
@@ -1069,11 +1114,11 @@ function ViewModel() {
         setTimeout(self.loadStatusInfo, 200);
     }
 
-    // On any status load we check Orphaned folders 
-    self.hasStatusInfo.subscribe(function(finishedLoading) { 
+    // On any status load we check Orphaned folders
+    self.hasStatusInfo.subscribe(function(finishedLoading) {
         // Loaded or just starting?
         if(!finishedLoading) return;
-        
+
         // Orphaned folders? If user clicked away we check again in 5 days
         if(self.statusInfo.folders().length >= 3 && orphanMsg) {
             // Check if not already there
@@ -1114,7 +1159,7 @@ function ViewModel() {
     $('[name="general-date-format"] option').each(function() {
         $(this).text(displayDateTime('', $(this).val()), '')
     })
-           
+
     // Update the date every minute
     setInterval(function() {
         $('[data-timestamp]').each(function() {
