@@ -10,7 +10,7 @@ function QueueListModel(parent) {
     self.rawScriptList = [];
 
     // Because SABNZB returns the name
-    // But when you want to set Priority you need the number.. 
+    // But when you want to set Priority you need the number..
     self.priorityName = [];
     self.priorityName["Force"] = 2;
     self.priorityName["High"] = 1;
@@ -58,15 +58,15 @@ function QueueListModel(parent) {
             self.dragging = false;
         }, 500)
     }
-    
+
     // Update slots from API data
     self.updateFromData = function(data) {
         // Get all ID's
         var itemIds = $.map(self.queueItems(), function(i) {
             return i.id;
         });
-        
-        // Did the category-list change? 
+
+        // Did the category-list change?
         // Otherwise KO will send updates to all <select> for every refresh()
         if(self.rawCatList != data.categories.toString()) {
             // Reformat categories
@@ -78,8 +78,8 @@ function QueueListModel(parent) {
             // Update
             self.rawCatList = data.categories.toString();
         }
-        
-        // Did the script-list change? 
+
+        // Did the script-list change?
         if(self.rawScriptList != data.scripts.toString()) {
             // Reformat script-list
             self.scriptsList($.map(data.scripts, function(script) {
@@ -93,7 +93,7 @@ function QueueListModel(parent) {
 
         // Set limit
         self.totalItems(data.noofslots);
-        
+
         // Container for new models
         var newItems = [];
 
@@ -112,7 +112,7 @@ function QueueListModel(parent) {
                 newItems.push(new QueueModel(self, item))
             }
         });
-        
+
         // Remove all items if there's any
         if(itemIds.length == self.paginationLimit()) {
             // Replace it, so only 1 Knockout DOM-update!
@@ -127,13 +127,13 @@ function QueueListModel(parent) {
                 }));
             });
         }
-        
+
         // New items, then add!
         if(newItems.length > 0) {
             ko.utils.arrayPushAll(self.queueItems, newItems);
             self.queueItems.valueHasMutated();
         }
-        
+
         // Sort every time (takes just few msec)
         self.queueItems.sort(function(a, b) {
             return a.index() < b.index() ? -1 : 1;
@@ -142,13 +142,12 @@ function QueueListModel(parent) {
 
     // Move in sortable
     self.move = function(event) {
-        var itemMoved = event.item;  
+        var itemMoved = event.item;
         // Up or down?
-        var corTerm = event.targetIndex > event.sourceIndex ? -1 : 1;         
+        var corTerm = event.targetIndex > event.sourceIndex ? -1 : 1;
         // See what the actual index is of the queue-object
         // This way we can see how we move up and down independent of pagination
         var itemReplaced = self.queueItems()[event.targetIndex+corTerm];
-
         callAPI({
             mode: "switch",
             value: itemMoved.id,
@@ -156,9 +155,28 @@ function QueueListModel(parent) {
         }).then(self.parent.refresh);
     };
 
+    // Move button clicked
+    self.moveButton = function(event,ui) {
+        var itemMoved = event;
+        var targetIndex;
+        if($(ui.currentTarget).is(".buttonMoveToTop")){
+            //we want to move to the top
+            targetIndex = 0;
+        } else {
+            // we want to move to the bottom
+			targetIndex = self.totalItems() - 1;
+        }
+        callAPI({
+            mode: "switch",
+            value: itemMoved.id,
+            value2: targetIndex
+        }).then(self.parent.refresh);
+
+    }
+
     // Save pagination state
-    self.paginationLimit.subscribe(function(newValue) {   
-        // Save in config if global 
+    self.paginationLimit.subscribe(function(newValue) {
+        // Save in config if global
         if(self.parent.useGlobalOptions()) {
             callAPI({
                 mode: "set_config",
@@ -168,12 +186,12 @@ function QueueListModel(parent) {
             })
         }
     });
-    
+
     // Do we show search box. So it doesn't dissapear when nothing is found
     self.hasQueueSearch = ko.pureComputed(function() {
-        return (self.pagination.hasPagination() || self.searchTerm())
+        return (self.pagination.hasPagination() || self.searchTerm() || (self.parent.hasQueue() && self.isMultiEditing()))
     })
-    
+
     // Searching in queue (rate-limited in decleration)
     self.searchTerm.subscribe(function() {
         // Refresh now
@@ -183,7 +201,7 @@ function QueueListModel(parent) {
             self.pagination.moveToPage(1);
         }
     })
-    
+
     // Clear searchterm
     self.clearSearchTerm = function(data, event) {
         // Was it escape key or click?
@@ -233,7 +251,7 @@ function QueueListModel(parent) {
                 dir = 'desc';
                 break;
         }
-        
+
         // Show notification
         showNotification('.main-notification-box-sorting', 2000)
 
@@ -252,12 +270,12 @@ function QueueListModel(parent) {
         self.isMultiEditing(!self.isMultiEditing())
         // Form
         var $form = $('form.multioperations-selector')
-        
+
         // Reset form and remove all checked ones
         $form[0].reset();
         self.multiEditItems.removeAll();
         $('.delete input[name="multiedit"], #multiedit-checkall').prop({'checked': false, 'indeterminate': false})
-        
+
         // Is the multi-edit in view?
         if(($form.offset().top + $form.outerHeight(true)) > ($(window).scrollTop()+$(window).height())) {
             // Scroll to form
@@ -281,24 +299,24 @@ function QueueListModel(parent) {
             // Update them all
             self.doMultiEditUpdate();
         } else {
-            // Go over them all to know which one to remove 
+            // Go over them all to know which one to remove
             self.multiEditItems.remove(function(inList) { return inList.id == item.id; })
         }
-        
+
         // Update check-all buton state
         setCheckAllState('#multiedit-checkall', '.queue-table input[name="multiedit"]')
         return true;
     }
-    
+
     // Check all
     self.checkAllJobs = function(item, event) {
         // Get which ones we care about
         var allChecks = $('.queue-table input[name="multiedit"]').filter(':not(:disabled):visible');
-        
+
         // We need to re-evaltuate the state of this check-all
         // Otherwise the 'inderterminate' will be overwritten by the click event!
         setCheckAllState('#multiedit-checkall', '.queue-table input[name="multiedit"]')
-        
+
         // Now we can check what happend
         // For when some are checked, or all are checked (but not partly)
         if(event.target.indeterminate || (event.target.checked && !event.target.indeterminate)) {
@@ -312,7 +330,7 @@ function QueueListModel(parent) {
             } else {
                 // Remove them seperate
                 allActive.each(function() {
-                    // Go over them all to know which one to remove 
+                    // Go over them all to know which one to remove
                     var item = ko.dataFor(this)
                     self.multiEditItems.remove(function(inList) { return inList.id == item.id; })
                     // Remove the check of this one
@@ -324,7 +342,7 @@ function QueueListModel(parent) {
             allChecks.prop('checked', true)
             allChecks.each(function() { self.multiEditItems.push(ko.dataFor(this)) })
             event.target.checked = true
-            
+
             // Now we fire the update
             self.doMultiEditUpdate()
         }
@@ -337,7 +355,7 @@ function QueueListModel(parent) {
     self.doMultiEditUpdate = function() {
         // Anything selected?
         if(self.multiEditItems().length < 1) return;
-        
+
         // Retrieve the current settings
         var newCat = $('.multioperations-selector select[name="Category"]').val()
         var newScript = $('.multioperations-selector select[name="Post-processing"]').val()
@@ -400,7 +418,7 @@ function QueueListModel(parent) {
     self.doMultiDelete = function() {
         // Anything selected?
         if(self.multiEditItems().length < 1) return;
-        
+
         // Need confirm
         if(!self.parent.confirmDeleteQueue() || confirm(glitterTranslate.removeDown)) {
             // List all the ID's
@@ -440,7 +458,7 @@ function QueueListModel(parent) {
             $.each(self.multiEditItems(), function(index) {
                 $('#multiedit_' + this.id).prop('checked', true);
             })
-            
+
             // Update check-all buton state
             setCheckAllState('#multiedit-checkall', '.queue-table input[name="multiedit"]')
         }, 100)
@@ -460,33 +478,34 @@ function QueueModel(parent, data) {
     self.password = ko.observable(data.password);
     self.index = ko.observable(data.index);
     self.status = ko.observable(data.status);
-    self.isGrabbing = ko.observable(data.status == 'Grabbing')
+    self.isGrabbing = ko.observable(data.status == 'Grabbing' || data.avg_age == '-')
+    self.isFetchingBlocks = data.status == 'Fetching' || data.priority == 'Repair' // No need to update
     self.totalMB = ko.observable(parseFloat(data.mb));
-    self.remainingMB = ko.observable(parseFloat(data.mbleft));
+    self.remainingMB = ko.observable(parseFloat(data.mbleft))
+    self.missingMB = ko.observable(parseFloat(data.mbmissing))
+    self.percentage = ko.observable(parseInt(data.percentage))
     self.avg_age = ko.observable(data.avg_age)
-    self.missing = ko.observable(data.missing)
+    self.direct_unpack = ko.observable(data.direct_unpack)
     self.category = ko.observable(data.cat);
     self.priority = ko.observable(parent.priorityName[data.priority]);
     self.script = ko.observable(data.script);
     self.unpackopts = ko.observable(parseInt(data.unpackopts)) // UnpackOpts fails if not parseInt'd!
     self.pausedStatus = ko.observable(data.status == 'Paused');
     self.timeLeft = ko.observable(data.timeleft);
-    
+
     // Initially empty
     self.nameForEdit = ko.observable();
     self.editingName = ko.observable(false);
     self.hasDropdown = ko.observable(false);
-    self.rating_avg_video = ko.observable(false)
-    self.rating_avg_audio = ko.observable(false)
-    
+
     // Color of the progress bar
     self.progressColor = ko.computed(function() {
         // Checking
         if(self.status() == 'Checking') {
             return '#58A9FA'
         }
-        // Check for missing data, the value is arbitrary!
-        if(self.missing() > 50) {
+        // Check for missing data, the value is arbitrary! (2%)
+        if(self.missingMB()/self.totalMB() > 0.02) {
             return '#F8A34E'
         }
         // Set to grey, only when not Force download
@@ -496,23 +515,24 @@ function QueueModel(parent, data) {
         // Nothing
         return '';
     });
-    
-    // MB's and percentages
-    self.downloadedMB = ko.computed(function() {
-        return(self.totalMB() - self.remainingMB()).toFixed(0);
-    });
-    self.percentageRounded = ko.pureComputed(function() {
-        return fixPercentages(((self.downloadedMB() / self.totalMB()) * 100).toFixed(2))
-    })
+
+    // MB's
     self.progressText = ko.pureComputed(function() {
-        return self.downloadedMB() + " MB / " + (self.totalMB() * 1).toFixed(0) + " MB";
+        return (self.totalMB() - self.remainingMB()).toFixed(0) + " MB / " + (self.totalMB() * 1).toFixed(0) + " MB";
     })
-    
+
     // Texts
-    self.missingText= ko.pureComputed(function() {
-        // Check for missing data, the value is arbitrary!
-        if(self.missing() > 50) {
-            return self.missing() + ' ' + glitterTranslate.misingArt
+    self.name_title = ko.pureComputed(function() {
+        // When hovering over the job
+        if(self.direct_unpack()) {
+            return self.name() + ' - ' + glitterTranslate.status['DirectUnpack'] + ': ' + self.direct_unpack()
+        }
+        return self.name()
+    })
+    self.missingText = ko.pureComputed(function() {
+        // Check for missing data, the value is arbitrary! (1%)
+        if(self.missingMB()/self.totalMB() > 0.01) {
+            return self.missingMB().toFixed(0) + ' MB ' + glitterTranslate.misingArt
         }
         return;
     })
@@ -521,6 +541,10 @@ function QueueModel(parent, data) {
         if(self.status() == 'Checking') {
             return glitterTranslate.checking
         }
+        // Grabbing
+        if(self.status() == 'Grabbing') {
+            return glitterTranslate.fetch
+        }
         // Pausing status
         if((self.parent.parent.downloadsPaused() && self.priority() != 2) || self.pausedStatus()) {
             return glitterTranslate.paused;
@@ -528,15 +552,27 @@ function QueueModel(parent, data) {
         // Just the time
         return rewriteTime(self.timeLeft());
     });
-    
+
+    // Icon to better show force-priority
+    self.queueIcon = ko.computed(function() {
+        // Force comes first
+        if(self.priority() == 2) {
+            return 'glyphicon-forward'
+        }
+        if(self.pausedStatus()) {
+            return 'glyphicon-play'
+        }
+        return 'glyphicon-pause'
+    })
+
     // Extra queue column
     self.extraText = ko.pureComputed(function() {
         // Picked anything?
         switch(self.parent.parent.extraQueueColumn()) {
             case 'category':
                 // Exception for *
-                if(self.category() == "*") 
-                    return glitterTranslate.defaultText 
+                if(self.category() == "*")
+                    return glitterTranslate.defaultText
                 return self.category();
             case 'priority':
                 // Onload-exception
@@ -561,23 +597,19 @@ function QueueModel(parent, data) {
         self.password(data.password);
         self.index(data.index);
         self.status(data.status)
-        self.isGrabbing(data.status == 'Grabbing')
+        self.isGrabbing(data.status == 'Grabbing' || data.avg_age == '-')
         self.totalMB(parseFloat(data.mb));
         self.remainingMB(parseFloat(data.mbleft));
+        self.missingMB(parseFloat(data.mbmissing))
+        self.percentage(parseInt(data.percentage))
         self.avg_age(data.avg_age)
-        self.missing(data.missing)
+        self.direct_unpack(data.direct_unpack)
         self.category(data.cat);
         self.priority(parent.priorityName[data.priority]);
         self.script(data.script);
         self.unpackopts(parseInt(data.unpackopts)) // UnpackOpts fails if not parseInt'd!
         self.pausedStatus(data.status == 'Paused');
         self.timeLeft(data.timeleft);
-
-        // If exists, otherwise false
-        if(data.rating_avg_video !== undefined) {
-            self.rating_avg_video(data.rating_avg_video === 0 ? '-' : data.rating_avg_video);
-            self.rating_avg_audio(data.rating_avg_audio === 0 ? '-' : data.rating_avg_audio);
-        }
     };
 
     // Pause individual download
@@ -593,11 +625,11 @@ function QueueModel(parent, data) {
     self.editName = function(data, event) {
         // Not when still grabbing
         if(self.isGrabbing()) return false;
-        
+
         // Change status and fill
         self.editingName(true)
         self.nameForEdit(self.name())
-        
+
         // Select
         $(event.target).parents('.name').find('input').select()
     }
@@ -617,7 +649,7 @@ function QueueModel(parent, data) {
                 mode: 'queue',
                 name: 'rename',
                 value: self.id,
-                value2: newName 
+                value2: newName
             }).then(self.parent.parent.refresh)
     })
 
@@ -628,7 +660,7 @@ function QueueModel(parent, data) {
         // Trigger update
         parent.parent.filelist.loadFiles(self)
     }
-    
+
     // Toggle calculation of dropdown
     // Turns out that the <select> in the dropdown are a hugggeeee slowdown on initial load!
     // Only loading on click cuts half the speed (especially on large queues)
@@ -652,7 +684,7 @@ function QueueModel(parent, data) {
     }
     self.changeScript = function(item) {
         // Not on empty handlers
-        if(!item.script()) return;
+        if(!item.script() || parent.scriptsList().length <= 1) return;
         callAPI({
             mode: 'change_script',
             value: item.id,
@@ -668,7 +700,7 @@ function QueueModel(parent, data) {
     }
     self.changePriority = function(item, event) {
         // Not if we are fetching extra blocks for repair!
-        if(item.status() == 'Fetching') return
+        if(item.isFetchingBlocks) return
         callAPI({
             mode: 'queue',
             name: 'priority',
@@ -689,7 +721,7 @@ function QueueModel(parent, data) {
 
             // Show notification
             showNotification('.main-notification-box-removing')
-            
+
             callAPI({
                 mode: 'queue',
                 name: 'delete',
