@@ -37,7 +37,7 @@ ko.bindingHandlers.longText = {
         if(allBindingsAccessor.get('longTextType') != "Script") {
             value = value.map(convertHTMLtoText)
         }
-        
+
         // Any <br>'s?
         var outputText = '';
         if(value.length > 4) {
@@ -51,6 +51,10 @@ ko.bindingHandlers.longText = {
             // Nothing special
             outputText += value.join('<br />');
         }
+
+        // Replace link to our website
+        outputText = outputText.replace('https://sabnzbd.org/not-complete', '<a href="https://sabnzbd.org/not-complete" class="history-status-dmca" target="_blank">https://sabnzbd.org/not-complete</a>')
+
         ko.bindingHandlers.html.update(element, function() {
             return outputText;
         });
@@ -69,7 +73,7 @@ ko.bindingHandlers.filedrop = {
             return;
         }
         // EDITED to prevent drag-and-drop from inside own screen
-        $(element).bind("dragstart", function(e) {        
+        $(element).bind("dragstart", function(e) {
             $(element).data('internal-drag', true)
             // Remove after timeout
             setTimeout(function() {
@@ -110,7 +114,7 @@ $(document).bind('dragover', function(e) {
 
 /*! Knockout Persist - v0.1.0 - 2015-12-28
 * https://github.com/spoike/knockout.persist
-* Copyright (c) 2013 Mikael Brassman; Licensed MIT 
+* Copyright (c) 2013 Mikael Brassman; Licensed MIT
 * Safihre edited to better detect if localStorage is possible */
 (function(ko) {
     // Don't crash on browsers that are missing localStorage
