@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,9 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+# Imported to be referenced from other files directly
 from sabnzbd.version import __version__, __baseline__
-__configversion__ = 18
-__queueversion__ = 8
 
 import os
 import logging
@@ -25,8 +24,6 @@ import datetime
 import tempfile
 import cPickle
 import pickle
-import zipfile
-import glob
 import gzip
 import subprocess
 import time
@@ -34,7 +31,7 @@ import socket
 import cherrypy
 import sys
 import re
-from threading import RLock, Lock, Condition, Thread
+from threading import Lock, Thread
 try:
     import sleepless
 except ImportError:
@@ -43,11 +40,12 @@ except ImportError:
 ##############################################################################
 # Determine platform flags
 ##############################################################################
-WIN32 = DARWIN = DARWIN_INTEL = POSIX = FOUNDATION = WIN64 = False
+WIN32 = DARWIN = FOUNDATION = WIN64 = False
 KERNEL32 = None
 
 if os.name == 'nt':
     WIN32 = True
+    from util.apireg import del_connection_info
     try:
         import ctypes
         KERNEL32 = ctypes.windll.LoadLibrary("Kernel32.dll")
@@ -56,25 +54,37 @@ if os.name == 'nt':
 elif os.name == 'posix':
     ORG_UMASK = os.umask(18)
     os.umask(ORG_UMASK)
-    POSIX = True
     import platform
     if platform.system().lower() == 'darwin':
         DARWIN = True
+        # 12 = Sierra, 11 = ElCaptain, 10 = Yosemite, 9 = Mavericks, 8 = MountainLion
+        DARWIN_VERSION = int(platform.mac_ver()[0].split('.')[1])
         try:
             import Foundation
             FOUNDATION = True
         except:
             pass
-        if '86' in platform.machine():
-            DARWIN_INTEL = True
 
-if DARWIN:
-    # 10 = Yosemite, 9 = Mavericks, 8 = MountainLion, 7 = Lion, 6 = SnowLeopard, 5 = Leopard
-    DARWIN_VERSION = int(platform.mac_ver()[0].split('.')[1])
-    DARWIN_64 = platform.mac_ver()[2] == 'x86_64'
-else:
-    DARWIN_VERSION = 0
 
+##############################################################################
+# SSL CHECKS
+##############################################################################
+import ssl
+HAVE_SSL_CONTEXT = None
+try:
+    # Test availability of SSLContext (python 2.7.9+)
+    ssl.SSLContext
+    HAVE_SSL_CONTEXT = True
+except:
+    HAVE_SSL_CONTEXT = False
+
+try:
+    import cryptography
+    HAVE_CRYPTOGRAPHY = cryptography.__version__
+except:
+    HAVE_CRYPTOGRAPHY = False
+
+# Now we can import safely
 from sabnzbd.nzbqueue import NzbQueue
 from sabnzbd.postproc import PostProcessor
 from sabnzbd.downloader import Downloader
@@ -95,9 +105,13 @@ from sabnzbd.bpsmeter import BPSMeter
 import sabnzbd.cfg as cfg
 import sabnzbd.database
 import sabnzbd.lang as lang
+import sabnzbd.par2file as par2file
 import sabnzbd.api
-from sabnzbd.decorators import *
-from sabnzbd.constants import *
+import sabnzbd.directunpacker as directunpacker
+from sabnzbd.decorators import synchronized
+from sabnzbd.constants import NORMAL_PRIORITY, VALID_ARCHIVES, \
+    REPAIR_REQUEST, QUEUE_FILE_NAME, QUEUE_VERSION, QUEUE_FILE_TMPL
+import sabnzbd.getipaddress as getipaddress
 
 LINUX_POWER = powersup.HAVE_DBUS
 
@@ -105,7 +119,8 @@ START = datetime.datetime.now()
 
 MY_NAME = None
 MY_FULLNAME = None
-NEW_VERSION = None
+RESTART_ARGS = []
+NEW_VERSION = (None, None)
 DIR_HOME = None
 DIR_APPDATA = None
 DIR_LCLDATA = None
@@ -131,52 +146,42 @@ BROWSER_URL = None
 CMDLINE = ''  # Rendering of original command line arguments
 
 WEB_DIR = None
-WEB_DIR2 = None
-WEB_DIRC = None
+WEB_DIR_CONFIG = None
 WIZARD_DIR = None
 WEB_COLOR = None
-WEB_COLOR2 = None
 SABSTOP = False
 RESTART_REQ = False
 PAUSED_ALL = False
 OLD_QUEUE = False
-SCHED_RESTART = False  # Set when restarted through scheduler
+TRIGGER_RESTART = False  # To trigger restart for Scheduler, WinService and Mac
 WINTRAY = None  # Thread for the Windows SysTray icon
 WEBUI_READY = False
 LAST_WARNING = None
 LAST_ERROR = None
 EXTERNAL_IPV6 = False
+LAST_HISTORY_UPDATE = 1
+
+# Performance measure for dashboard
+PYSTONE_SCORE = 0
+DOWNLOAD_DIR_SPEED = 0
+COMPLETE_DIR_SPEED = 0
 
 __INITIALIZED__ = False
 __SHUTTING_DOWN__ = False
 
 
-##############################################################################
-# Table to map 0.5.x style language code to new style
-##############################################################################
-LANG_MAP = {
-    'de-de': 'de',
-    'dk-da': 'da',  # Should have been "da-dk"
-    'fr-fr': 'fr',
-    'nl-du': 'nl',  # Should have been "du-nl"
-    'no-no': 'nb',  # Norsk Bokmal
-    'sv-se': 'sv',
-    'us-en': 'en'  # Should have been "en-us"
-}
-
-
 ##############################################################################
 # Signal Handler
 ##############################################################################
 def sig_handler(signum=None, frame=None):
     global SABSTOP, WINTRAY
-    if sabnzbd.WIN32 and type(signum) != type(None) and DAEMON and signum == 5:
+    if sabnzbd.WIN32 and signum is not None and DAEMON and signum == 5:
         # Ignore the "logoff" event when running as a Win32 daemon
         return True
-    if type(signum) != type(None):
+    if signum is not None:
         logging.warning(T('Signal %s caught, saving and exiting...'), signum)
     try:
-        save_state(flag=True)
+        save_state()
         sabnzbd.zconfig.remove_server()
     finally:
         if sabnzbd.WIN32:
@@ -200,11 +205,10 @@ INIT_LOCK = Lock()
 def connect_db(thread_index=0):
     # Create a connection and store it in the current thread
     if not (hasattr(cherrypy.thread_data, 'history_db') and cherrypy.thread_data.history_db):
-        cherrypy.thread_data.history_db = sabnzbd.database.get_history_handle()
+        cherrypy.thread_data.history_db = sabnzbd.database.HistoryDB()
     return cherrypy.thread_data.history_db
 
 
-
 @synchronized(INIT_LOCK)
 def initialize(pause_downloader=False, clean_up=False, evalSched=False, repair=0):
     global __INITIALIZED__, __SHUTTING_DOWN__,\
@@ -250,9 +254,9 @@ def initialize(pause_downloader=False, clean_up=False, evalSched=False, repair=0
     cfg.cherryhost.callback(guard_restart)
     cfg.cherryport.callback(guard_restart)
     cfg.web_dir.callback(guard_restart)
-    cfg.web_dir2.callback(guard_restart)
     cfg.web_color.callback(guard_restart)
-    cfg.web_color2.callback(guard_restart)
+    cfg.username.callback(guard_restart)
+    cfg.password.callback(guard_restart)
     cfg.log_dir.callback(guard_restart)
     cfg.https_port.callback(guard_restart)
     cfg.https_cert.callback(guard_restart)
@@ -260,13 +264,13 @@ def initialize(pause_downloader=False, clean_up=False, evalSched=False, repair=0
     cfg.enable_https.callback(guard_restart)
     cfg.top_only.callback(guard_top_only)
     cfg.pause_on_post_processing.callback(guard_pause_on_pp)
-    cfg.growl_server.callback(sabnzbd.growler.change_value)
-    cfg.growl_password.callback(sabnzbd.growler.change_value)
+    cfg.growl_server.callback(sabnzbd.notifier.change_value)
+    cfg.growl_password.callback(sabnzbd.notifier.change_value)
     cfg.quota_size.callback(guard_quota_size)
     cfg.quota_day.callback(guard_quota_dp)
     cfg.quota_period.callback(guard_quota_dp)
     cfg.fsys_type.callback(guard_fsys_type)
-    cfg.language.callback(sabnzbd.growler.reset_growl)
+    cfg.language.callback(sabnzbd.notifier.reset_growl)
     cfg.enable_https_verification.callback(guard_https_ver)
     guard_https_ver()
 
@@ -274,16 +278,12 @@ def initialize(pause_downloader=False, clean_up=False, evalSched=False, repair=0
     sabnzbd.encoding.change_fsys(cfg.fsys_type())
 
     # Set cache limit
-    if sabnzbd.WIN32 or sabnzbd.DARWIN:
-        if cfg.cache_limit() == '' or cfg.cache_limit() == '200M':
-            cfg.cache_limit.set('450M')
+    if not cfg.cache_limit() or (cfg.cache_limit() in ('200M', '450M') and (sabnzbd.WIN32 or sabnzbd.DARWIN)):
+        cfg.cache_limit.set(misc.get_cache_limit())
     ArticleCache.do.new_limit(cfg.cache_limit.get_int())
 
     check_incomplete_vs_complete()
 
-    # Handle language upgrade from 0.5.x to 0.6.x
-    cfg.language.set(LANG_MAP.get(cfg.language(), cfg.language()))
-
     # Set language files
     lang.set_locale_info('SABnzbd', DIR_LANGUAGE)
     lang.set_language(cfg.language())
@@ -293,9 +293,6 @@ def initialize(pause_downloader=False, clean_up=False, evalSched=False, repair=0
 
     sabnzbd.change_queue_complete_action(cfg.queue_complete(), new=False)
 
-    sabnzbd.EXTERNAL_IPV6 = test_ipv6()
-    logging.debug('External IPv6 test result: %s', sabnzbd.EXTERNAL_IPV6)
-
     # One time conversion "speedlimit" in schedules.
     if not cfg.sched_converted():
         schedules = cfg.schedules()
@@ -306,35 +303,33 @@ def initialize(pause_downloader=False, clean_up=False, evalSched=False, repair=0
             else:
                 newsched.append(sched)
         cfg.schedules.set(newsched)
-        cfg.sched_converted.set(True)
+        cfg.sched_converted.set(1)
 
+    # Second time schedule conversion
+    if cfg.sched_converted() != 2:
+        cfg.schedules.set(['%s %s' % (1, schedule) for schedule in cfg.schedules()])
+        cfg.sched_converted.set(2)
+        config.save_config()
 
     if check_repair_request():
         repair = 2
         pause_downloader = True
-    else:
-        # Check crash detection file
-        # if load_admin(TERM_FLAG_FILE, remove=True):
-            # Repair mode 2 is a bit over an over-reaction!
-        pass  # repair = 2
-
-    # Set crash detection file
-    # save_admin(1, TERM_FLAG_FILE)
 
     # Initialize threads
     rss.init()
 
     paused = BPSMeter.do.read()
 
-    PostProcessor()
 
     NzbQueue()
 
+    Downloader(pause_downloader or paused)
+
     Assembler()
 
-    NzbQueue.do.read_queue(repair)
+    PostProcessor()
 
-    Downloader(pause_downloader or paused)
+    NzbQueue.do.read_queue(repair)
 
     DirScanner()
 
@@ -386,8 +381,14 @@ def halt():
         logging.info('SABnzbd shutting down...')
         __SHUTTING_DOWN__ = True
 
+        # Stop the windows tray icon
+        if sabnzbd.WINTRAY:
+            sabnzbd.WINTRAY.terminate = True
+
         sabnzbd.zconfig.remove_server()
 
+        sabnzbd.directunpacker.abort_all()
+
         rss.stop()
 
         logging.debug('Stopping URLGrabber')
@@ -431,7 +432,7 @@ def halt():
 
         # Save State
         try:
-            save_state(flag=True)
+            save_state()
         except:
             logging.error(T('Fatal error at saving state'), exc_info=True)
 
@@ -446,6 +447,33 @@ def halt():
         __INITIALIZED__ = False
 
 
+def trigger_restart(timeout=None):
+    """ Trigger a restart by setting a flag an shutting down CP """
+    # Sometimes we need to wait a bit to send good-bye to the browser
+    if timeout:
+        time.sleep(timeout)
+
+    # Add extra arguments
+    if sabnzbd.downloader.Downloader.do.paused:
+        sabnzbd.RESTART_ARGS.append('-p')
+    sys.argv = sabnzbd.RESTART_ARGS
+
+    # Stop all services
+    sabnzbd.halt()
+    cherrypy.engine.exit()
+
+    if sabnzbd.WIN32:
+        # Remove connection info for faster restart
+        del_connection_info()
+
+    # Leave the harder restarts to the polling in SABnzbd.py
+    if sabnzbd.WIN_SERVICE or getattr(sys, 'frozen', None) == 'macosx_app':
+        sabnzbd.TRIGGER_RESTART = True
+    else:
+        # Do the restart right now
+        cherrypy.engine._do_execv()
+
+
 ##############################################################################
 # Misc Wrappers
 ##############################################################################
@@ -488,6 +516,7 @@ def guard_fsys_type():
     """ Callback for change of file system naming type """
     sabnzbd.encoding.change_fsys(cfg.fsys_type())
 
+
 def set_https_verification(value):
     prev = False
     try:
@@ -512,20 +541,26 @@ def add_url(url, pp=None, script=None, cat=None, priority=None, nzbname=None):
     """ Add NZB based on a URL, attributes optional """
     if 'http' not in url:
         return
-    if pp and pp == "-1":
+    if not pp or pp == "-1":
         pp = None
     if script and script.lower() == 'default':
         script = None
     if cat and cat.lower() == 'default':
         cat = None
     logging.info('Fetching %s', url)
+
+    # Add feed name if it came from RSS
     msg = T('Trying to fetch NZB from %s') % url
+    if nzbname:
+        msg = '%s - %s' % (nzbname, msg)
+
+    # Generate the placeholder
     future_nzo = NzbQueue.do.generate_future(msg, pp, script, cat, url=url, priority=priority, nzbname=nzbname)
     URLGrabber.do.add(url, future_nzo)
     return future_nzo.nzo_id
 
 
-def save_state(flag=False):
+def save_state():
     """ Save all internal bookkeeping to disk """
     ArticleCache.do.flush_articles()
     NzbQueue.do.save()
@@ -534,9 +569,6 @@ def save_state(flag=False):
     Rating.do.save()
     DirScanner.do.save()
     PostProcessor.do.save()
-    # if flag:
-    #    # Remove crash detector
-    #    load_admin(TERM_FLAG_FILE, remove=True)
 
 
 def pause_all():
@@ -556,12 +588,9 @@ def unpause_all():
 
 
 ##############################################################################
-# NZB_LOCK Methods
+# NZB Saving Methods
 ##############################################################################
-NZB_LOCK = Lock()
 
-
-@synchronized(NZB_LOCK)
 def backup_exists(filename):
     """ Return True if backup exists and no_dupes is set """
     path = cfg.nzb_backup_dir.get_path()
@@ -575,35 +604,29 @@ def backup_nzb(filename, data):
         save_compressed(path, filename, data)
 
 
-@synchronized(NZB_LOCK)
 def save_compressed(folder, filename, data):
     """ Save compressed NZB file in folder """
-    # Need to go to the save folder to
-    # prevent the pathname being embedded in the GZ file
-    here = os.getcwd()
-    os.chdir(misc.short_path(folder))
-
     if filename.endswith('.nzb'):
         filename += '.gz'
     else:
         filename += '.nzb.gz'
     logging.info("Backing up %s", os.path.join(folder, filename))
     try:
-        f = gzip.GzipFile(filename, 'wb')
-        f.write(data)
-        f.flush()
-        f.close()
+        # Have to get around the path being put inside the tgz
+        with open(os.path.join(folder, filename), 'wb') as tgz_file:
+            f = gzip.GzipFile(filename, fileobj=tgz_file)
+            f.write(data)
+            f.flush()
+            f.close()
     except:
         logging.error(T('Saving %s failed'), os.path.join(folder, filename))
         logging.info("Traceback: ", exc_info=True)
 
-    os.chdir(here)
-
 
 ##############################################################################
-# CV synchronized (notifies downloader)
+# Unsynchronized methods
 ##############################################################################
-@synchronized_CV
+
 def add_nzbfile(nzbfile, pp=None, script=None, cat=None, priority=NORMAL_PRIORITY, nzbname=None, reuse=False, password=None):
     """ Add disk-based NZB file, optional attributes,
         'reuse' flag will suppress duplicate detection
@@ -673,9 +696,6 @@ def add_nzbfile(nzbfile, pp=None, script=None, cat=None, priority=NORMAL_PRIORIT
                                  keep=keep, reuse=reuse, password=password)
 
 
-##############################################################################
-# Unsynchronized methods
-##############################################################################
 def enable_server(server):
     """ Enable server (scheduler only) """
     try:
@@ -739,19 +759,17 @@ def system_standby():
 def shutdown_program():
     """ Stop program after halting and saving """
     logging.info("Performing sabnzbd shutdown")
-    Thread(target=halt).start()
-    while __INITIALIZED__:
-        time.sleep(1.0)
-    os._exit(0)
+    sabnzbd.halt()
+    cherrypy.engine.exit()
+    sabnzbd.SABSTOP = True
 
 
 def restart_program():
     """ Restart program (used by scheduler) """
-    global SCHED_RESTART
     logging.info("Scheduled restart request")
     # Just set the stop flag, because stopping CherryPy from
     # the scheduler is not reliable
-    cherrypy.engine.execv = SCHED_RESTART = True
+    sabnzbd.TRIGGER_RESTART = True
 
 
 def change_queue_complete_action(action, new=True):
@@ -787,7 +805,6 @@ def change_queue_complete_action(action, new=True):
 
     # keep the name of the action for matching the current select in queue.tmpl
     QUEUECOMPLETE = action
-
     QUEUECOMPLETEACTION = _action
     QUEUECOMPLETEARG = _argument
 
@@ -795,11 +812,15 @@ def change_queue_complete_action(action, new=True):
 def run_script(script):
     """ Run a user script (queue complete only) """
     command = [os.path.join(cfg.script_dir.get_path(), script)]
-    stup, need_shell, command, creationflags = sabnzbd.newsunpack.build_command(command)
-    logging.info('Spawning external command %s', command)
-    subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
-                     stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                     startupinfo=stup, creationflags=creationflags)
+    if os.path.exists(command[0]):
+        try:
+            stup, need_shell, command, creationflags = sabnzbd.newsunpack.build_command(command)
+            logging.info('Spawning external command %s', command)
+            subprocess.Popen(command, shell=need_shell, stdin=subprocess.PIPE,
+                             stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                             startupinfo=stup, creationflags=creationflags)
+        except:
+            logging.debug("Failed script %s, Traceback: ", script, exc_info=True)
 
 
 def empty_queues():
@@ -825,22 +846,10 @@ def keep_awake():
                 sleepless.allow_sleep()
 
 
-def CheckFreeSpace():
-    """ Check if enough disk space is free, if not pause downloader and send email """
-    if cfg.download_free() and not sabnzbd.downloader.Downloader.do.paused:
-        if misc.diskfree(cfg.download_dir.get_path()) < cfg.download_free.get_float() / GIGI:
-            logging.warning(T('Too little diskspace forcing PAUSE'))
-            # Pause downloader, but don't save, since the disk is almost full!
-            Downloader.do.pause(save=False)
-            emailer.diskfull()
-
-
 ################################################################################
 # Data IO                                                                      #
 ################################################################################
-IO_LOCK = RLock()
 
-@synchronized(IO_LOCK)
 def get_new_id(prefix, folder, check_list=None):
     """ Return unique prefixed admin identifier within folder
         optionally making sure that id is not in the check_list.
@@ -857,63 +866,64 @@ def get_new_id(prefix, folder, check_list=None):
         except:
             logging.error(T('Failure in tempfile.mkstemp'))
             logging.info("Traceback: ", exc_info=True)
+            break
     # Cannot create unique id, crash the process
     raise IOError
 
 
-@synchronized(IO_LOCK)
 def save_data(data, _id, path, do_pickle=True, silent=False):
     """ Save data to a diskfile """
     if not silent:
-        logging.debug("Saving data for %s in %s", _id, path)
+        logging.debug('[%s] Saving data for %s in %s', misc.caller_name(), _id, path)
     path = os.path.join(path, _id)
 
-    try:
-        _f = open(path, 'wb')
-        if do_pickle:
-            if cfg.use_pickle():
-                pickler = pickle.Pickler(_f, 2)
+    # We try 3 times, to avoid any dict or access problems
+    for t in xrange(3):
+        try:
+            with open(path, 'wb') as data_file:
+                if do_pickle:
+                    if cfg.use_pickle():
+                        pickle.dump(data, data_file)
+                    else:
+                        cPickle.dump(data, data_file)
+                else:
+                    data_file.write(data)
+            break
+        except:
+            if silent:
+                # This can happen, probably a removed folder
+                pass
+            elif t == 2:
+                logging.error(T('Saving %s failed'), path)
+                logging.info("Traceback: ", exc_info=True)
             else:
-                pickler = cPickle.Pickler(_f, 2)
-            pickler.dump(data)
-            _f.flush()
-            _f.close()
-            pickler.clear_memo()
-            del pickler
-        else:
-            _f.write(data)
-            _f.flush()
-            _f.close()
-    except:
-        logging.error(T('Saving %s failed'), path)
-        logging.info("Traceback: ", exc_info=True)
+                # Wait a tiny bit before trying again
+                time.sleep(0.1)
 
 
-@synchronized(IO_LOCK)
 def load_data(_id, path, remove=True, do_pickle=True, silent=False):
     """ Read data from disk file """
     path = os.path.join(path, _id)
 
     if not os.path.exists(path):
-        logging.info("%s missing", path)
+        logging.info("[%s] %s missing", misc.caller_name(), path)
         return None
 
     if not silent:
-        logging.debug("Loading data for %s from %s", _id, path)
+        logging.debug("[%s] Loading data for %s from %s", misc.caller_name(), _id, path)
 
     try:
-        _f = open(path, 'rb')
-        if do_pickle:
-            if cfg.use_pickle():
-                data = pickle.load(_f)
+        with open(path, 'rb') as data_file:
+            if do_pickle:
+                if cfg.use_pickle():
+                    data = pickle.load(data_file)
+                else:
+                    data = cPickle.load(data_file)
             else:
-                data = cPickle.load(_f)
-        else:
-            data = _f.read()
-        _f.close()
+                data = data_file.read()
 
         if remove:
-            os.remove(path)
+            misc.remove_file(path)
     except:
         logging.error(T('Loading %s failed'), path)
         logging.info("Traceback: ", exc_info=True)
@@ -922,62 +932,56 @@ def load_data(_id, path, remove=True, do_pickle=True, silent=False):
     return data
 
 
-@synchronized(IO_LOCK)
 def remove_data(_id, path):
     """ Remove admin file """
     path = os.path.join(path, _id)
     try:
         if os.path.exists(path):
-            os.remove(path)
-            logging.info("%s removed", path)
+            misc.remove_file(path)
     except:
         logging.debug("Failed to remove %s", path)
 
 
-@synchronized(IO_LOCK)
-def save_admin(data, _id, do_pickle=True):
+def save_admin(data, _id):
     """ Save data in admin folder in specified format """
     path = os.path.join(cfg.admin_dir.get_path(), _id)
-    logging.info("Saving data for %s in %s", _id, path)
+    logging.debug("[%s] Saving data for %s in %s", misc.caller_name(), _id, path)
 
-    try:
-        _f = open(path, 'wb')
-        if do_pickle:
-            pickler = cPickle.Pickler(_f, 2)
-            pickler.dump(data)
-            _f.flush()
-            _f.close()
-            pickler.clear_memo()
-            del pickler
-        else:
-            _f.write(data)
-            _f.flush()
-            _f.close()
-    except:
-        logging.error(T('Saving %s failed'), path)
-        logging.info("Traceback: ", exc_info=True)
+    # We try 3 times, to avoid any dict or access problems
+    for t in xrange(3):
+        try:
+            with open(path, 'wb') as data_file:
+                if cfg.use_pickle():
+                    data = pickle.dump(data, data_file)
+                else:
+                    data = cPickle.dump(data, data_file)
+            break
+        except:
+            if t == 2:
+                logging.error(T('Saving %s failed'), path)
+                logging.info("Traceback: ", exc_info=True)
+            else:
+                # Wait a tiny bit before trying again
+                time.sleep(0.1)
 
 
-@synchronized(IO_LOCK)
-def load_admin(_id, remove=False, do_pickle=True, silent=False):
+def load_admin(_id, remove=False, silent=False):
     """ Read data in admin folder in specified format """
     path = os.path.join(cfg.admin_dir.get_path(), _id)
-    logging.info("Loading data for %s from %s", _id, path)
+    logging.debug("[%s] Loading data for %s from %s", misc.caller_name(), _id, path)
 
     if not os.path.exists(path):
-        logging.info("%s missing", path)
+        logging.info("[%s] %s missing", misc.caller_name(), path)
         return None
 
     try:
-        f = open(path, 'rb')
-        if do_pickle:
-            data = cPickle.load(f)
-        else:
-            data = f.read()
-        f.close()
-
+        with open(path, 'rb') as data_file:
+            if cfg.use_pickle():
+                data = pickle.load(data_file)
+            else:
+                data = cPickle.load(data_file)
         if remove:
-            os.remove(path)
+            misc.remove_file(path)
     except:
         if not silent:
             excepterror = str(sys.exc_info()[0])
@@ -1031,18 +1035,13 @@ def check_repair_request():
     path = os.path.join(cfg.admin_dir.get_path(), REPAIR_REQUEST)
     if os.path.exists(path):
         try:
-            os.remove(path)
+            misc.remove_file(path)
         except:
             pass
         return True
     return False
 
 
-def SimpleRarExtract(rarfile, fn):
-    """ Wrapper for call to newsunpack, required to avoid circular imports """
-    return sabnzbd.newsunpack.SimpleRarExtract(rarfile, fn)
-
-
 def check_all_tasks():
     """ Check every task and restart safe ones, else restart program
         Return True when everything is under control
@@ -1107,7 +1106,7 @@ def pid_file(pid_path=None, pid_file=None, port=0):
                 f.write('%d\n' % os.getpid())
                 f.close()
             else:
-                os.remove(DIR_PID)
+                misc.remove_file(DIR_PID)
         except:
             logging.warning('Cannot access PID file %s', DIR_PID)
 
@@ -1129,11 +1128,12 @@ def wait_for_download_folder():
         logging.debug('Waiting for "incomplete" folder')
         time.sleep(2.0)
 
+
 def check_old_queue():
     """ Check for old queue (when a new queue is not present) """
     old = False
     if not os.path.exists(os.path.join(cfg.admin_dir.get_path(), QUEUE_FILE_NAME)):
-        for ver in (QUEUE_VERSION -1 , QUEUE_VERSION - 2, QUEUE_VERSION - 3):
+        for ver in (QUEUE_VERSION - 1, QUEUE_VERSION - 2, QUEUE_VERSION - 3):
             data = load_admin(QUEUE_FILE_TMPL % str(ver))
             if data:
                 break
@@ -1141,8 +1141,7 @@ def check_old_queue():
             old = bool(data and isinstance(data, tuple) and len(data[1]))
         except (TypeError, IndexError):
             pass
-        if old and sabnzbd.WIN32 and ver < 10 and sabnzbd.DIR_LCLDATA != sabnzbd.DIR_HOME \
-            and misc.is_relative_path(cfg.download_dir()):
+        if old and sabnzbd.WIN32 and ver < 10 and sabnzbd.DIR_LCLDATA != sabnzbd.DIR_HOME and misc.is_relative_path(cfg.download_dir()):
             # For Windows and when version < 10: adjust old default location
             cfg.download_dir.set('Documents/' + cfg.download_dir())
     return old
@@ -1153,17 +1152,13 @@ def highest_server(me):
     return sabnzbd.downloader.Downloader.do.highest_server(me)
 
 
-def proxy_pre_queue(name, pp, cat, script, priority, size, groups):
-    return sabnzbd.newsunpack.pre_queue(name, pp, cat, script, priority, size, groups)
-
-
 def test_ipv6():
     """ Check if external IPv6 addresses are reachable """
     if not cfg.selftest_host():
         # User disabled the test, assume active IPv6
         return True
     try:
-        info = socket.getaddrinfo(cfg.selftest_host(), 443, socket.AF_INET6, socket.SOCK_STREAM, socket.IPPROTO_IP, socket.AI_CANONNAME)
+        info = getipaddress.addresslookup6(cfg.selftest_host())
     except:
         logging.debug("Test IPv6: Disabling IPv6, because it looks like it's not available. Reason: %s", sys.exc_info()[0] )
         return False
@@ -1182,3 +1177,11 @@ def test_ipv6():
     except:
         logging.debug('Test IPv6: Problem during IPv6 connect. Disabling IPv6. Reason: %s', sys.exc_info()[0])
         return False
+
+
+def history_updated():
+    """ To make sure we always have a fresh history """
+    sabnzbd.LAST_HISTORY_UPDATE += 1
+    # Never go over the limit
+    if sabnzbd.LAST_HISTORY_UPDATE+1 >= sys.maxint:
+        sabnzbd.LAST_HISTORY_UPDATE = 1
