@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,7 +27,8 @@ import time
 import json
 import cherrypy
 import locale
-import socket
+
+from threading import Thread
 try:
     locale.setlocale(locale.LC_ALL, "")
 except:
@@ -39,24 +40,24 @@ try:
 except ImportError:
     pass
 
-
 import sabnzbd
 from sabnzbd.constants import VALID_ARCHIVES, Status, \
-     TOP_PRIORITY, REPAIR_PRIORITY, HIGH_PRIORITY, HIGH_PRIORITY, NORMAL_PRIORITY, LOW_PRIORITY, \
+     TOP_PRIORITY, REPAIR_PRIORITY, HIGH_PRIORITY, NORMAL_PRIORITY, LOW_PRIORITY, \
      KIBI, MEBI, GIGI, JOB_ADMIN
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.downloader import Downloader
-from sabnzbd.nzbqueue import NzbQueue, set_priority, sort_queue, scan_jobs, repair_job
+from sabnzbd.nzbqueue import NzbQueue
 import sabnzbd.scheduler as scheduler
 from sabnzbd.skintext import SKIN_TEXT
 from sabnzbd.utils.json import JsonWriter
 
 from sabnzbd.utils.rsslib import RSS, Item
 from sabnzbd.utils.pathbrowser import folders_at_path
-from sabnzbd.misc import loadavg, to_units, diskfree, disktotal, get_ext, \
+from sabnzbd.utils.getperformance import getcpu
+from sabnzbd.misc import loadavg, to_units, diskspace, get_ext, \
     get_filename, int_conv, globber, globber_full, time_format, remove_all, \
-    starts_with_path, cat_convert, clip_path, create_https_certificates
+    starts_with_path, cat_convert, clip_path, create_https_certificates, calc_age
 from sabnzbd.encoding import xml_name, unicoder, special_fixer, platform_encode, html_escape
 from sabnzbd.postproc import PostProcessor
 from sabnzbd.articlecache import ArticleCache
@@ -64,6 +65,7 @@ from sabnzbd.utils.servertests import test_nntp_server_dict
 from sabnzbd.bpsmeter import BPSMeter
 from sabnzbd.rating import Rating
 from sabnzbd.getipaddress import localipv4, publicipv4, ipv6
+from sabnzbd.newsunpack import userxbit
 from sabnzbd.database import build_history_info, unpack_history_info, HistoryDB
 import sabnzbd.notifier
 import sabnzbd.rss
@@ -95,6 +97,7 @@ else:
 # Flag for using the fast json encoder, unless it fails
 FAST_JSON = True
 
+
 def api_handler(kwargs):
     """ API Dispatcher """
     mode = kwargs.get('mode', '')
@@ -102,9 +105,6 @@ def api_handler(kwargs):
     name = kwargs.get('name', '')
     callback = kwargs.get('callback', '')
 
-    # Extend the timeout of API calls to 10minutes
-    cherrypy.response.timeout = 60*10
-
     if isinstance(mode, list):
         mode = mode[0]
     if isinstance(output, list):
@@ -137,6 +137,7 @@ def _api_set_config(name, output, kwargs):
     res, data = config.get_dconfig(kwargs.get('section'), kwargs.get('keyword'))
     return report(output, keyword='config', data=data)
 
+
 def _api_set_config_default(name, output, kwargs):
     """ API: Reset requested config variables back to defaults. Currently only for misc-section """
     keywords = kwargs.get('keyword', [])
@@ -160,13 +161,8 @@ def _api_del_config(name, output, kwargs):
 
 def _api_qstatus(name, output, kwargs):
     """ API: accepts output """
-    if output == 'json':
-        # Compatibility Fix:
-        # Old qstatus did not have a keyword, so do not use one now.
-        keyword = ''
-    else:
-        keyword = 'queue'
-    return report(output, keyword=keyword, data=qstatus_data())
+    info, pnfo_list, bytespersec = build_queue()
+    return report(output, data=remove_callable(info))
 
 
 def _api_queue(name, output, kwargs):
@@ -193,7 +189,7 @@ def _api_queue_delete_nzf(output, value, kwargs):
     """ API: accepts value(=nzo_id), value2(=nzf_id) """
     value2 = kwargs.get('value2')
     if value and value2:
-        removed = NzbQueue.do.remove_nzf(value, value2)
+        removed = NzbQueue.do.remove_nzf(value, value2, force_delete=True)
         return report(output, keyword='', data={'status': bool(removed), 'nzf_ids': removed})
     else:
         return report(output, _MSG_NO_VALUE2)
@@ -247,7 +243,7 @@ def _api_queue_priority(output, value, kwargs):
                 priority = int(value2)
             except:
                 return report(output, _MSG_INT_VALUE)
-            pos = set_priority(value, priority)
+            pos = NzbQueue.do.set_priority(value, priority)
             # Returns the position in the queue, -1 is incorrect job-id
             return report(output, keyword='position', data=pos)
         except:
@@ -261,7 +257,7 @@ def _api_queue_sort(output, value, kwargs):
     sort = kwargs.get('sort')
     direction = kwargs.get('dir', '')
     if sort:
-        sort_queue(sort, direction)
+        NzbQueue.do.sort_queue(sort, direction)
         return report(output)
     else:
         return report(output, _MSG_NO_VALUE2)
@@ -269,21 +265,12 @@ def _api_queue_sort(output, value, kwargs):
 
 def _api_queue_default(output, value, kwargs):
     """ API: accepts output, sort, dir, start, limit """
-    sort = kwargs.get('sort')
-    direction = kwargs.get('dir', '')
     start = int_conv(kwargs.get('start'))
     limit = int_conv(kwargs.get('limit'))
-    trans = kwargs.get('trans')
     search = kwargs.get('search')
 
     if output in ('xml', 'json'):
-        if sort and sort != 'index':
-            reverse = direction.lower() == 'desc'
-            sort_queue(sort, reverse)
-
-        info, pnfo_list, bytespersec = build_queue(start=start, limit=limit, output=output, trans=trans, search=search)
-        info['categories'] = info.pop('cat_list')
-        info['scripts'] = info.pop('script_list')
+        info, pnfo_list, bytespersec = build_queue(start=start, limit=limit, output=output, search=search)
         return report(output, keyword='queue', data=remove_callable(info))
     elif output == 'rss':
         return rss_qstatus()
@@ -384,6 +371,15 @@ def _api_retry(name, output, kwargs):
         return report(output, _MSG_NO_ITEM)
 
 
+def _api_cancel_pp(name, output, kwargs):
+    """ API: accepts name, output, value(=nzo_id) """
+    nzo_id = kwargs.get('value')
+    if PostProcessor.do.cancel_pp(nzo_id):
+        return report(output, keyword='', data={'status': True, 'nzo_id': nzo_id})
+    else:
+        return report(output, _MSG_NO_ITEM)
+
+
 def _api_addlocalfile(name, output, kwargs):
     """ API: accepts name, output, pp, script, cat, priority, nzbname """
     if name and isinstance(name, list):
@@ -476,7 +472,7 @@ def _api_change_opts(name, output, kwargs):
 
 def _api_fullstatus(name, output, kwargs):
     """ API: full history status"""
-    status = build_status(skip_dashboard=kwargs.get('skip_dashboard'), output=output)
+    status = build_status(skip_dashboard=kwargs.get('skip_dashboard', 1), output=output)
     return report(output, keyword='status', data=remove_callable(status))
 
 
@@ -485,13 +481,14 @@ def _api_history(name, output, kwargs):
     value = kwargs.get('value', '')
     start = int_conv(kwargs.get('start'))
     limit = int_conv(kwargs.get('limit'))
+    last_history_update = int_conv(kwargs.get('last_history_update', 0))
     search = kwargs.get('search')
     failed_only = kwargs.get('failed_only')
     categories = kwargs.get('category')
-    last_history_update = kwargs.get('last_history_update', 0)
+
 
     # Do we need to send anything?
-    if int(last_history_update) == int(sabnzbd.LAST_HISTORY_UPDATE):
+    if last_history_update == sabnzbd.LAST_HISTORY_UPDATE:
         return report(output, keyword='history', data=False)
 
     if categories and not isinstance(categories, list):
@@ -511,22 +508,18 @@ def _api_history(name, output, kwargs):
                 history_db.remove_failed(search)
             if special in ('all', 'completed'):
                 history_db.remove_completed(search)
-            # Update the last check time
-            sabnzbd.LAST_HISTORY_UPDATE = time.time()
+            sabnzbd.history_updated()
             return report(output)
         elif value:
             jobs = value.split(',')
             for job in jobs:
                 del_hist_job(job, del_files)
-            # Update the last check time
-            sabnzbd.LAST_HISTORY_UPDATE = time.time()
+            sabnzbd.history_updated()
             return report(output)
         else:
             return report(output, _MSG_NO_VALUE)
     elif not name:
-        history = build_header(prim=True)
-        if 'noofslots_total' in history:
-            del history['noofslots_total']
+        history = {}
         grand, month, week, day = BPSMeter.do.get_sums()
         history['total_size'], history['month_size'], history['week_size'], history['day_size'] = \
                to_units(grand), to_units(month), to_units(week), to_units(day)
@@ -535,7 +528,8 @@ def _api_history(name, output, kwargs):
                                                                               search=search, failed_only=failed_only,
                                                                               categories=categories,
                                                                               output=output)
-        history['last_history_update'] = int(sabnzbd.LAST_HISTORY_UPDATE)
+        history['last_history_update'] = sabnzbd.LAST_HISTORY_UPDATE
+        history['version'] = sabnzbd.__version__
         return report(output, keyword='history', data=remove_callable(history))
     else:
         return report(output, _MSG_NOT_IMPLEMENTED)
@@ -598,6 +592,7 @@ def _api_resume(name, output, kwargs):
 
 def _api_shutdown(name, output, kwargs):
     """ API: accepts output """
+    logging.info('Shutdown requested by API')
     sabnzbd.halt()
     cherrypy.engine.exit()
     sabnzbd.SABSTOP = True
@@ -651,16 +646,17 @@ def _api_auth(name, output, kwargs):
 
 def _api_restart(name, output, kwargs):
     """ API: accepts output """
-    sabnzbd.halt()
-    cherrypy.engine.restart()
+    logging.info('Restart requested by API')
+    # Do the shutdown async to still send goodbye to browser
+    Thread(target=sabnzbd.trigger_restart, kwargs={'timeout': 1}).start()
     return report(output)
 
 
 def _api_restart_repair(name, output, kwargs):
     """ API: accepts output """
+    logging.info('Queue repair requested by API')
     sabnzbd.request_repair()
-    sabnzbd.halt()
-    cherrypy.engine.restart()
+    sabnzbd.trigger_restart()
     return report(output)
 
 
@@ -679,18 +675,17 @@ def _api_osx_icon(name, output, kwargs):
 
 def _api_rescan(name, output, kwargs):
     """ API: accepts output """
-    scan_jobs(all=False, action=True)
+    NzbQueue.do.scan_jobs(all=False, action=True)
     return report(output)
 
 
 def _api_eval_sort(name, output, kwargs):
     """ API: evaluate sorting expression """
-    import sabnzbd.tvsort
     name = kwargs.get('name', '')
     value = kwargs.get('value', '')
     title = kwargs.get('title')
     multipart = kwargs.get('movieextra', '')
-    path = sabnzbd.tvsort.eval_sort(value, title, name, multipart)
+    path = sabnzbd.sorting.eval_sort(value, title, name, multipart)
     if path is None:
         return report(output, _MSG_NOT_IMPLEMENTED)
     else:
@@ -746,12 +741,14 @@ def _api_test_email(name, output, kwargs):
         res = None
     return report(output, error=res)
 
+
 def _api_test_windows(name, output, kwargs):
     """ API: send a test to Windows, return result """
     logging.info("Sending test notification")
     res = sabnzbd.notifier.send_windows('SABnzbd', T('Test Notification'), 'other')
     return report(output, error=res)
 
+
 def _api_test_notif(name, output, kwargs):
     """ API: send a test to Notification Center, return result """
     logging.info("Sending test notification")
@@ -793,12 +790,14 @@ def _api_test_pushbullet(name, output, kwargs):
     res = sabnzbd.notifier.send_pushbullet('SABnzbd', T('Test Notification'), 'other', force=True, test=kwargs)
     return report(output, error=res)
 
+
 def _api_test_nscript(name, output, kwargs):
     """ API: execute a test notification script, return result """
     logging.info("Executing notification script")
     res = sabnzbd.notifier.send_nscript('SABnzbd', T('Test Notification'), 'other', force=True, test=kwargs)
     return report(output, error=res)
 
+
 def _api_undefined(name, output, kwargs):
     """ API: accepts output """
     return report(output, _MSG_NOT_IMPLEMENTED)
@@ -840,14 +839,10 @@ def _api_config_get_speedlimit(output, kwargs):
 
 
 def _api_config_set_colorscheme(output, kwargs):
-    """ API: accepts output, value(=color for primary), value2(=color for secondary) """
+    """ API: accepts output"""
     value = kwargs.get('value')
-    value2 = kwargs.get('value2')
     if value:
         cfg.web_color.set(value)
-    if value2:
-        cfg.web_color2.set(value2)
-    if value or value2:
         return report(output)
     else:
         return report(output, _MSG_NO_VALUE)
@@ -907,8 +902,8 @@ def _api_server_stats(name, output, kwargs):
 
     stats['servers'] = {}
     for svr in config.get_servers():
-        t, m, w, d = BPSMeter.do.amounts(svr)
-        stats['servers'][svr] = {'total': t or 0, 'month': m or 0, 'week': w or 0, 'day': d or 0}
+        t, m, w, d, daily = BPSMeter.do.amounts(svr)
+        stats['servers'][svr] = {'total': t or 0, 'month': m or 0, 'week': w or 0, 'day': d or 0, 'daily': daily or {} }
 
     return report(output, keyword='', data=stats)
 
@@ -926,6 +921,7 @@ _api_table = {
     'translate': (_api_translate, 2),
     'addfile': (_api_addfile, 1),
     'retry': (_api_retry, 2),
+    'cancel_pp': (_api_cancel_pp, 2),
     'addlocalfile': (_api_addlocalfile, 1),
     'switch': (_api_switch, 2),
     'change_cat': (_api_change_cat, 2),
@@ -1181,17 +1177,29 @@ def handle_cat_api(output, kwargs):
     return name
 
 
-def build_status(web_dir=None, root=None, prim=True, skip_dashboard=False, output=None):
+def build_status(skip_dashboard=False, output=None):
     # build up header full of basic information
-    info = build_header(prim, web_dir)
+    info = build_header()
 
     info['logfile'] = sabnzbd.LOGFILE
     info['weblogfile'] = sabnzbd.WEBLOGFILE
     info['loglevel'] = str(cfg.log_level())
-    info['folders'] = [xml_name(item) for item in sabnzbd.nzbqueue.scan_jobs(all=False, action=False)]
+    info['folders'] = [xml_name(item) for item in NzbQueue.do.scan_jobs(all=False, action=False)]
     info['configfn'] = xml_name(config.get_filename())
 
-    # Dashboard: Begin
+    # Dashboard: Speed of System
+    info['cpumodel'] = getcpu()
+    info['pystone'] = sabnzbd.PYSTONE_SCORE
+
+    # Dashboard: Speed of Download directory:
+    info['downloaddir'] = sabnzbd.cfg.download_dir.get_path()
+    info['downloaddirspeed'] = sabnzbd.DOWNLOAD_DIR_SPEED
+
+    # Dashboard: Speed of Complete directory:
+    info['completedir'] = sabnzbd.cfg.complete_dir.get_path()
+    info['completedirspeed'] = sabnzbd.COMPLETE_DIR_SPEED
+
+    # Dashboard: Connection information
     if not int_conv(skip_dashboard):
         info['localipv4'] = localipv4()
         info['publicipv4'] = publicipv4()
@@ -1203,33 +1211,6 @@ def build_status(web_dir=None, root=None, prim=True, skip_dashboard=False, outpu
         except:
             info['dnslookup'] = None
 
-        # Dashboard: Speed of System
-        from sabnzbd.utils.getperformance import getpystone, getcpu
-        info['pystone'] = getpystone()
-        info['cpumodel'] = getcpu()
-        # Dashboard: Speed of Download directory:
-        info['downloaddir'] = sabnzbd.cfg.download_dir.get_path()
-        try:
-            sabnzbd.downloaddirspeed  # The persistent var
-        except:
-            # does not yet exist, so create it:
-            sabnzbd.downloaddirspeed = 0  # 0 means ... not yet determined
-        info['downloaddirspeed'] = sabnzbd.downloaddirspeed
-        # Dashboard: Speed of Complete directory:
-        info['completedir'] = sabnzbd.cfg.complete_dir.get_path()
-        try:
-            sabnzbd.completedirspeed  # The persistent var
-        except:
-            # does not yet exist, so create it:
-            sabnzbd.completedirspeed = 0  # 0 means ... not yet determined
-        info['completedirspeed'] = sabnzbd.completedirspeed
-
-        try:
-            sabnzbd.dashrefreshcounter  # The persistent var @UndefinedVariable
-        except:
-            sabnzbd.dashrefreshcounter = 0
-        info['dashrefreshcounter'] = sabnzbd.dashrefreshcounter
-
     info['servers'] = []
     servers = sorted(Downloader.do.servers[:], key=lambda svr: '%02d%s' % (svr.priority, svr.displayname.lower()))
     for server in servers:
@@ -1285,6 +1266,7 @@ def build_status(web_dir=None, root=None, prim=True, skip_dashboard=False, outpu
                             'servertotalconn': server.threads,
                             'serverconnections': serverconnections,
                             'serverssl': server.ssl,
+                            'serversslinfo': server.ssl_info,
                             'serveractive': server.active,
                             'servererror': server.errormsg,
                             'serverpriority': server.priority,
@@ -1294,22 +1276,19 @@ def build_status(web_dir=None, root=None, prim=True, skip_dashboard=False, outpu
             info['servers'].append((server.displayname, '', connected, serverconnections, server.ssl,
                                       server.active, server.errormsg, server.priority, server.optional))
 
-    wlist = []
-    for w in sabnzbd.GUIHANDLER.content():
-        w = w.replace('WARNING', T('WARNING:')).replace('ERROR', T('ERROR:'))
-        wlist.insert(0, unicoder(w))
-    info['warnings'] = wlist
+    info['warnings'] = sabnzbd.GUIHANDLER.content()
 
     return info
 
-def build_queue(web_dir=None, root=None, prim=True, webdir='', start=0, limit=0, trans=False, output=None, search=None):
+
+def build_queue(start=0, limit=0, trans=False, output=None, search=None):
     if output:
         converter = unicoder
     else:
         converter = xml_name
 
     # build up header full of basic information
-    info, pnfo_list, bytespersec, q_size, bytes_left_previous_page = build_queue_header(prim, webdir, search=search, start=start, limit=limit)
+    info, pnfo_list, bytespersec, q_size, bytes_left_previous_page = build_queue_header(search=search, start=start, limit=limit, output=output)
 
     datestart = datetime.datetime.now()
     priorities = {TOP_PRIORITY: 'Force', REPAIR_PRIORITY: 'Repair', HIGH_PRIORITY: 'High', NORMAL_PRIORITY: 'Normal', LOW_PRIORITY: 'Low'}
@@ -1317,8 +1296,8 @@ def build_queue(web_dir=None, root=None, prim=True, webdir='', start=0, limit=0,
     start = int_conv(start)
 
     info['refresh_rate'] = str(cfg.refresh_rate()) if cfg.refresh_rate() > 0 else ''
-    info['script_list'] = list_scripts()
-    info['cat_list'] = list_cats(output is None)
+    info['scripts'] = list_scripts()
+    info['categories'] = list_cats(output is None)
     info['rating_enable'] = bool(cfg.rating_enable())
     info['noofslots'] = q_size
     info['start'] = start
@@ -1345,7 +1324,6 @@ def build_queue(web_dir=None, root=None, prim=True, webdir='', start=0, limit=0,
         priority = pnfo.priority
         mbleft = (bytesleft / MEBI)
         mb = (bytes / MEBI)
-        missing = pnfo.missing
 
         slot = {'index': n, 'nzo_id': str(nzo_id)}
         slot['unpackopts'] = str(sabnzbd.opts_to_pp(pnfo.repair, pnfo.unpack, pnfo.delete))
@@ -1359,7 +1337,8 @@ def build_queue(web_dir=None, root=None, prim=True, webdir='', start=0, limit=0,
         slot['size'] = format_bytes(bytes)
         slot['sizeleft'] = format_bytes(bytesleft)
         slot['percentage'] = "%s" % (int(((mb - mbleft) / mb) * 100)) if mb != mbleft else '0'
-        slot['missing'] = pnfo.missing
+        slot['mbmissing'] = "%.2f" % (pnfo.bytes_missing / MEBI)
+        slot['direct_unpack'] = pnfo.direct_unpack
         if not output:
             slot['mb_fmt'] = locale.format('%d', int(mb), True)
             slot['mbdone_fmt'] = locale.format('%d', int(mb - mbleft), True)
@@ -1372,12 +1351,13 @@ def build_queue(web_dir=None, root=None, prim=True, webdir='', start=0, limit=0,
             else:
                 slot['status'] = Status.DOWNLOADING
         else:
-            # ensure compatibility of API status
-            if status in (Status.DELETED, ): status = Status.DOWNLOADING
+            # Ensure compatibility of API status
+            if status == Status.DELETED or priority == TOP_PRIORITY:
+                status = Status.DOWNLOADING
             slot['status'] = "%s" % (status)
 
         if (Downloader.do.paused or Downloader.do.postproc or is_propagating or  \
-           status not in (Status.DOWNLOADING, Status.QUEUED)) and priority != TOP_PRIORITY:
+           status not in (Status.DOWNLOADING, Status.FETCHING, Status.QUEUED)) and priority != TOP_PRIORITY:
             slot['timeleft'] = '0:00:00'
             slot['eta'] = 'unknown'
         else:
@@ -1391,8 +1371,9 @@ def build_queue(web_dir=None, root=None, prim=True, webdir='', start=0, limit=0,
                 datestart = datetime.datetime.now()
                 slot['eta'] = 'unknown'
 
-        if status == Status.GRABBING:
-            slot['avg_age'] = '---'
+        # Do not show age when it's not known
+        if average_date.year < 2000:
+            slot['avg_age'] = '-'
         else:
             slot['avg_age'] = calc_age(average_date, bool(trans))
 
@@ -1422,60 +1403,6 @@ def fast_queue():
     return paused, bytes_left, bpsnow, time_left
 
 
-def qstatus_data():
-    """ Build up the queue status as a nested object and output as a JSON object """
-
-    qnfo = NzbQueue.do.queue_info()
-    pnfo_list = qnfo.list
-
-    jobs = []
-    bytesleftprogess = 0
-    bpsnow = BPSMeter.do.get_bps()
-    for pnfo in pnfo_list:
-        filename = pnfo.filename
-        bytesleft = pnfo.bytes_left / MEBI
-        bytesleftprogess += pnfo.bytes_left
-        bytes = pnfo.bytes / MEBI
-        nzo_id = pnfo.nzo_id
-        jobs.append({"id": nzo_id,
-                        "mb": bytes,
-                        "mbleft": bytesleft,
-                        "filename": unicoder(filename),
-                        "timeleft": calc_timeleft(bytesleftprogess, bpsnow)})
-
-    state = "IDLE"
-    if Downloader.do.paused:
-        state = Status.PAUSED
-    elif qnfo.bytes_left / MEBI > 0:
-        state = Status.DOWNLOADING
-
-    speed_limit = Downloader.do.get_limit()
-    if speed_limit <= 0:
-        speed_limit = 100
-
-    status = {
-        "state": state,
-        "pp_active": not PostProcessor.do.empty(),
-        "paused": Downloader.do.paused,
-        "pause_int": scheduler.pause_int(),
-        "kbpersec": bpsnow / KIBI,
-        "speed": to_units(bpsnow, dec_limit=1),
-        "mbleft": qnfo.bytes_left / MEBI,
-        "mb": qnfo.bytes / MEBI,
-        "noofslots": len(pnfo_list),
-        "noofslots_total": qnfo.q_fullsize,
-        "have_warnings": str(sabnzbd.GUIHANDLER.count()),
-        "diskspace1": diskfree(cfg.download_dir.get_path()),
-        "diskspace2": diskfree(cfg.complete_dir.get_path()),
-        "timeleft": calc_timeleft(qnfo.bytes_left, bpsnow),
-        "loadavg": loadavg(),
-        "speedlimit": "{1:0.{0}f}".format(int(speed_limit % 1 > 0), speed_limit),
-        "speedlimit_abs": str(Downloader.do.get_limit_abs() or ''),
-        "jobs": jobs
-    }
-    return status
-
-
 def build_file_list(nzo_id):
     """ Build file lists for specified job
     """
@@ -1488,27 +1415,23 @@ def build_file_list(nzo_id):
         active_files = pnfo.active_files
         queued_files = pnfo.queued_files
 
-        n = 0
         for nzf in finished_files:
             jobs.append({'filename': xml_name(nzf.filename if nzf.filename else nzf.subject),
                          'mbleft': "%.2f" % (nzf.bytes_left / MEBI),
                          'mb': "%.2f" % (nzf.bytes / MEBI),
                          'bytes': "%.2f" % nzf.bytes,
                          'age': calc_age(nzf.date),
-                         'id': str(n),
+                         'nzf_id': nzf.nzf_id,
                          'status': 'finished'})
-            n += 1
 
         for nzf in active_files:
             jobs.append({'filename': xml_name(nzf.filename if nzf.filename else nzf.subject),
                          'mbleft': "%.2f" % (nzf.bytes_left / MEBI),
                          'mb': "%.2f" % (nzf.bytes / MEBI),
                          'bytes': "%.2f" % nzf.bytes,
-                         'nzf_id': nzf.nzf_id,
                          'age': calc_age(nzf.date),
-                         'id': str(n),
+                         'nzf_id': nzf.nzf_id,
                          'status': 'active'})
-            n += 1
 
         for nzf in queued_files:
             jobs.append({'filename': xml_name(nzf.filename if nzf.filename else nzf.subject),
@@ -1517,12 +1440,12 @@ def build_file_list(nzo_id):
                          'mb': "%.2f" % (nzf.bytes / MEBI),
                          'bytes': "%.2f" % nzf.bytes,
                          'age': calc_age(nzf.date),
-                         'id': str(n),
+                         'nzf_id': nzf.nzf_id,
                          'status': 'queued'})
-            n += 1
 
     return jobs
 
+
 def rss_qstatus():
     """ Return a RSS feed with the queue status """
     qnfo = NzbQueue.do.queue_info()
@@ -1531,8 +1454,7 @@ def rss_qstatus():
     rss = RSS()
     rss.channel.title = "SABnzbd Queue"
     rss.channel.description = "Overview of current downloads"
-    rss.channel.link = "http://%s:%s/sabnzbd/queue" % (
-        cfg.cherryhost(), cfg.cherryport())
+    rss.channel.link = "http://%s:%s%s/queue" % (cfg.cherryhost(), cfg.cherryport(), cfg.url_base())
     rss.channel.language = "en"
 
     item = Item()
@@ -1563,7 +1485,7 @@ def rss_qstatus():
 
         item = Item()
         item.title = name
-        item.link = "http://%s:%s/sabnzbd/history" % (cfg.cherryhost(), cfg.cherryport())
+        item.link = "http://%s:%s%s/history" % (cfg.cherryhost(), cfg.cherryport(), cfg.url_base())
         item.guid = nzo_id
         status_line = []
         status_line.append('<tr>')
@@ -1587,13 +1509,12 @@ def options_list(output):
     return report(output, keyword='options', data={
         'yenc': sabnzbd.decoder.HAVE_YENC,
         'par2': sabnzbd.newsunpack.PAR2_COMMAND,
-        'par2c': sabnzbd.newsunpack.PAR2C_COMMAND,
+        'multipar': sabnzbd.newsunpack.MULTIPAR_COMMAND,
         'rar': sabnzbd.newsunpack.RAR_COMMAND,
         'zip': sabnzbd.newsunpack.ZIP_COMMAND,
         '7zip': sabnzbd.newsunpack.SEVEN_COMMAND,
         'nice': sabnzbd.newsunpack.NICE_COMMAND,
-        'ionice': sabnzbd.newsunpack.IONICE_COMMAND,
-        'ssl': sabnzbd.newswrapper.HAVE_SSL
+        'ionice': sabnzbd.newsunpack.IONICE_COMMAND
     })
 
 
@@ -1603,13 +1524,14 @@ def retry_job(job, new_nzb, password):
         history_db = sabnzbd.connect_db()
         futuretype, url, pp, script, cat = history_db.get_other(job)
         if futuretype:
-            if pp == 'X': pp = None
+            if pp == 'X':
+                pp = None
             sabnzbd.add_url(url, pp, script, cat)
             history_db.remove_history(job)
         else:
             path = history_db.get_path(job)
             if path:
-                nzo_id = repair_job(platform_encode(path), new_nzb, password)
+                nzo_id = NzbQueue.do.repair_job(platform_encode(path), new_nzb, password)
                 history_db.remove_history(job)
                 return nzo_id
     return None
@@ -1653,6 +1575,7 @@ def Tspec(txt):
     else:
         return txt
 
+
 _SKIN_CACHE = {}    # Stores pre-translated acronyms
 # This special is to be used in interface.py for template processing
 # to be passed for the $T function: so { ..., 'T' : Ttemplate, ...}
@@ -1676,24 +1599,13 @@ def clear_trans_cache():
     sabnzbd.WEBUI_READY = True
 
 
-def build_header(prim, webdir=''):
+def build_header(webdir='', output=None):
     """ Build the basic header """
     try:
         uptime = calc_age(sabnzbd.START)
     except:
         uptime = "-"
 
-    if prim:
-        color = sabnzbd.WEB_COLOR
-    else:
-        color = sabnzbd.WEB_COLOR2
-    if not color:
-        color = ''
-
-    header = {'T': Ttemplate, 'Tspec': Tspec, 'Tx': Ttemplate, 'version': sabnzbd.__version__,
-               'paused': Downloader.do.paused or Downloader.do.postproc,
-               'pause_int': scheduler.pause_int(), 'paused_all': sabnzbd.PAUSED_ALL,
-               'uptime': uptime, 'color_scheme': color}
     speed_limit = Downloader.do.get_limit()
     if speed_limit <= 0:
         speed_limit = 100
@@ -1701,36 +1613,54 @@ def build_header(prim, webdir=''):
     if speed_limit_abs <= 0:
         speed_limit_abs = ''
 
-    free1 = diskfree(cfg.download_dir.get_path())
-    free2 = diskfree(cfg.complete_dir.get_path())
+    diskspace_info = diskspace()
 
-    header['helpuri'] = 'https://sabnzbd.org/wiki/'
-    header['diskspace1'] = "%.2f" % free1
-    header['diskspace2'] = "%.2f" % free2
-    header['diskspace1_norm'] = to_units(free1 * GIGI)
-    header['diskspace2_norm'] = to_units(free2 * GIGI)
-    header['diskspacetotal1'] = "%.2f" % disktotal(cfg.download_dir.get_path())
-    header['diskspacetotal2'] = "%.2f" % disktotal(cfg.complete_dir.get_path())
+    header = {}
+
+    # We don't output everything for API
+    if not output:
+        header['T'] = Ttemplate
+        header['Tspec'] = Tspec
+        header['Tx'] = Ttemplate
+        header['uptime'] = uptime
+        header['color_scheme'] = sabnzbd.WEB_COLOR or ''
+        header['helpuri'] = 'https://sabnzbd.org/wiki/'
+
+        header['restart_req'] = sabnzbd.RESTART_REQ
+        header['pid'] = os.getpid()
+        header['active_lang'] = cfg.language()
+
+        header['my_lcldata'] = sabnzbd.DIR_LCLDATA
+        header['my_home'] = sabnzbd.DIR_HOME
+        header['webdir'] = webdir or sabnzbd.WEB_DIR
+        header['url_base'] = cfg.url_base()
+
+        header['nt'] = sabnzbd.WIN32
+        header['darwin'] = sabnzbd.DARWIN
+
+        header['power_options'] = sabnzbd.WIN32 or sabnzbd.DARWIN or sabnzbd.LINUX_POWER
+        header['pp_pause_event'] = sabnzbd.scheduler.pp_pause_event()
+
+        header['session'] = cfg.api_key()
+        header['new_release'], header['new_rel_url'] = sabnzbd.NEW_VERSION
+
+    header['version'] = sabnzbd.__version__
+    header['paused'] = Downloader.do.paused or Downloader.do.postproc
+    header['pause_int'] = scheduler.pause_int()
+    header['paused_all'] = sabnzbd.PAUSED_ALL
+
+    header['diskspace1'] = "%.2f" % diskspace_info['download_dir'][1]
+    header['diskspace2'] = "%.2f" % diskspace_info['complete_dir'][1]
+    header['diskspace1_norm'] = to_units(diskspace_info['download_dir'][1] * GIGI)
+    header['diskspace2_norm'] = to_units(diskspace_info['complete_dir'][1] * GIGI)
+    header['diskspacetotal1'] = "%.2f" % diskspace_info['download_dir'][0]
+    header['diskspacetotal2'] = "%.2f" % diskspace_info['complete_dir'][0]
     header['loadavg'] = loadavg()
-    # Special formatting so only decimal points when needed
     header['speedlimit'] = "{1:0.{0}f}".format(int(speed_limit % 1 > 0), speed_limit)
     header['speedlimit_abs'] = "%s" % speed_limit_abs
-    header['restart_req'] = sabnzbd.RESTART_REQ
+
     header['have_warnings'] = str(sabnzbd.GUIHANDLER.count())
-    header['last_warning'] = sabnzbd.GUIHANDLER.last().replace('WARNING', ('WARNING:')).replace('ERROR', T('ERROR:'))
-    header['active_lang'] = cfg.language()
-    header['my_lcldata'] = sabnzbd.DIR_LCLDATA
-    header['my_home'] = sabnzbd.DIR_HOME
-
-    header['webdir'] = webdir
-    header['pid'] = os.getpid()
-
     header['finishaction'] = sabnzbd.QUEUECOMPLETE
-    header['nt'] = sabnzbd.WIN32
-    header['darwin'] = sabnzbd.DARWIN
-    header['power_options'] = sabnzbd.WIN32 or sabnzbd.DARWIN or sabnzbd.LINUX_POWER
-
-    header['session'] = cfg.api_key()
 
     header['quota'] = to_units(BPSMeter.do.quota)
     header['have_quota'] = bool(BPSMeter.do.quota > 0.0)
@@ -1741,22 +1671,13 @@ def build_header(prim, webdir=''):
     header['cache_size'] = format_bytes(anfo.cache_size)
     header['cache_max'] = str(anfo.cache_limit)
 
-    header['pp_pause_event'] = sabnzbd.scheduler.pp_pause_event()
-
-    if sabnzbd.NEW_VERSION:
-        header['new_release'], header['new_rel_url'] = sabnzbd.NEW_VERSION
-    else:
-        header['new_release'] = ''
-        header['new_rel_url'] = ''
-
     return header
 
 
-
-def build_queue_header(prim, webdir='', search=None, start=0, limit=0):
+def build_queue_header(search=None, start=0, limit=0, output=None):
     """ Build full queue header """
 
-    header = build_header(prim, webdir)
+    header = build_header(output=output)
 
     bytespersec = BPSMeter.do.get_bps()
     qnfo = NzbQueue.do.queue_info(search=search, start=start, limit=limit)
@@ -1765,7 +1686,7 @@ def build_queue_header(prim, webdir='', search=None, start=0, limit=0):
     bytes = qnfo.bytes
 
     header['kbpersec'] = "%.2f" % (bytespersec / KIBI)
-    header['speed'] = to_units(bytespersec, spaces=1, dec_limit=1)
+    header['speed'] = to_units(bytespersec, spaces=1)
     header['mbleft'] = "%.2f" % (bytesleft / MEBI)
     header['mb'] = "%.2f" % (bytes / MEBI)
     header['sizeleft'] = format_bytes(bytesleft)
@@ -1872,9 +1793,6 @@ def build_history(start=None, limit=None, verbose=False, verbose_list=None, sear
     cookie = cherrypy.request.cookie
     if 'history_verbosity' in cookie:
         k = cookie['history_verbosity'].value
-        c_path = cookie['history_verbosity']['path']
-        c_age = cookie['history_verbosity']['max-age']
-        c_version = cookie['history_verbosity']['version']
 
         if k == 'all':
             details_show_all = True
@@ -1950,21 +1868,6 @@ def build_history(start=None, limit=None, verbose=False, verbose_list=None, sear
     return (items, fetched_items, total_items)
 
 
-def format_history_for_queue():
-    """ Retrieves the information on currently active history items, and formats them for displaying in the queue """
-    slotinfo = []
-    history_items = get_active_history()
-
-    for item in history_items:
-        slot = {'nzo_id': item['nzo_id'],
-                'bookmark': '', 'filename': xml_name(item['name']), 'loaded': False,
-                'stages': item['stage_log'], 'status': item['status'], 'bytes': item['bytes'],
-                'size': item['size']}
-        slotinfo.append(slot)
-
-    return slotinfo
-
-
 def get_active_history(queue=None, items=None):
     """ Get the currently in progress and active history queue. """
     if items is None:
@@ -1979,7 +1882,7 @@ def get_active_history(queue=None, items=None):
             item['url'], item['status'], item['nzo_id'], item['storage'], item['path'], item['script_log'], \
             item['script_line'], item['download_time'], item['postproc_time'], item['stage_log'], \
             item['downloaded'], item['completeness'], item['fail_message'], item['url_info'], item['bytes'], \
-            dummy, dummy = history
+            dummy, dummy, item['password'] = history
         item['action_line'] = nzo.action_line
         item = unpack_history_info(item)
 
@@ -2030,40 +1933,6 @@ def calc_timeleft(bytesleft, bps):
         return '0:00:00'
 
 
-def calc_age(date, trans=False):
-    """ Calculate the age difference between now and date.
-        Value is returned as either days, hours, or minutes.
-        When 'trans' is True, time symbols will be translated.
-    """
-    if trans:
-        d = T('d')  # : Single letter abbreviation of day
-        h = T('h')  # : Single letter abbreviation of hour
-        m = T('m')  # : Single letter abbreviation of minute
-    else:
-        d = 'd'
-        h = 'h'
-        m = 'm'
-    try:
-        now = datetime.datetime.now()
-        # age = str(now - date).split(".")[0] #old calc_age
-
-        # time difference
-        dage = now - date
-        seconds = dage.seconds
-        # only one value should be returned
-        # if it is less than 1 day then it returns in hours, unless it is less than one hour where it returns in minutes
-        if dage.days:
-            age = '%s%s' % (dage.days, d)
-        elif seconds / 3600:
-            age = '%s%s' % (seconds / 3600, h)
-        else:
-            age = '%s%s' % (seconds / 60, m)
-    except:
-        age = "-"
-
-    return age
-
-
 def std_time(when):
     # Fri, 16 Nov 2007 16:42:01 GMT +0100
     item = time.strftime(time_format('%a, %d %b %Y %H:%M:%S'), time.localtime(when)).decode(codepage)
@@ -2081,7 +1950,7 @@ def list_scripts(default=False, none=True):
                 if (sabnzbd.WIN32 and os.path.splitext(script)[1].lower() in PATHEXT and
                                       not win32api.GetFileAttributes(script) & win32file.FILE_ATTRIBUTE_HIDDEN) or \
                    script.endswith('.py') or \
-                   (not sabnzbd.WIN32 and os.access(script, os.X_OK) and not os.path.basename(script).startswith('.')):
+                   (not sabnzbd.WIN32 and userxbit(script) and not os.path.basename(script).startswith('.')):
                     lst.append(os.path.basename(script))
         if none:
             lst.insert(0, 'None')
@@ -2091,8 +1960,10 @@ def list_scripts(default=False, none=True):
 
 
 def list_cats(default=True):
-    """ Return list of categories, when default==False use '*' for Default category """
-    lst = sorted(config.get_categories().keys())
+    """ Return list of (ordered) categories,
+        when default==False use '*' for Default category
+    """
+    lst = [cat['name'] for cat in config.get_ordered_categories()]
     if default:
         lst.remove('*')
         lst.insert(0, 'Default')
