@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,10 +22,11 @@ sabnzbd.articlecache - Article cache handling
 import sys
 import logging
 import threading
+import struct
 
 import sabnzbd
 from sabnzbd.decorators import synchronized
-from sabnzbd.constants import GIGI, ANFO, Status
+from sabnzbd.constants import GIGI, ANFO
 
 
 ARTICLE_LOCK = threading.Lock()
@@ -38,9 +39,15 @@ class ArticleCache(object):
         self.__cache_limit_org = 0
         self.__cache_limit = 0
         self.__cache_size = 0
-
         self.__article_list = []    # List of buffered articles
         self.__article_table = {}   # Dict of buffered articles
+
+        # On 32 bit we only allow the user to set 1GB
+        # For 64 bit we allow up to 4GB, in case somebody wants that
+        self.__cache_upper_limit = GIGI
+        if sabnzbd.DARWIN or sabnzbd.WIN64 or (struct.calcsize("P") * 8) == 64:
+            self.__cache_upper_limit = 4*GIGI
+
         ArticleCache.do = self
 
     @synchronized(ARTICLE_LOCK)
@@ -52,14 +59,14 @@ class ArticleCache(object):
         """ Called when cache limit changes """
         self.__cache_limit_org = limit
         if limit < 0:
-            self.__cache_limit = GIGI
+            self.__cache_limit = self.__cache_upper_limit
         else:
-            self.__cache_limit = min(limit, GIGI)
+            self.__cache_limit = min(limit, self.__cache_upper_limit)
 
     @synchronized(ARTICLE_LOCK)
     def reserve_space(self, data):
         """ Is there space left in the set limit? """
-        data_size = sys.getsizeof(data)*64
+        data_size = sys.getsizeof(data) * 64
         self.__cache_size += data_size
         if self.__cache_size + data_size > self.__cache_limit:
             return False
@@ -69,32 +76,30 @@ class ArticleCache(object):
     @synchronized(ARTICLE_LOCK)
     def free_reserve_space(self, data):
         """ Remove previously reserved space """
-        data_size = sys.getsizeof(data)*64
+        data_size = sys.getsizeof(data) * 64
         self.__cache_size -= data_size
         return self.__cache_size + data_size < self.__cache_limit
 
-
     @synchronized(ARTICLE_LOCK)
     def save_article(self, article, data):
-        nzf = article.nzf
-        nzo = nzf.nzo
-
-        if nzo.is_gone():
+        if article.nzf.nzo.is_gone():
             # Do not discard this article because the
             # file might still be processed at this moment!!
-            if sabnzbd.LOG_ALL:
-                logging.debug("%s is discarded", article)
             return
 
-        saved_articles = article.nzf.nzo.saved_articles
+        # Register article
+        if article not in article.nzf.nzo.saved_articles:
+            article.nzf.nzo.saved_articles.append(article)
 
-        if article not in saved_articles:
-            saved_articles.append(article)
+        if article.lowest_partnum and not article.nzf.import_finished:
+            # Write the first-fetched articles to disk
+            # Otherwise the cache could overflow
+            self.__flush_article(article, data)
+            return
 
         if self.__cache_limit:
             if self.__cache_limit < 0:
                 self.__add_to_cache(article, data)
-
             else:
                 data_size = len(data)
 
@@ -126,8 +131,6 @@ class ArticleCache(object):
             data = self.__article_table.pop(article)
             self.__article_list.remove(article)
             self.__cache_size -= len(data)
-            if sabnzbd.LOG_ALL:
-                logging.debug("Loaded %s from cache", article)
         elif article.art_id:
             data = sabnzbd.load_data(article.art_id, nzo.workpath, remove=True,
                                      do_pickle=False, silent=True)
@@ -147,8 +150,6 @@ class ArticleCache(object):
 
     @synchronized(ARTICLE_LOCK)
     def purge_articles(self, articles):
-        if sabnzbd.LOG_ALL:
-            logging.debug("Purgable articles -> %s", articles)
         for article in articles:
             if article in self.__article_list:
                 self.__article_list.remove(article)
@@ -164,14 +165,10 @@ class ArticleCache(object):
         if nzo.is_gone():
             # Do not discard this article because the
             # file might still be processed at this moment!!
-            if sabnzbd.LOG_ALL:
-                logging.debug("%s is discarded", article)
             return
 
         art_id = article.get_art_id()
         if art_id:
-            if sabnzbd.LOG_ALL:
-                logging.debug("Flushing %s to disk", article)
             # Save data, but don't complain when destination folder is missing
             # because this flush may come after completion of the NZO.
             sabnzbd.save_data(data, art_id, nzo.workpath, do_pickle=False, silent=True)
@@ -183,11 +180,8 @@ class ArticleCache(object):
             self.__cache_size -= len(self.__article_table[article])
         else:
             self.__article_list.append(article)
-
         self.__article_table[article] = data
         self.__cache_size += len(data)
-        if sabnzbd.LOG_ALL:
-            logging.debug("Added %s to cache", article)
 
 
 # Create the instance
