@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -28,15 +28,16 @@ import bz2
 import threading
 
 import sabnzbd
-from sabnzbd.constants import *
-from sabnzbd.utils.rarfile import is_rarfile, RarFile
+from sabnzbd.constants import SCAN_FILE_NAME, VALID_ARCHIVES
+import sabnzbd.utils.rarfile as rarfile
+from sabnzbd.encoding import platform_encode
+from sabnzbd.decorators import NzbQueueLocker
 from sabnzbd.newsunpack import is_sevenfile, SevenZip
 import sabnzbd.nzbstuff as nzbstuff
 import sabnzbd.misc as misc
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 
-
 def name_to_cat(fname, cat=None):
     """ Retrieve category from file name, but only if "cat" is None. """
     if cat is None and fname.startswith('{{'):
@@ -62,6 +63,41 @@ def CompareStat(tup1, tup2):
     return True
 
 
+def is_archive(path):
+    """ Check if file in path is an ZIP, RAR or 7z file
+    :param path: path to file
+    :return: (zf, status, expected_extension)
+            status: -1==Error/Retry, 0==OK, 1==Ignore
+    """
+    if zipfile.is_zipfile(path):
+        try:
+            zf = zipfile.ZipFile(path)
+            return 0, zf, '.zip'
+        except:
+            logging.info(T('Cannot read %s'), path, exc_info=True)
+            return -1, None, ''
+    elif rarfile.is_rarfile(path):
+        try:
+            # Set path to tool to open it
+            rarfile.UNRAR_TOOL = sabnzbd.newsunpack.RAR_COMMAND
+            zf = rarfile.RarFile(path)
+            return 0, zf, '.rar'
+        except:
+            logging.info(T('Cannot read %s'), path, exc_info=True)
+            return -1, None, ''
+    elif is_sevenfile(path):
+        try:
+            zf = SevenZip(path)
+            return 0, zf, '.7z'
+        except:
+            logging.info(T('Cannot read %s'), path, exc_info=True)
+            return -1, None, ''
+    else:
+        logging.info('Archive %s is not a real archive!', os.path.basename(path))
+        return 1, None, ''
+
+
+@NzbQueueLocker
 def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
                        priority=None, url='', nzbname=None, password=None, nzo_id=None):
     """ Analyse ZIP file and create job(s).
@@ -69,43 +105,26 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
         returns (status, nzo_ids)
             status: -1==Error/Retry, 0==OK, 1==Ignore
     """
-    from sabnzbd.nzbqueue import add_nzo
     nzo_ids = []
     if catdir is None:
         catdir = cat
 
     filename, cat = name_to_cat(filename, catdir)
 
-    if zipfile.is_zipfile(path):
-        try:
-            zf = zipfile.ZipFile(path)
-        except:
-            return -1, []
-    elif is_rarfile(path):
-        try:
-            zf = RarFile(path)
-        except:
-            return -1, []
-    elif is_sevenfile(path):
-        try:
-            zf = SevenZip(path)
-        except:
-            return -1, []
-    else:
-        return 1, []
+    status, zf, extension = is_archive(path)
+
+    if status != 0:
+        return status, []
 
     status = 1
     names = zf.namelist()
-    names.sort()
     nzbcount = 0
     for name in names:
         name = name.lower()
-        if not (name.endswith('.nzb') or name.endswith('.nfo') or name.endswith('/')):
-            status = 1
-            break
-        elif name.endswith('.nzb'):
+        if name.endswith('.nzb'):
             status = 0
             nzbcount += 1
+
     if status == 0:
         if nzbcount != 1:
             nzbname = None
@@ -114,28 +133,36 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
                 try:
                     data = zf.read(name)
                 except:
+                    logging.error(T('Cannot read %s'), name, exc_info=True)
                     zf.close()
                     return -1, []
                 name = os.path.basename(name)
                 if data:
+                    nzo = None
                     try:
                         nzo = nzbstuff.NzbObject(name, pp, script, data, cat=cat, url=url,
                                                  priority=priority, nzbname=nzbname)
                         if not nzo.password:
                             nzo.password = password
+                    except (TypeError, ValueError) as e:
+                        # Duplicate or empty, ignore
+                        pass
                     except:
-                        nzo = None
+                        # Something else is wrong, show error
+                        logging.error(T('Error while adding %s, removing'), name, exc_info=True)
+
                     if nzo:
                         if nzo_id:
                             # Re-use existing nzo_id, when a "future" job gets it payload
                             sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
                             nzo.nzo_id = nzo_id
-                        nzo_ids.append(add_nzo(nzo))
+                            nzo_id = None
+                        nzo_ids.append(sabnzbd.nzbqueue.NzbQueue.do.add(nzo))
                         nzo.update_rating()
         zf.close()
         try:
             if not keep:
-                os.remove(path)
+                misc.remove_file(path)
         except:
             logging.error(T('Error removing %s'), misc.clip_path(path))
             logging.info("Traceback: ", exc_info=True)
@@ -147,6 +174,7 @@ def ProcessArchiveFile(filename, path, pp=None, script=None, cat=None, catdir=No
     return status, nzo_ids
 
 
+@NzbQueueLocker
 def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=None, keep=False,
                       priority=None, nzbname=None, reuse=False, nzo_info=None, dup_check=True, url='',
                       password=None, nzo_id=None):
@@ -155,7 +183,6 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
         returns (status, nzo_ids)
             status: -2==Error/retry, -1==Error, 0==OK, 1==OK-but-ignorecannot-delete
     """
-    from sabnzbd.nzbqueue import add_nzo
     nzo_ids = []
     if catdir is None:
         catdir = cat
@@ -166,11 +193,11 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
         b2 = f.read(1)
         f.close()
 
-        if (b1 == '\x1f' and b2 == '\x8b'):
+        if b1 == '\x1f' and b2 == '\x8b':
             # gzip file or gzip in disguise
             name = filename.replace('.nzb.gz', '.nzb')
             f = gzip.GzipFile(path, 'rb')
-        elif (b1 == 'B' and b2 == 'Z'):
+        elif b1 == 'B' and b2 == 'Z':
             # bz2 file or bz2 in disguise
             name = filename.replace('.nzb.bz2', '.nzb')
             f = bz2.BZ2File(path, 'rb')
@@ -209,6 +236,8 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
             # Looks like an incomplete file, retry
             return -2, nzo_ids
         else:
+            # Something else is wrong, show error
+            logging.error(T('Error while adding %s, removing'), name, exc_info=True)
             return -1, nzo_ids
 
     if nzo:
@@ -216,11 +245,11 @@ def ProcessSingleFile(filename, path, pp=None, script=None, cat=None, catdir=Non
             # Re-use existing nzo_id, when a "future" job gets it payload
             sabnzbd.nzbqueue.NzbQueue.do.remove(nzo_id, add_to_history=False)
             nzo.nzo_id = nzo_id
-        nzo_ids.append(add_nzo(nzo, quiet=reuse))
+        nzo_ids.append(sabnzbd.nzbqueue.NzbQueue.do.add(nzo, quiet=reuse))
         nzo.update_rating()
     try:
         if not keep:
-            os.remove(path)
+            misc.remove_file(path)
     except:
         logging.error(T('Error removing %s'), misc.clip_path(path))
         logging.info("Traceback: ", exc_info=True)
@@ -289,13 +318,12 @@ class DirScanner(threading.Thread):
 
     def stop(self):
         """ Stop the dir scanner """
-        self.save()
         logging.info('Dirscanner shutting down')
         self.shutdown = True
 
     def save(self):
         """ Save dir scanner bookkeeping """
-        sabnzbd.save_admin((self.dirscan_dir, self.ignored, self.suspected), sabnzbd.SCAN_FILE_NAME)
+        sabnzbd.save_admin((self.dirscan_dir, self.ignored, self.suspected), SCAN_FILE_NAME)
 
     def run(self):
         """ Start the scanner """
@@ -325,7 +353,7 @@ class DirScanner(threading.Thread):
                 files = []
 
             for filename in files:
-                path = os.path.join(folder, filename)
+                path = os.path.join(folder, platform_encode(filename))
                 if os.path.isdir(path) or path in self.ignored or filename[0] == '.':
                     continue
 
@@ -409,7 +437,7 @@ class DirScanner(threading.Thread):
 
                 cats = config.get_categories()
                 for dd in list:
-                    dpath = os.path.join(dirscan_dir, dd)
+                    dpath = os.path.join(dirscan_dir, platform_encode(dd))
                     if os.path.isdir(dpath) and dd.lower() in cats:
                         run_dir(dpath, dd.lower())
             self.busy = False
