@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -27,20 +27,21 @@ from nntplib import NNTPPermanentError
 import socket
 import random
 import sys
+import Queue
 
 import sabnzbd
-from sabnzbd.decorators import synchronized, synchronized_CV, CV
+from sabnzbd.decorators import synchronized, NzbQueueLocker, DOWNLOADER_CV
+from sabnzbd.constants import MAX_DECODE_QUEUE, LIMIT_DECODE_QUEUE
 from sabnzbd.decoder import Decoder
-from sabnzbd.utils.sslinfo import ssl_protocols
 from sabnzbd.newswrapper import NewsWrapper, request_server_info
-import sabnzbd.growler as growler
-from sabnzbd.constants import *
+from sabnzbd.articlecache import ArticleCache
+import sabnzbd.notifier as notifier
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.bpsmeter import BPSMeter
 import sabnzbd.scheduler
-from sabnzbd.misc import from_units, int_conv
-from sabnzbd.utils.happyeyeballs import *
+from sabnzbd.misc import from_units, nntp_to_msg
+from sabnzbd.utils.happyeyeballs import happyeyeballs
 
 
 # Timeout penalty in minutes for each cause
@@ -59,8 +60,8 @@ TIMER_LOCK = RLock()
 
 class Server(object):
 
-    def __init__(self, id, displayname, host, port, timeout, threads, priority, ssl, ssl_type, send_group, username=None,
-                 password=None, optional=False, retention=0, categories=None):
+    def __init__(self, id, displayname, host, port, timeout, threads, priority, ssl, ssl_verify, send_group, username=None,
+                 password=None, optional=False, retention=0):
 
         self.id = id
         self.newid = None
@@ -72,7 +73,7 @@ class Server(object):
         self.threads = threads
         self.priority = priority
         self.ssl = ssl
-        self.ssl_type = None
+        self.ssl_verify = ssl_verify
         self.optional = optional
         self.retention = retention
         self.send_group = send_group
@@ -80,8 +81,6 @@ class Server(object):
         self.username = username
         self.password = password
 
-        self.categories = categories
-
         self.busy_threads = []
         self.idle_threads = []
         self.active = True
@@ -89,40 +88,50 @@ class Server(object):
         self.errormsg = ''
         self.warning = ''
         self.info = None     # Will hold getaddrinfo() list
+        self.ssl_info = ''  # Will hold the type and cipher of SSL connection
         self.request = False  # True if a getaddrinfo() request is pending
         self.have_body = 'free.xsusenet.com' not in host
         self.have_stat = True  # Assume server has "STAT", until proven otherwise
 
-        if ssl:
-            # When the user has set a supported protocol, use it
-            if ssl_type and ssl_type in ssl_protocols():
-                self.ssl_type = ssl_type
-
         for i in range(threads):
             self.idle_threads.append(NewsWrapper(self, i + 1))
 
     @property
     def hostip(self):
-        """ based on value of load_balancing() and self.info:
-            0: return the host name itself (so: do nothing)
-            1 and self.info has more than 1 entry (read: IP address): Return a random entry from the possible IPs
-            2 and self.info has more than 1 entry (read: IP address): Return the quickest IP based on the happyeyeballs algorithm
+        """ In case a server still has active connections, we use the same IP again
+            If new connection then based on value of load_balancing() and self.info:
+            0 - return the first entry, so all threads use the same IP
+            1 - and self.info has more than 1 entry (read: IP address): Return a random entry from the possible IPs
+            2 - and self.info has more than 1 entry (read: IP address): Return the quickest IP based on the happyeyeballs algorithm
             In case of problems: return the host name itself
         """
-        if cfg.load_balancing() == 1 and self.info and len(self.info) > 1:
+        # Check if already a successful ongoing connection
+        if self.busy_threads and self.busy_threads[0].nntp:
+            # Re-use that IP
+            logging.debug('%s: Re-using address %s', self.host, self.busy_threads[0].nntp.host)
+            return self.busy_threads[0].nntp.host
+
+        # Determine new IP
+        if cfg.load_balancing() == 0 and self.info:
+            # Just return the first one, so all next threads use the same IP
+            ip = self.info[0][4][0]
+            logging.debug('%s: Connecting to address %s', self.host, ip)
+
+        elif cfg.load_balancing() == 1 and self.info and len(self.info) > 1:
             # Return a random entry from the possible IPs
             rnd = random.randint(0, len(self.info) - 1)
             ip = self.info[rnd][4][0]
-            logging.debug('For server %s, using IP %s' % (self.host, ip))
+            logging.debug('%s: Connecting to address %s', self.host, ip)
+
         elif cfg.load_balancing() == 2 and self.info and len(self.info) > 1:
             # RFC6555 / Happy Eyeballs:
             ip = happyeyeballs(self.host, port=self.port, ssl=self.ssl)
             if ip:
-                logging.debug('For server %s, using IP %s as server' % (self.host, ip))
+                logging.debug('%s: Connecting to address %s', self.host, ip)
             else:
                 # nothing returned, so there was a connection problem
                 ip = self.host
-                logging.debug('For server %s, no successful IP connection possible' % (self.host))
+                logging.debug('%s: No successful IP connection was possible', self.host)
         else:
             ip = self.host
         return ip
@@ -164,6 +173,10 @@ class Downloader(Thread):
         # Used for reducing speed
         self.delayed = False
 
+        # Used to see if we can add a slowdown to the Downloader-loop
+        self.can_be_slowed = None
+        self.can_be_slowed_timer = 0
+
         self.postproc = False
 
         self.shutdown = False
@@ -178,12 +191,21 @@ class Downloader(Thread):
         self.write_fds = {}
 
         self.servers = []
+        self.server_dict = {} # For faster lookups, but is not updated later!
+        self.server_nr = 0
         self._timers = {}
 
         for server in config.get_servers():
             self.init_server(None, server)
 
-        self.decoder = Decoder(self.servers)
+        self.decoder_queue = Queue.Queue()
+
+        # Initialize decoders, only 1 for non-SABYenc
+        self.decoder_workers = []
+        nr_decoders = 2 if sabnzbd.decoder.SABYENC_ENABLED else 1
+        for i in range(nr_decoders):
+            self.decoder_workers.append(Decoder(self.servers, self.decoder_queue))
+
         Downloader.do = self
 
     def init_server(self, oldserver, newserver):
@@ -204,12 +226,11 @@ class Downloader(Thread):
             timeout = srv.timeout()
             threads = srv.connections()
             priority = srv.priority()
-            ssl = srv.ssl() and sabnzbd.newswrapper.HAVE_SSL
-            ssl_type = srv.ssl_type()
+            ssl = srv.ssl()
+            ssl_verify = srv.ssl_verify()
             username = srv.username()
             password = srv.password()
             optional = srv.optional()
-            categories = srv.categories()
             retention = float(srv.retention() * 24 * 3600)  # days ==> seconds
             send_group = srv.send_group()
             create = True
@@ -225,54 +246,55 @@ class Downloader(Thread):
                     break
 
         if create and enabled and host and port and threads:
-            self.servers.append(Server(newserver, displayname, host, port, timeout, threads, priority, ssl,
-                                            ssl_type, send_group,
-                                            username, password, optional, retention, categories=categories))
+            server = Server(newserver, displayname, host, port, timeout, threads, priority, ssl, ssl_verify,
+                                            send_group, username, password, optional, retention)
+            self.servers.append(server)
+            self.server_dict[newserver] = server
+
+        # Update server-count
+        self.server_nr = len(self.servers)
 
         return
 
-    @synchronized_CV
+    @NzbQueueLocker
     def set_paused_state(self, state):
         """ Set downloader to specified paused state """
         self.paused = state
 
-    @synchronized_CV
+    @NzbQueueLocker
     def resume(self):
-        logging.info("Resuming")
-        if self.paused:
-            growler.send_notification("SABnzbd", T('Resuming'), 'download')
+        # Do not notify when SABnzbd is still starting
+        if self.paused and sabnzbd.WEB_DIR:
+            logging.info("Resuming")
+            notifier.send_notification("SABnzbd", T('Resuming'), 'download')
         self.paused = False
 
-    @synchronized_CV
-    def pause(self, save=True):
+    @NzbQueueLocker
+    def pause(self):
         """ Pause the downloader, optionally saving admin """
         if not self.paused:
             self.paused = True
             logging.info("Pausing")
-            growler.send_notification("SABnzbd", T('Paused'), 'download')
+            notifier.send_notification("SABnzbd", T('Paused'), 'download')
             if self.is_paused():
                 BPSMeter.do.reset()
             if cfg.autodisconnect():
                 self.disconnect()
-            if save:
-                sabnzbd.save_state()
 
-    @synchronized_CV
     def delay(self):
         logging.debug("Delaying")
         self.delayed = True
 
-    @synchronized_CV
+    @NzbQueueLocker
     def undelay(self):
         logging.debug("Undelaying")
         self.delayed = False
 
-    @synchronized_CV
     def wait_for_postproc(self):
         logging.info("Waiting for post-processing to finish")
         self.postproc = True
 
-    @synchronized_CV
+    @NzbQueueLocker
     def resume_from_postproc(self):
         logging.info("Post-processing finished, resuming download")
         self.postproc = False
@@ -280,7 +302,6 @@ class Downloader(Thread):
     def disconnect(self):
         self.force_disconnect = True
 
-    @synchronized_CV
     def limit_speed(self, value):
         ''' Set the actual download speed in Bytes/sec
             When 'value' ends with a '%' sign or is within 1-100, it is interpreted as a pecentage of the maximum bandwidth
@@ -322,11 +343,10 @@ class Downloader(Thread):
             self.bandwidth_limit = 0
 
     def is_paused(self):
-        from sabnzbd.nzbqueue import NzbQueue
         if not self.paused:
             return False
         else:
-            if NzbQueue.do.has_forced_items():
+            if sabnzbd.nzbqueue.NzbQueue.do.has_forced_items():
                 return False
             else:
                 return True
@@ -335,7 +355,6 @@ class Downloader(Thread):
         """ Return True when this server has the highest priority of the active ones
             0 is the highest priority
         """
-
         for server in self.servers:
             if server is not me and server.active and server.priority < me.priority:
                 return False
@@ -345,7 +364,6 @@ class Downloader(Thread):
         return filter(nzo.server_in_try_list, self.servers)
 
     def maybe_block_server(self, server):
-        from sabnzbd.nzbqueue import NzbQueue
         if server.optional and server.active and (server.bad_cons / server.threads) > 3:
             # Optional and active server had too many problems,
             # disable it now and send a re-enable plan to the scheduler
@@ -361,21 +379,49 @@ class Downloader(Thread):
             # Make sure server address resolution is refreshed
             server.info = None
 
-            NzbQueue.do.reset_all_try_lists()
+            sabnzbd.nzbqueue.NzbQueue.do.reset_all_try_lists()
+
+    def decode(self, article, lines, raw_data):
+        self.decoder_queue.put((article, lines, raw_data))
+        # See if there's space left in cache, pause otherwise
+        # But do allow some articles to enter queue, in case of full cache
+        qsize = self.decoder_queue.qsize()
+        if (not ArticleCache.do.reserve_space(lines) and qsize > MAX_DECODE_QUEUE) or (qsize > LIMIT_DECODE_QUEUE):
+            sabnzbd.downloader.Downloader.do.delay()
 
     def run(self):
-        from sabnzbd.nzbqueue import NzbQueue
-        self.decoder.start()
+        # First check IPv6 connectivity
+        sabnzbd.EXTERNAL_IPV6 = sabnzbd.test_ipv6()
+        logging.debug('External IPv6 test result: %s', sabnzbd.EXTERNAL_IPV6)
+
+        # Then have to check the quality of SSL verification
+        if sabnzbd.HAVE_SSL_CONTEXT:
+            try:
+                import ssl
+                ctx = ssl.create_default_context()
+                base_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                ssl_sock = ctx.wrap_socket(base_sock, server_hostname=cfg.selftest_host())
+                ssl_sock.settimeout(2.0)
+                ssl_sock.connect((cfg.selftest_host(), 443))
+                ssl_sock.close()
+            except:
+                # Seems something is still wrong
+                sabnzbd.set_https_verification(0)
+                sabnzbd.HAVE_SSL_CONTEXT = False
+        logging.debug('SSL verification test: %s', sabnzbd.HAVE_SSL_CONTEXT)
+
+        # Start decoders
+        for decoder in self.decoder_workers:
+            decoder.start()
 
         # Kick BPS-Meter to check quota
         BPSMeter.do.update()
 
         while 1:
             for server in self.servers:
-                assert isinstance(server, Server)
                 for nw in server.busy_threads[:]:
                     if (nw.nntp and nw.nntp.error_msg) or (nw.timeout and time.time() > nw.timeout):
-                        if (nw.nntp and nw.nntp.error_msg):
+                        if nw.nntp and nw.nntp.error_msg:
                             self.__reset_nw(nw, "", warn=False)
                         else:
                             self.__reset_nw(nw, "timed out")
@@ -389,46 +435,42 @@ class Downloader(Thread):
                         if newid:
                             self.init_server(None, newid)
                         self.__restart -= 1
-                        NzbQueue.do.reset_all_try_lists()
+                        sabnzbd.nzbqueue.NzbQueue.do.reset_all_try_lists()
                         # Have to leave this loop, because we removed element
                         break
                     else:
                         # Restart pending, don't add new articles
                         continue
 
-                assert isinstance(server, Server)
                 if not server.idle_threads or server.restart or self.is_paused() or self.shutdown or self.delayed or self.postproc:
                     continue
 
-                if not (server.active and NzbQueue.do.has_articles_for(server)):
+                if not server.active:
                     continue
 
                 for nw in server.idle_threads[:]:
-                    assert isinstance(nw, NewsWrapper)
                     if nw.timeout:
                         if time.time() < nw.timeout:
                             continue
                         else:
                             nw.timeout = None
 
-                    if not server.active:
-                        break
-
                     if server.info is None:
                         self.maybe_block_server(server)
                         request_server_info(server)
                         break
 
-                    article = NzbQueue.do.get_article(server, self.servers)
+                    article = sabnzbd.nzbqueue.NzbQueue.do.get_article(server, self.servers)
 
                     if not article:
                         break
 
                     if server.retention and article.nzf.nzo.avg_stamp < time.time() - server.retention:
-                        # Article too old for the server, treat as missing
-                        if sabnzbd.LOG_ALL:
-                            logging.debug('Article %s too old for %s', article.article, server.id)
-                        self.decoder.decode(article, None)
+                        # Let's get rid of all the articles for this server at once
+                        logging.info('Job %s too old for %s, moving on', article.nzf.nzo.work_name, server.id)
+                        while article:
+                            self.decode(article, None, None)
+                            article = article.nzf.nzo.get_article(server, self.servers)
                         break
 
                     server.idle_threads.remove(nw)
@@ -440,8 +482,7 @@ class Downloader(Thread):
                         self.__request_article(nw)
                     else:
                         try:
-                            logging.info("%s@%s: Initiating connection",
-                                              nw.thrdnum, server.id)
+                            logging.info("%s@%s: Initiating connection", nw.thrdnum, server.id)
                             nw.init_connect(self.write_fds)
                         except:
                             logging.error(T('Failed to initialize %s@%s with reason: %s'), nw.thrdnum, server.id, sys.exc_info()[1])
@@ -456,8 +497,10 @@ class Downloader(Thread):
                         break
 
                 if empty:
-                    self.decoder.stop()
-                    self.decoder.join()
+                    # Start decoders
+                    for decoder in self.decoder_workers:
+                        decoder.stop()
+                        decoder.join()
 
                     for server in self.servers:
                         server.stop(self.read_fds, self.write_fds)
@@ -482,6 +525,23 @@ class Downloader(Thread):
             if readkeys or writekeys:
                 read, write, error = select.select(readkeys, writekeys, (), 1.0)
 
+                # Why check so often when so few things happened?
+                if self.can_be_slowed and len(readkeys) >= 8 and len(read) <= 2:
+                    time.sleep(0.01)
+
+                # Need to initialize the check during first 20 seconds
+                if self.can_be_slowed is None or self.can_be_slowed_timer:
+                    # Wait for stable speed to start testing
+                    if not self.can_be_slowed_timer and BPSMeter.do.get_stable_speed(timespan=10):
+                        self.can_be_slowed_timer = time.time()
+
+                    # Check 10 seconds after enabling slowdown
+                    if self.can_be_slowed_timer and time.time() > self.can_be_slowed_timer + 10:
+                        # Now let's check if it was stable in the last 10 seconds
+                        self.can_be_slowed = BPSMeter.do.get_stable_speed(timespan=10)
+                        self.can_be_slowed_timer = 0
+                        logging.debug('Downloader-slowdown: %r', self.can_be_slowed)
+
             else:
                 read, write, error = ([], [], [])
 
@@ -489,11 +549,11 @@ class Downloader(Thread):
 
                 time.sleep(1.0)
 
-                CV.acquire()
-                while (NzbQueue.do.is_empty() or self.is_paused() or self.delayed or self.postproc) and not \
+                DOWNLOADER_CV.acquire()
+                while (sabnzbd.nzbqueue.NzbQueue.do.is_empty() or self.is_paused() or self.delayed or self.postproc) and not \
                        self.shutdown and not self.__restart:
-                    CV.wait()
-                CV.release()
+                    DOWNLOADER_CV.wait()
+                DOWNLOADER_CV.release()
 
                 self.force_disconnect = False
 
@@ -545,24 +605,17 @@ class Downloader(Thread):
                     BPSMeter.do.update(server.id, bytes)
 
                     if nzo:
-                        if server.id in nzo.servercount:
-                            nzo.servercount[server.id] += bytes
-                        else:
-                            nzo.servercount[server.id] = bytes
-                        nzo.bytes_downloaded += bytes
-                        nzo.update_avg_kbs(BPSMeter.do.get_bps())
+                        nzo.update_download_stats(BPSMeter.do.get_bps(), server.id, bytes)
 
-                if len(nw.lines) == 1:
-                    code = nw.lines[0][:3]
-                    if not nw.connected or code == '480':
+                if not done and nw.status_code != '222':
+                    if not nw.connected or nw.status_code == '480':
                         done = False
 
                         try:
-                            nw.finish_connect(code)
+                            nw.finish_connect(nw.status_code)
                             if sabnzbd.LOG_ALL:
-                                logging.debug("%s@%s last message -> %s", nw.thrdnum, nw.server.id, nw.lines[0])
-                            nw.lines = []
-                            nw.data = ''
+                                logging.debug("%s@%s last message -> %s", nw.thrdnum, nw.server.id, nntp_to_msg(nw.data))
+                            nw.clear_data()
                         except NNTPPermanentError, error:
                             # Handle login problems
                             block = False
@@ -575,30 +628,39 @@ class Downloader(Thread):
                                 # Too many connections: remove this thread and reduce thread-setting for server
                                 # Plan to go back to the full number after a penalty timeout
                                 if server.active:
-                                    server.errormsg = T('Too many connections to server %s') % display_msg
-                                    logging.error(T('Too many connections to server %s'), server.id)
+                                    errormsg = T('Too many connections to server %s') % display_msg
+                                    if server.errormsg != errormsg:
+                                        server.errormsg = errormsg
+                                        logging.warning(T('Too many connections to server %s'), server.id)
                                     self.__reset_nw(nw, None, warn=False, destroy=True, quit=True)
                                     self.plan_server(server.id, _PENALTY_TOOMANY)
                                     server.threads -= 1
                             elif ecode in ('502', '481', '482') and clues_too_many_ip(msg):
                                 # Account sharing?
                                 if server.active:
-                                    server.errormsg = T('Probable account sharing') + display_msg
-                                    name = ' (%s)' % server.id
-                                    logging.error(T('Probable account sharing') + name)
-                                    penalty = _PENALTY_SHARE
+                                    errormsg = T('Probable account sharing') + display_msg
+                                    if server.errormsg != errormsg:
+                                        server.errormsg = errormsg
+                                        name = ' (%s)' % server.id
+                                        logging.warning(T('Probable account sharing') + name)
+                                penalty = _PENALTY_SHARE
+                                block = True
                             elif ecode in ('481', '482', '381') or (ecode == '502' and clues_login(msg)):
                                 # Cannot login, block this server
                                 if server.active:
-                                    server.errormsg = T('Failed login for server %s') % display_msg
-                                    logging.error(T('Failed login for server %s'), server.id)
+                                    errormsg = T('Failed login for server %s') % display_msg
+                                    if server.errormsg != errormsg:
+                                        server.errormsg = errormsg
+                                        logging.error(T('Failed login for server %s'), server.id)
                                 penalty = _PENALTY_PERM
                                 block = True
-                            elif ecode == '502':
+                            elif ecode in ('502', '482'):
                                 # Cannot connect (other reasons), block this server
                                 if server.active:
-                                    server.errormsg = T('Cannot connect to server %s [%s]') % ('', display_msg)
-                                    logging.warning(T('Cannot connect to server %s [%s]'), server.id, msg)
+                                    errormsg = T('Cannot connect to server %s [%s]') % ('', display_msg)
+                                    if server.errormsg != errormsg:
+                                        server.errormsg = errormsg
+                                        logging.warning(T('Cannot connect to server %s [%s]'), server.id, msg)
                                 if clues_pay(msg):
                                     penalty = _PENALTY_PERM
                                 else:
@@ -613,23 +675,23 @@ class Downloader(Thread):
                             else:
                                 # Unknown error, just keep trying
                                 if server.active:
-                                    server.errormsg = T('Cannot connect to server %s [%s]') % ('', display_msg)
-                                    logging.error(T('Cannot connect to server %s [%s]'), server.id, msg)
-                                    penalty = _PENALTY_UNKNOWN
+                                    errormsg = T('Cannot connect to server %s [%s]') % ('', display_msg)
+                                    if server.errormsg != errormsg:
+                                        server.errormsg = errormsg
+                                        logging.warning(T('Cannot connect to server %s [%s]'), server.id, msg)
+                                penalty = _PENALTY_UNKNOWN
+                                block = True
                             if block or (penalty and server.optional):
                                 if server.active:
                                     server.active = False
-                                    if (not server.optional) and cfg.no_penalties():
-                                        penalty = _PENALTY_SHORT
                                     if penalty and (block or server.optional):
-                                        logging.info('Server %s ignored for %s minutes', server.id, penalty)
                                         self.plan_server(server.id, penalty)
-                                    NzbQueue.do.reset_all_try_lists()
+                                    sabnzbd.nzbqueue.NzbQueue.do.reset_all_try_lists()
                                 self.__reset_nw(nw, None, warn=False, quit=True)
                             continue
                         except:
                             logging.error(T('Connecting %s@%s failed, message=%s'),
-                                              nw.thrdnum, nw.server.id, nw.lines[0])
+                                              nw.thrdnum, nw.server.id, nntp_to_msg(nw.data))
                             # No reset-warning needed, above logging is sufficient
                             self.__reset_nw(nw, None, warn=False)
 
@@ -637,39 +699,33 @@ class Downloader(Thread):
                             logging.info("Connecting %s@%s finished", nw.thrdnum, nw.server.id)
                             self.__request_article(nw)
 
-                    elif code == '223':
+                    elif nw.status_code == '223':
                         done = True
                         logging.debug('Article <%s> is present', article.article)
-                        self.decoder.decode(article, nw.lines)
 
-                    elif code == '211':
+                    elif nw.status_code == '211':
                         done = False
-
-                        logging.debug("group command ok -> %s",
-                                      nw.lines)
+                        logging.debug("group command ok -> %s", nntp_to_msg(nw.data))
                         nw.group = nw.article.nzf.nzo.group
-                        nw.lines = []
-                        nw.data = ''
+                        nw.clear_data()
                         self.__request_article(nw)
 
-                    elif code in ('411', '423', '430'):
+                    elif nw.status_code in ('411', '423', '430'):
                         done = True
-                        nw.lines = None
+                        logging.debug('Thread %s@%s: Article %s missing (error=%s)',
+                                        nw.thrdnum, nw.server.id, article.article, nw.status_code)
+                        nw.clear_data()
 
-                        logging.info('Thread %s@%s: Article ' +
-                                        '%s missing (error=%s)',
-                                        nw.thrdnum, nw.server.id, article.article, code)
-
-                    elif code == '480':
+                    elif nw.status_code == '480':
                         if server.active:
                             server.active = False
                             server.errormsg = T('Server %s requires user/password') % ''
                             self.plan_server(server.id, 0)
-                            NzbQueue.do.reset_all_try_lists()
+                            sabnzbd.nzbqueue.NzbQueue.do.reset_all_try_lists()
                         msg = T('Server %s requires user/password') % nw.server.id
                         self.__reset_nw(nw, msg, quit=True)
 
-                    elif code == '500':
+                    elif nw.status_code == '500':
                         if nzo.precheck:
                             # Assume "STAT" command is not supported
                             server.have_stat = False
@@ -678,15 +734,15 @@ class Downloader(Thread):
                             # Assume "BODY" command is not supported
                             server.have_body = False
                             logging.debug('Server %s does not support BODY', server.id)
-                        nw.lines = []
-                        nw.data = ''
+                        nw.clear_data()
                         self.__request_article(nw)
 
                 if done:
                     server.bad_cons = 0  # Succesful data, clear "bad" counter
+                    server.errormsg = server.warning = ''
                     if sabnzbd.LOG_ALL:
                         logging.debug('Thread %s@%s: %s done', nw.thrdnum, server.id, article.article)
-                    self.decoder.decode(article, nw.lines)
+                    self.decode(article, nw.lines, nw.data)
 
                     nw.soft_reset()
                     server.busy_threads.remove(nw)
@@ -703,7 +759,6 @@ class Downloader(Thread):
         return None
 
     def __reset_nw(self, nw, errormsg, warn=True, wait=True, destroy=False, quit=False):
-        from sabnzbd.nzbqueue import NzbQueue
         server = nw.server
         article = nw.article
         fileno = None
@@ -735,22 +790,19 @@ class Downloader(Thread):
         if article:
             if article.tries > cfg.max_art_tries() and (article.fetcher.optional or not cfg.max_art_opt()):
                 # Too many tries on this server, consider article missing
-                self.decoder.decode(article, None)
+                self.decode(article, None, None)
             else:
-                # Remove this server from try_list
-                article.fetcher = None
-
-                nzf = article.nzf
-                nzo = nzf.nzo
-
-                # Allow all servers to iterate over each nzo/nzf again ##
-                NzbQueue.do.reset_try_lists(nzf, nzo)
+                # Allow all servers to iterate over each nzo/nzf again
+                sabnzbd.nzbqueue.NzbQueue.do.reset_try_lists(article)
 
         if destroy:
             nw.terminate(quit=quit)
         else:
             nw.hard_reset(wait, quit=quit)
 
+        # Empty SSL info, it might change on next connect
+        server.ssl_info = ''
+
     def __request_article(self, nw):
         try:
             nzo = nw.article.nzf.nzo
@@ -785,6 +837,10 @@ class Downloader(Thread):
     @synchronized(TIMER_LOCK)
     def plan_server(self, server_id, interval):
         """ Plan the restart of a server in 'interval' minutes """
+        if cfg.no_penalties() and interval > _PENALTY_SHORT:
+            # Overwrite in case of no_penalties
+            interval = _PENALTY_SHORT
+
         logging.debug('Set planned server resume %s in %s mins', server_id, interval)
         if server_id not in self._timers:
             self._timers[server_id] = []
@@ -802,7 +858,7 @@ class Downloader(Thread):
                 del self._timers[server_id]
                 self.init_server(server_id, server_id)
 
-    @synchronized_CV
+    @NzbQueueLocker
     @synchronized(TIMER_LOCK)
     def unblock(self, server_id):
         # Remove timer
@@ -821,7 +877,7 @@ class Downloader(Thread):
         for server_id in self._timers.keys():
             self.unblock(server_id)
 
-    @synchronized_CV
+    @NzbQueueLocker
     @synchronized(TIMER_LOCK)
     def check_timers(self):
         """ Make sure every server without a non-expired timer is active """
@@ -841,27 +897,26 @@ class Downloader(Thread):
                     logging.debug('Forcing activation of server %s', server.id)
                     self.init_server(server.id, server.id)
 
-    @synchronized_CV
     def update_server(self, oldserver, newserver):
         self.init_server(oldserver, newserver)
 
-    @synchronized_CV
+    @NzbQueueLocker
     def wakeup(self):
         """ Just rattle the semaphore """
         pass
 
     def stop(self):
         self.shutdown = True
-        growler.send_notification("SABnzbd", T('Shutting down'), 'startup')
+        notifier.send_notification("SABnzbd", T('Shutting down'), 'startup')
 
 
 def stop():
-    CV.acquire()
+    DOWNLOADER_CV.acquire()
     try:
         Downloader.do.stop()
     finally:
-        CV.notifyAll()
-        CV.release()
+        DOWNLOADER_CV.notify_all()
+        DOWNLOADER_CV.release()
     try:
         Downloader.do.join()
     except:
@@ -881,7 +936,8 @@ def clues_too_many(text):
     """ Check for any "too many connections" clues in the response code """
     text = text.lower()
     for clue in ('exceed', 'connections', 'too many', 'threads', 'limit'):
-        if clue in text:
+        # Not 'download limit exceeded' error
+        if (clue in text) and ('download' not in text):
             return True
     return False
 
@@ -898,7 +954,7 @@ def clues_too_many_ip(text):
 def clues_pay(text):
     """ Check for messages about payments """
     text = text.lower()
-    for clue in ('credits', 'paym', 'expired'):
+    for clue in ('credits', 'paym', 'expired', 'exceeded'):
         if clue in text:
             return True
     return False
