@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,30 +24,32 @@ import sys
 import logging
 import urllib
 import re
+import ctypes
 import shutil
 import threading
 import subprocess
 import socket
 import time
+import datetime
 import fnmatch
 import stat
-try:
-    socket.ssl
-    _HAVE_SSL = True
-except:
-    _HAVE_SSL = False
+import inspect
+import urllib2
 from urlparse import urlparse
 
 import sabnzbd
 from sabnzbd.decorators import synchronized
-from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, GIGI, MEBI
+from sabnzbd.constants import DEFAULT_PRIORITY, FUTURE_Q_FOLDER, JOB_ADMIN, \
+     GIGI, MEBI, DEF_ARTICLE_CACHE_DEFAULT, DEF_ARTICLE_CACHE_MAX
 import sabnzbd.config as config
 import sabnzbd.cfg as cfg
 from sabnzbd.encoding import unicoder, special_fixer, gUTF
 
-RE_VERSION = re.compile(r'(\d+)\.(\d+)\.(\d+)([a-zA-Z]*)(\d*)')
-RE_UNITS = re.compile(r'(\d+\.*\d*)\s*([KMGTP]{0,1})', re.I)
 TAB_UNITS = ('', 'K', 'M', 'G', 'T', 'P')
+RE_UNITS = re.compile(r'(\d+\.*\d*)\s*([KMGTP]{0,1})', re.I)
+RE_VERSION = re.compile(r'(\d+)\.(\d+)\.(\d+)([a-zA-Z]*)(\d*)')
+RE_IP4 = re.compile(r'inet\s+(addr:\s*){0,1}(\d+\.\d+\.\d+\.\d+)')
+RE_IP6 = re.compile(r'inet6\s+(addr:\s*){0,1}([0-9a-f:]+)', re.I)
 
 # Check if strings are defined for AM and PM
 HAVE_AMPM = bool(time.strftime('%p', time.localtime()))
@@ -61,6 +63,49 @@ def time_format(fmt):
         return fmt
 
 
+def calc_age(date, trans=False):
+    """ Calculate the age difference between now and date.
+        Value is returned as either days, hours, or minutes.
+        When 'trans' is True, time symbols will be translated.
+    """
+    if trans:
+        d = T('d')  # : Single letter abbreviation of day
+        h = T('h')  # : Single letter abbreviation of hour
+        m = T('m')  # : Single letter abbreviation of minute
+    else:
+        d = 'd'
+        h = 'h'
+        m = 'm'
+    try:
+        now = datetime.datetime.now()
+        # age = str(now - date).split(".")[0] #old calc_age
+
+        # time difference
+        dage = now - date
+        seconds = dage.seconds
+        # only one value should be returned
+        # if it is less than 1 day then it returns in hours, unless it is less than one hour where it returns in minutes
+        if dage.days:
+            age = '%s%s' % (dage.days, d)
+        elif seconds / 3600:
+            age = '%s%s' % (seconds / 3600, h)
+        else:
+            age = '%s%s' % (seconds / 60, m)
+    except:
+        age = "-"
+
+    return age
+
+
+def monthrange(start, finish):
+    """ Calculate months between 2 dates, used in the Config template """
+    months = (finish.year - start.year) * 12 + finish.month + 1
+    for i in xrange(start.month, months):
+        year  = (i - 1) / 12 + start.year
+        month = (i - 1) % 12 + 1
+        yield datetime.date(year, month, 1)
+
+
 def safe_lower(txt):
     """ Return lowercased string. Return '' for None """
     if txt:
@@ -69,13 +114,22 @@ def safe_lower(txt):
         return ''
 
 
+def safe_fnmatch(f, pattern):
+    """ fnmatch will fail if the pattern contains any of it's
+        key characters, like [, ] or !.
+    """
+    try:
+        return fnmatch.fnmatch(f, pattern)
+    except re.error:
+        return False
+
+
 def globber(path, pattern=u'*'):
     """ Return matching base file/folder names in folder `path` """
     # Cannot use glob.glob() because it doesn't support Windows long name notation
     if os.path.exists(path):
-        return [f for f in os.listdir(path) if fnmatch.fnmatch(f, pattern)]
-    else:
-        return []
+        return [f for f in os.listdir(path) if safe_fnmatch(f, pattern)]
+    return []
 
 
 def globber_full(path, pattern=u'*'):
@@ -83,13 +137,12 @@ def globber_full(path, pattern=u'*'):
     # Cannot use glob.glob() because it doesn't support Windows long name notation
     if os.path.exists(path):
         try:
-            return [os.path.join(path, f) for f in os.listdir(path) if fnmatch.fnmatch(f, pattern)]
+            return [os.path.join(path, f) for f in os.listdir(path) if safe_fnmatch(f, pattern)]
         except UnicodeDecodeError:
             # This happens on Linux when names are incorrectly encoded, retry using a non-Unicode path
             path = path.encode('utf-8')
-            return [os.path.join(path, f) for f in os.listdir(path) if fnmatch.fnmatch(f, pattern)]
-    else:
-        return []
+            return [os.path.join(path, f) for f in os.listdir(path) if safe_fnmatch(f, pattern)]
+    return []
 
 
 def cat_to_opts(cat, pp=None, script=None, priority=None):
@@ -116,7 +169,7 @@ def cat_to_opts(cat, pp=None, script=None, priority=None):
         if safe_lower(script) in ('', 'default'):
             script = def_cat.script()
 
-    if priority is None or priority == DEFAULT_PRIORITY:
+    if priority is None or priority == '' or priority == DEFAULT_PRIORITY:
         priority = my_cat.priority()
         if priority == DEFAULT_PRIORITY:
             priority = def_cat.priority()
@@ -151,42 +204,41 @@ def wildcard_to_re(text):
 def cat_convert(cat):
     """ Convert indexer's category/group-name to user categories.
         If no match found, but indexer-cat equals user-cat, then return user-cat
+        If no match found, but the indexer-cat starts with the user-cat, return user-cat
         If no match found, return None
     """
-    newcat = cat
-    found = False
-
     if cat and cat.lower() != 'none':
-        cats = config.get_categories()
+        cats = config.get_ordered_categories()
+        raw_cats = config.get_categories()
         for ucat in cats:
             try:
-                indexer = cats[ucat].newzbin()
-                if type(indexer) != type([]):
+                # Ordered cat-list has tags only as string
+                indexer = raw_cats[ucat['name']].newzbin()
+                if not isinstance(indexer, list):
                     indexer = [indexer]
             except:
                 indexer = []
             for name in indexer:
                 if re.search('^%s$' % wildcard_to_re(name), cat, re.I):
-                    if '.' not in name:
-                        logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat)
+                    if '.' in name:
+                        logging.debug('Convert group "%s" to user-cat "%s"', cat, ucat['name'])
                     else:
-                        logging.debug('Convert group "%s" to user-cat "%s"', cat, ucat)
-                    newcat = ucat
-                    found = True
-                    break
-            if found:
-                break
+                        logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat['name'])
+                    return ucat['name']
 
-        if not found:
-            for ucat in cats:
-                if cat.lower() == ucat.lower():
-                    found = True
-                    break
+        # Try to find full match between user category and indexer category
+        for ucat in cats:
+            if cat.lower() == ucat['name'].lower():
+                logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat['name'])
+                return ucat['name']
 
-    if found:
-        return newcat
-    else:
-        return None
+        # Try to find partial match between user category and indexer category
+        for ucat in cats:
+            if cat.lower().startswith(ucat['name'].lower()):
+                logging.debug('Convert index site category "%s" to user-cat "%s"', cat, ucat['name'])
+                return ucat['name']
+
+    return None
 
 
 ##############################################################################
@@ -207,15 +259,32 @@ def replace_win_devices(name):
             if lname == dev or lname.startswith(dev + '.'):
                 name = '_' + name
                 break
+
+    # Remove special NTFS filename
+    if lname.startswith('$mft'):
+        name = name.replace('$', 'S', 1)
+
     return name
 
+
+def has_win_device(p):
+    """ Return True if filename part contains forbidden name
+        Before and after sanitizing
+    """
+    p = os.path.split(p)[1].lower()
+    for dev in _DEVICES:
+        if p == dev or p.startswith(dev + '.') or p.startswith('_' + dev + '.'):
+            return True
+    return False
+
+
 if sabnzbd.WIN32:
     # the colon should be here too, but we'll handle that separately
-    CH_ILLEGAL = r'\/<>?*|"'
-    CH_LEGAL = r'++{}!@#`'
+    CH_ILLEGAL = '\/<>?*|"\t'
+    CH_LEGAL = '++{}!@#`+'
 else:
-    CH_ILLEGAL = r'/'
-    CH_LEGAL = r'+'
+    CH_ILLEGAL = '/'
+    CH_LEGAL = '+'
 
 
 def sanitize_filename(name):
@@ -235,6 +304,9 @@ def sanitize_filename(name):
             # Compensate for the foolish way par2 on OSX handles a colon character
             name = name[name.rfind(':') + 1:]
 
+    if sabnzbd.WIN32 or cfg.sanitize_safe():
+        name = replace_win_devices(name)
+
     lst = []
     for ch in name.strip():
         if ch in illegal:
@@ -286,12 +358,7 @@ def sanitize_foldername(name, limit=True):
         else:
             lst.append(ch)
     name = ''.join(lst)
-
     name = name.strip()
-    if name != '.' and name != '..':
-        name = name.rstrip('.')
-    if not name:
-        name = 'unknown'
 
     if sabnzbd.WIN32 or cfg.sanitize_safe():
         name = replace_win_devices(name)
@@ -300,6 +367,12 @@ def sanitize_foldername(name, limit=True):
     if limit and len(name) > maxlen:
         name = name[:maxlen]
 
+    # And finally, make sure it doesn't end in a dot
+    if name != '.' and name != '..':
+        name = name.rstrip('.')
+    if not name:
+        name = 'unknown'
+
     return name
 
 
@@ -329,20 +402,29 @@ def sanitize_and_trim_path(path):
     return os.path.abspath(os.path.normpath(new_path))
 
 
-def flag_file(path, flag, create=False):
-    """ Create verify flag file or return True if it already exists """
-    path = os.path.join(path, JOB_ADMIN)
-    path = os.path.join(path, flag)
-    if create:
-        try:
-            f = open(path, 'w')
-            f.write('ok\n')
-            f.close()
-            return True
-        except IOError:
-            return False
-    else:
-        return os.path.exists(path)
+def sanitize_files_in_folder(folder):
+    """ Sanitize each file in the folder, return list of new names
+    """
+    lst = []
+    for root, _, files in os.walk(folder):
+        for file_ in files:
+            path = os.path.join(root, file_)
+            new_path = os.path.join(root, sanitize_filename(file_))
+            if path != new_path:
+                try:
+                    os.rename(path, new_path)
+                    path = new_path
+                except:
+                    logging.debug('Cannot rename %s to %s', path, new_path)
+            lst.append(path)
+    return lst
+
+
+def is_obfuscated_filename(filename):
+    """ Check if this file has an extension, if not, it's
+        probably obfuscated and we don't use it
+    """
+    return (os.path.splitext(filename)[1] == '')
 
 
 ##############################################################################
@@ -657,12 +739,12 @@ def from_units(val):
         return 0.0
 
 
-def to_units(val, spaces=0, dec_limit=2, postfix=''):
+def to_units(val, spaces=0, postfix=''):
     """ Convert number to K/M/G/T/P notation
         Add "spaces" if not ending in letter
-        dig_limit==1 show single decimal for M and higher
-        dig_limit==2 show single decimal for G and higher
+        Show single decimal for M and higher
     """
+    dec_limit = 1
     decimals = 0
     if val < 0:
         sign = '-'
@@ -690,6 +772,31 @@ def to_units(val, spaces=0, dec_limit=2, postfix=''):
     return fmt % (sign, val, unit, postfix)
 
 
+def caller_name(skip=2):
+    """Get a name of a caller in the format module.method
+       Originally used: https://gist.github.com/techtonik/2151727
+       Adapted for speed by using sys calls directly
+    """
+    # Only do the tracing on Debug (function is always called)
+    if cfg.log_level() != 2:
+        return 'N/A'
+
+    parentframe = sys._getframe(skip)
+    function_name = parentframe.f_code.co_name
+
+    # Modulename not available in the binaries, we can use the filename instead
+    if getattr(sys, 'frozen', None):
+        module_name = inspect.getfile(parentframe)
+    else:
+        module_name = inspect.getmodule(parentframe).__name__
+
+    # For decorated functions we have to go deeper
+    if function_name in ('call_func', 'wrap') and skip == 2:
+        return caller_name(4)
+
+    return ".".join([module_name, function_name])
+
+
 def same_file(a, b):
     """ Return 0 if A and B have nothing in common
         return 1 if A and B are actually the same path
@@ -718,7 +825,7 @@ def exit_sab(value):
     sys.stdout.flush()
     if getattr(sys, 'frozen', None) == 'macosx_app':
         sabnzbd.SABSTOP = True
-        from PyObjCTools import AppHelper  # @UnresolvedImport
+        from PyObjCTools import AppHelper
         AppHelper.stopEventLoop()
     sys.exit(value)
 
@@ -740,6 +847,14 @@ def split_host(srv):
     return (host, port)
 
 
+def get_from_url(url):
+    """ Retrieve URL and return content """
+    try:
+        return urllib2.urlopen(url).read()
+    except:
+        return None
+
+
 def check_mount(path):
     """ Return False if volume isn't mounted on Linux or OSX
         Retry 6 times with an interval of 1 sec.
@@ -760,12 +875,80 @@ def check_mount(path):
     return not m
 
 
+def get_cache_limit():
+    """ Depending on OS, calculate cache limits.
+        In ArticleCache it will make sure we stay
+        within system limits for 32/64 bit
+    """
+    # Calculate, if possible
+    try:
+        if sabnzbd.WIN32:
+            # Windows
+            mem_bytes = get_windows_memory()
+        elif sabnzbd.DARWIN:
+            # macOS
+            mem_bytes = get_darwin_memory()
+        else:
+            # Linux
+            mem_bytes = (os.sysconf('SC_PAGE_SIZE') * os.sysconf('SC_PHYS_PAGES'))
+
+        # Use 1/4th of available memory
+        mem_bytes = mem_bytes/4
+
+        # We don't want to set a value that's too high
+        if mem_bytes > from_units(DEF_ARTICLE_CACHE_MAX):
+            return DEF_ARTICLE_CACHE_MAX
+
+        # We make sure it's at least a valid value
+        if mem_bytes > from_units('32M'):
+            return to_units(mem_bytes)
+    except:
+        pass
+
+    # Always at least minimum on Windows/macOS
+    if sabnzbd.WIN32 and sabnzbd.DARWIN:
+        return DEF_ARTICLE_CACHE_DEFAULT
+
+    # If failed, leave empty for Linux so user needs to decide
+    return ''
+
+
+def get_windows_memory():
+    """ Use ctypes to extract available memory """
+    class MEMORYSTATUSEX(ctypes.Structure):
+        _fields_ = [
+            ("dwLength", ctypes.c_ulong),
+            ("dwMemoryLoad", ctypes.c_ulong),
+            ("ullTotalPhys", ctypes.c_ulonglong),
+            ("ullAvailPhys", ctypes.c_ulonglong),
+            ("ullTotalPageFile", ctypes.c_ulonglong),
+            ("ullAvailPageFile", ctypes.c_ulonglong),
+            ("ullTotalVirtual", ctypes.c_ulonglong),
+            ("ullAvailVirtual", ctypes.c_ulonglong),
+            ("sullAvailExtendedVirtual", ctypes.c_ulonglong),
+        ]
+
+        def __init__(self):
+            # have to initialize this to the size of MEMORYSTATUSEX
+            self.dwLength = ctypes.sizeof(self)
+            super(MEMORYSTATUSEX, self).__init__()
+
+    stat = MEMORYSTATUSEX()
+    ctypes.windll.kernel32.GlobalMemoryStatusEx(ctypes.byref(stat))
+    return stat.ullTotalPhys
+
+
+def get_darwin_memory():
+    """ Use system-call to extract total memory on macOS """
+    system_output = sabnzbd.newsunpack.run_simple(['sysctl', 'hw.memsize'])
+    return float(system_output.split()[1])
+
+
 ##############################################################################
-# Locked directory operations
+# Locked directory operations to avoid problems with simultaneous add/remove
 ##############################################################################
 DIR_LOCK = threading.RLock()
 
-
 @synchronized(DIR_LOCK)
 def get_unique_path(dirpath, n=0, create_dir=True):
     """ Determine a unique folder or filename """
@@ -788,14 +971,16 @@ def get_unique_path(dirpath, n=0, create_dir=True):
 
 @synchronized(DIR_LOCK)
 def get_unique_filename(path):
-    """ Check if path is unique. If not, add number like: "/path/name.NUM.ext". """
+    """ Check if path is unique.
+        If not, add number like: "/path/name.NUM.ext".
+    """
     num = 1
+    new_path, fname = os.path.split(path)
+    name, ext = os.path.splitext(fname)
     while os.path.exists(path):
-        path, fname = os.path.split(path)
-        name, ext = os.path.splitext(fname)
         fname = "%s.%d%s" % (name, num, ext)
         num += 1
-        path = os.path.join(path, fname)
+        path = os.path.join(new_path, fname)
     return path
 
 
@@ -820,26 +1005,30 @@ def move_to_path(path, new_path):
     new_path = os.path.abspath(new_path)
     if overwrite and os.path.exists(new_path):
         try:
-            os.remove(new_path)
+            remove_file(new_path)
         except:
             overwrite = False
     if not overwrite:
         new_path = get_unique_filename(new_path)
 
     if new_path:
-        logging.debug("Moving. Old path:%s new path:%s overwrite?:%s",
-                                                  path, new_path, overwrite)
+        logging.debug("Moving (overwrite: %s) %s => %s", overwrite, path, new_path)
         try:
             # First try cheap rename
             renamer(path, new_path)
         except:
             # Cannot rename, try copying
+            logging.debug("File could not be renamed, trying copying: %s", path)
             try:
                 if not os.path.exists(os.path.dirname(new_path)):
                     create_dirs(os.path.dirname(new_path))
                 shutil.copyfile(path, new_path)
-                os.remove(path)
+                remove_file(path)
             except:
+                # Check if the old-file actually exists (possible delete-delays)
+                if not os.path.exists(path):
+                    logging.debug("File not moved, original path gone: %s", path)
+                    return True, None
                 if not (cfg.marker_file() and cfg.marker_file() in path):
                     logging.error(T('Failed moving %s to %s'), clip_path(path), clip_path(new_path))
                     logging.info("Traceback: ", exc_info=True)
@@ -875,11 +1064,8 @@ def get_filepath(path, nzo, filename):
     # It does no umask setting
     # It uses the dir_lock for the (rare) case that the
     # download_dir is equal to the complete_dir.
-    dirname = nzo.work_name
-    created = nzo.created
-
-    dName = dirname
-    if not created:
+    dName = nzo.work_name
+    if not nzo.created:
         for n in xrange(200):
             dName = dirname
             if n:
@@ -908,6 +1094,103 @@ def get_filepath(path, nzo, filename):
     return fullPath
 
 
+@synchronized(DIR_LOCK)
+def renamer(old, new):
+    """ Rename file/folder with retries for Win32 """
+    # Sanitize last part of new name
+    path, name = os.path.split(new)
+    # Use the more stringent folder rename to end up with a nicer name,
+    # but do not trim size
+    new = os.path.join(path, sanitize_foldername(name, False))
+
+    logging.debug('Renaming "%s" to "%s"', old, new)
+    if sabnzbd.WIN32:
+        retries = 15
+        while retries > 0:
+            # First we try 3 times with os.rename
+            if retries > 12:
+                try:
+                    os.rename(old, new)
+                    return
+                except:
+                    retries -= 1
+                    time.sleep(3)
+                    continue
+
+            # Now we try the back-up method
+            logging.debug('Could not rename, trying move for %s to %s', old, new)
+            try:
+                shutil.move(old, new)
+                return
+            except WindowsError, err:
+                logging.debug('Error renaming "%s" to "%s" <%s>', old, new, err)
+                if err[0] == 32:
+                    logging.debug('Retry rename %s to %s', old, new)
+                    retries -= 1
+                else:
+                    raise WindowsError(err)
+            time.sleep(3)
+        raise WindowsError(err)
+    else:
+        shutil.move(old, new)
+
+
+@synchronized(DIR_LOCK)
+def remove_dir(path):
+    """ Remove directory with retries for Win32 """
+    if sabnzbd.WIN32:
+        retries = 15
+        while retries > 0:
+            try:
+                remove_dir(path)
+                return
+            except WindowsError, err:
+                if err[0] == 32:
+                    logging.debug('Retry delete %s', path)
+                    retries -= 1
+                else:
+                    raise WindowsError(err)
+            time.sleep(3)
+        raise WindowsError(err)
+    else:
+        remove_dir(path)
+
+
+@synchronized(DIR_LOCK)
+def remove_all(path, pattern='*', keep_folder=False, recursive=False):
+    """ Remove folder and all its content (optionally recursive) """
+    if os.path.exists(path):
+        files = globber_full(path, pattern)
+        if pattern == '*' and not sabnzbd.WIN32:
+            files.extend(globber_full(path, '.*'))
+
+        for f in files:
+            if os.path.isfile(f):
+                try:
+                    remove_file(f)
+                except:
+                    logging.info('Cannot remove file %s', f)
+            elif recursive:
+                remove_all(f, pattern, False, True)
+        if not keep_folder:
+            try:
+                remove_dir(path)
+            except:
+                logging.info('Cannot remove folder %s', path)
+
+
+def remove_file(path):
+    """ Wrapper function so any file removal is logged """
+    logging.debug('[%s] Deleting file %s', caller_name(), path)
+    os.remove(path)
+
+
+def remove_dir(dir):
+    """ Wrapper function so any dir removal is logged """
+    logging.debug('[%s] Deleting dir %s', caller_name(), dir)
+    os.rmdir(dir)
+
+
 def trim_win_path(path):
     """ Make sure Windows path stays below 70 by trimming last part """
     if sabnzbd.WIN32 and len(path) > 69:
@@ -919,15 +1202,6 @@ def trim_win_path(path):
     return path
 
 
-def check_win_maxpath(folder):
-    """ Return False if any file path in folder exceeds the Windows maximum """
-    if sabnzbd.WIN32:
-        for p in os.listdir(folder):
-            if len(os.path.join(folder, p)) > 259:
-                return False
-    return True
-
-
 def make_script_path(script):
     """ Return full script path, if any valid script exists, else None """
     s_path = None
@@ -1074,83 +1348,91 @@ if sabnzbd.WIN32:
     except:
         pass
 
-    def diskfree(_dir):
-        """ Return amount of free diskspace in GBytes """
+    def diskspace_base(_dir):
+        """ Return amount of free and used diskspace in GBytes """
         _dir = find_dir(_dir)
         try:
             available, disk_size, total_free = win32api.GetDiskFreeSpaceEx(_dir)
-            return available / GIGI
+            return disk_size / GIGI, available / GIGI
         except:
-            return 0.0
+            return 0.0, 0.0
 
-    def disktotal(_dir):
-        """ Return amount of free diskspace in GBytes """
-        _dir = find_dir(_dir)
-        try:
-            available, disk_size, total_free = win32api.GetDiskFreeSpaceEx(_dir)
-            return disk_size / GIGI
-        except:
-            return 0.0
 else:
     try:
         os.statvfs
         # posix diskfree
-
-        def diskfree(_dir):
-            """ Return amount of free diskspace in GBytes """
-            _dir = find_dir(_dir)
-            try:
-                s = os.statvfs(_dir)
-                if s.f_bavail < 0:
-                    return float(sys.maxint) * float(s.f_frsize) / GIGI
-                else:
-                    return float(s.f_bavail) * float(s.f_frsize) / GIGI
-            except OSError:
-                return 0.0
-
-        def disktotal(_dir):
-            """ Return amount of total diskspace in GBytes """
+        def diskspace_base(_dir):
+            """ Return amount of free and used diskspace in GBytes """
             _dir = find_dir(_dir)
             try:
                 s = os.statvfs(_dir)
                 if s.f_blocks < 0:
-                    return float(sys.maxint) * float(s.f_frsize) / GIGI
+                    disk_size = float(sys.maxint) * float(s.f_frsize)
                 else:
-                    return float(s.f_blocks) * float(s.f_frsize) / GIGI
-            except OSError:
-                return 0.0
+                    disk_size = float(s.f_blocks) * float(s.f_frsize)
+                if s.f_bavail < 0:
+                    available = float(sys.maxint) * float(s.f_frsize)
+                else:
+                    available = float(s.f_bavail) * float(s.f_frsize)
+                return disk_size / GIGI, available / GIGI
+            except:
+                return 0.0, 0.0
     except ImportError:
-        def diskfree(_dir):
-            return 10.0
-
-        def disktotal(_dir):
-            return 20.0
+        def diskspace_base(_dir):
+            return 20.0, 10.0
 
 
+# Store all results to speed things up
+__DIRS_CHECKED = []
+__DISKS_SAME = None
+__LAST_DISK_RESULT = {'download_dir': [], 'complete_dir': []}
+__LAST_DISK_CALL = 0
+
+def diskspace(force=False):
+    """ Wrapper to cache results """
+    global __DIRS_CHECKED, __DISKS_SAME, __LAST_DISK_RESULT, __LAST_DISK_CALL
+
+    # Reset everything when folders changed
+    dirs_to_check = [cfg.download_dir.get_path(), cfg.complete_dir.get_path()]
+    if __DIRS_CHECKED != dirs_to_check:
+        __DIRS_CHECKED = dirs_to_check
+        __DISKS_SAME = None
+        __LAST_DISK_RESULT = {'download_dir': [], 'complete_dir': []}
+        __LAST_DISK_CALL = 0
+
+    # When forced, ignore any cache to avoid problems in UI
+    if force:
+        __LAST_DISK_CALL = 0
+
+    # Check against cache
+    if time.time() > __LAST_DISK_CALL + 10.0:
+        # Same disk? Then copy-paste
+        __LAST_DISK_RESULT['download_dir'] = diskspace_base(cfg.download_dir.get_path())
+        __LAST_DISK_RESULT['complete_dir'] = __LAST_DISK_RESULT['download_dir'] if __DISKS_SAME else diskspace_base(cfg.complete_dir.get_path())
+        __LAST_DISK_CALL = time.time()
+
+    # Do we know if it's same disk?
+    if __DISKS_SAME is None:
+        __DISKS_SAME = (__LAST_DISK_RESULT['download_dir'] == __LAST_DISK_RESULT['complete_dir'])
+
+    return __LAST_DISK_RESULT
+
+
+##############################################################################
+# Other support functions
+##############################################################################
 def create_https_certificates(ssl_cert, ssl_key):
     """ Create self-signed HTTPS certificates and store in paths 'ssl_cert' and 'ssl_key' """
-    try:
-        from OpenSSL import crypto
-        from sabnzbd.utils.certgen import createKeyPair, createCertRequest, createCertificate, \
-             TYPE_RSA, serial
-    except:
-        logging.warning(T('pyopenssl module missing, please install for https access'))
+    if not sabnzbd.HAVE_CRYPTOGRAPHY:
+        logging.error(T('%s missing'), 'Python Cryptography')
         return False
 
-    # Create the CA Certificate
-    cakey = createKeyPair(TYPE_RSA, 2048)
-    careq = createCertRequest(cakey, digest='sha256', CN='Certificate Authority')
-    cacert = createCertificate(careq, (careq, cakey), serial, (0, 60 * 60 * 24 * 365 * 10), digest='sha256')  # ten years
-
-    cname = 'SABnzbd'
-    pkey = createKeyPair(TYPE_RSA, 2048)
-    req = createCertRequest(pkey, digest='sha256', CN=cname)
-    cert = createCertificate(req, (cacert, cakey), serial, (0, 60 * 60 * 24 * 365 * 10), digest='sha256')  # ten years
-
     # Save the key and certificate to disk
     try:
-        open(ssl_key, 'w').write(crypto.dump_privatekey(crypto.FILETYPE_PEM, pkey))
-        open(ssl_cert, 'w').write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
+        from sabnzbd.utils.certgen import generate_key, generate_local_cert
+        private_key = generate_key(key_size=2048, output_file=ssl_key)
+        generate_local_cert(private_key, days_valid=3560, output_file=ssl_cert, LN=u'SABnzbd', ON=u'SABnzbd', CN=u'localhost')
+        logging.info('Self-signed certificates generated successfully')
     except:
         logging.error(T('Error creating SSL key and certificate'))
         logging.info("Traceback: ", exc_info=True)
@@ -1159,6 +1441,54 @@ def create_https_certificates(ssl_cert, ssl_key):
     return True
 
 
+def get_all_passwords(nzo):
+    """ Get all passwords, from the NZB, meta and password file """
+    if nzo.password:
+        logging.info('Found a password that was set by the user: %s', nzo.password)
+        passwords = [nzo.password.strip()]
+    else:
+        passwords = []
+
+    meta_passwords = nzo.meta.get('password', [])
+    pw = nzo.nzo_info.get('password')
+    if pw:
+        meta_passwords.append(pw)
+
+    if meta_passwords:
+        if nzo.password == meta_passwords[0]:
+            # this nzo.password came from meta, so don't use it twice
+            passwords.extend(meta_passwords[1:])
+        else:
+            passwords.extend(meta_passwords)
+        logging.info('Read %s passwords from meta data in NZB: %s', len(meta_passwords), meta_passwords)
+
+    pw_file = cfg.password_file.get_path()
+    if pw_file:
+        try:
+            with open(pw_file, 'r') as pwf:
+                lines = pwf.read().split('\n')
+            # Remove empty lines and space-only passwords and remove surrounding spaces
+            pws = [pw.strip('\r\n ') for pw in lines if pw.strip('\r\n ')]
+            logging.debug('Read these passwords from file: %s', pws)
+            passwords.extend(pws)
+            logging.info('Read %s passwords from file %s', len(pws), pw_file)
+
+            # Check size
+            if len(pws) > 30:
+                logging.warning(T('Your password file contains more than 30 passwords, testing all these passwords takes a lot of time. Try to only list useful passwords.'))
+        except:
+            logging.warning('Failed to read the passwords file %s', pw_file)
+
+    if nzo.password:
+        # If an explicit password was set, add a retry without password, just in case.
+        passwords.append('')
+    elif not passwords or nzo.encrypted < 1:
+        # If we're not sure about encryption, start with empty password
+        # and make sure we have at least the empty password
+        passwords.insert(0, '')
+    return passwords
+
+
 def find_on_path(targets):
     """ Search the PATH for a program and return full path """
     if sabnzbd.WIN32:
@@ -1177,8 +1507,6 @@ def find_on_path(targets):
     return None
 
 
-_RE_IP4 = re.compile(r'inet\s+(addr:\s*){0,1}(\d+\.\d+\.\d+\.\d+)')
-_RE_IP6 = re.compile(r'inet6\s+(addr:\s*){0,1}([0-9a-f:]+)', re.I)
 def ip_extract():
     """ Return list of IP addresses of this system """
     ips = []
@@ -1205,84 +1533,14 @@ def ip_extract():
         output = p.stdout.read()
         p.wait()
         for line in output.split('\n'):
-            m = _RE_IP4.search(line)
+            m = RE_IP4.search(line)
             if not (m and m.group(2)):
-                m = _RE_IP6.search(line)
+                m = RE_IP6.search(line)
             if m and m.group(2):
                 ips.append(m.group(2))
     return ips
 
 
-def renamer(old, new):
-    """ Rename file/folder with retries for Win32 """
-    # Sanitize last part of new name
-    path, name = os.path.split(new)
-    # Use the more stringent folder rename to end up with a nicer name,
-    # but do not trim size
-    new = os.path.join(path, sanitize_foldername(name, False))
-
-    logging.debug('Renaming "%s" to "%s"', old, new)
-    if sabnzbd.WIN32:
-        retries = 15
-        while retries > 0:
-            try:
-                shutil.move(old, new)
-                return
-            except WindowsError, err:
-                logging.debug('Error renaming "%s" to "%s" <%s>', old, new, err)
-                if err[0] == 32:
-                    logging.debug('Retry rename %s to %s', old, new)
-                    retries -= 1
-                else:
-                    raise WindowsError(err)
-            time.sleep(3)
-        raise WindowsError(err)
-    else:
-        shutil.move(old, new)
-
-
-def remove_dir(path):
-    """ Remove directory with retries for Win32 """
-    if sabnzbd.WIN32:
-        retries = 15
-        while retries > 0:
-            try:
-                os.rmdir(path)
-                return
-            except WindowsError, err:
-                if err[0] == 32:
-                    logging.debug('Retry delete %s', path)
-                    retries -= 1
-                else:
-                    raise WindowsError(err)
-            time.sleep(3)
-        raise WindowsError(err)
-    else:
-        os.rmdir(path)
-
-
-def remove_all(path, pattern='*', keep_folder=False, recursive=False):
-    """ Remove folder and all its content (optionally recursive) """
-    if os.path.exists(path):
-        files = globber_full(path, pattern)
-        if pattern == '*' and not sabnzbd.WIN32:
-            files.extend(globber_full(path, '.*'))
-
-        for f in files:
-            if os.path.isfile(f):
-                try:
-                    os.remove(f)
-                except:
-                    logging.info('Cannot remove file %s', f)
-            elif recursive:
-                remove_all(f, pattern, False, True)
-        if not keep_folder:
-            try:
-                os.rmdir(path)
-            except:
-                logging.info('Cannot remove folder %s', path)
-
-
 def is_writable(path):
     """ Return True is file is writable (also when non-existent) """
     if os.path.isfile(path):
@@ -1291,15 +1549,6 @@ def is_writable(path):
         return True
 
 
-def format_source_url(url):
-    """ Format URL suitable for 'Source' stage """
-    if _HAVE_SSL:
-        prot = 'https'
-    else:
-        prot = 'http:'
-    return url
-
-
 def get_base_url(url):
     """ Return only the true root domain for the favicon, so api.oznzb.com -> oznzb.com
         But also api.althub.co.za -> althub.co.za
@@ -1338,6 +1587,7 @@ def starts_with_path(path, prefix):
 def set_chmod(path, permissions, report):
     """ Set 'permissions' on 'path', report any errors when 'report' is True """
     try:
+        logging.debug('Applying permissions %s (octal) to %s', oct(permissions), path)
         os.chmod(path, permissions)
     except:
         lpath = path.lower()
@@ -1377,31 +1627,10 @@ def set_permissions(path, recursive=True):
             set_chmod(path, umask_file, report)
 
 
-def short_path(path, always=True):
-    """ For Windows, return shortened ASCII path, for others same path
-        When `always` is off, only return a short path when size is above 259
-    """
-    if sabnzbd.WIN32:
-        import win32api
-        path = os.path.normpath(path)
-        if always or len(path) > 259:
-            # First make the path "long"
-            path = long_path(path)
-            if os.path.exists(path):
-                # Existing path can always be shortened
-                path = win32api.GetShortPathName(path)
-            else:
-                # For new path, shorten only existing part (recursive)
-                path1, name = os.path.split(path)
-                path = os.path.join(short_path(path1, always), name)
-        path = clip_path(path)
-    return path
-
-
 def clip_path(path):
     r""" Remove \\?\ or \\?\UNC\ prefix from Windows path """
     if sabnzbd.WIN32 and path and '?' in path:
-        path = path.replace(u'\\\\?\\UNC\\', u'\\\\').replace(u'\\\\?\\', u'')
+        path = path.replace(u'\\\\?\\UNC\\', u'\\\\', 1).replace(u'\\\\?\\', u'', 1)
     return path
 
 
@@ -1434,3 +1663,11 @@ def get_urlbase(url):
     """ Return the base URL (like http://server.domain.com/) """
     parsed_uri = urlparse(url)
     return '{uri.scheme}://{uri.netloc}/'.format(uri=parsed_uri)
+
+
+def nntp_to_msg(text):
+    """ Format raw NNTP data for display """
+    if isinstance(text, list):
+        text = text[0]
+    lines = text.split('\r\n')
+    return lines[0]
