@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@ sabnzbd.panic - Send panic message to the browser
 import os
 import logging
 import tempfile
+import ctypes
 try:
     import webbrowser
 except ImportError:
@@ -29,14 +30,12 @@ except ImportError:
 
 import sabnzbd
 import sabnzbd.cfg as cfg
+from sabnzbd.encoding import unicoder
 
-PANIC_NONE = 0
 PANIC_PORT = 1
 PANIC_TEMPL = 2
 PANIC_QUEUE = 3
-PANIC_FWALL = 4
 PANIC_OTHER = 5
-PANIC_XPORT = 6
 PANIC_SQLITE = 7
 PANIC_HOST = 8
 
@@ -60,16 +59,6 @@ def MSG_BAD_NEWS():
 '''
 
 
-def MSG_BAD_FWALL():
-    return Ta(r'''
-    SABnzbd is not compatible with some software firewalls.<br>
-    %s<br>
-    Sorry, but we cannot solve this incompatibility right now.<br>
-    Please file a complaint at your firewall supplier.<br>
-    <br>
-''')
-
-
 def MSG_BAD_PORT():
     return Ta(r'''
     SABnzbd needs a free tcp/ip port for its internal web server.<br>
@@ -85,21 +74,6 @@ def MSG_BAD_PORT():
         Ta(r'If you get this error message again, please try a different number.<br>')
 
 
-def MSG_ILL_PORT():
-    return Ta(r'''
-    SABnzbd needs a free tcp/ip port for its internal web server.<br>
-    Port %s on %s was tried , but the account used for SABnzbd has no permission to use it.<br>
-    On OSX and Linux systems, normal users must use ports above 1023.<br>
-    <br>
-    Please restart SABnzbd with a different port number.''') + \
-        '''<br>
-    <br>
-    %s<br>
-      &nbsp;&nbsp;&nbsp;&nbsp;%s --server %s:%s<br>
-    <br>''' + \
-        Ta(r'If you get this error message again, please try a different number.<br>')
-
-
 def MSG_BAD_HOST():
     return Ta(r'''
     SABnzbd needs a valid host address for its internal web server.<br>
@@ -143,15 +117,6 @@ def MSG_OTHER():
     return T('SABnzbd detected a fatal error:') + '<br>%s<br><br>%s<br>'
 
 
-def MSG_OLD_QUEUE():
-    return Ta(r'''
-    SABnzbd detected a queue from an older release.<br><br>
-    You can convert the queue by clicking "Repair" in Status-&gt;"Queue Repair".<br><br>
-    You may choose to stop SABnzbd and finish the queue with the older program.<br><br>
-    Click OK to proceed to SABnzbd''') + \
-        ('''<br><br><FORM><input type="button" onclick="this.form.action='/.'; this.form.submit(); return false;" value="%s"/></FORM>''' % T('OK'))
-
-
 def MSG_SQLITE():
     return Ta(r'''
     SABnzbd detected that the file sqlite3.dll is missing.<br><br>
@@ -174,22 +139,10 @@ def panic_message(panic, a=None, b=None):
         newport = int(b) + 1
         newport = "%s" % newport
         msg = MSG_BAD_PORT() % (b, a, os_str, prog_path, a, newport)
-    elif panic == PANIC_XPORT:
-        if int(b) < 1023:
-            newport = 1024
-        else:
-            newport = int(b) + 1
-        newport = "%s" % newport
-        msg = MSG_ILL_PORT() % (b, a, os_str, prog_path, a, newport)
     elif panic == PANIC_TEMPL:
         msg = MSG_BAD_TEMPL() % a
     elif panic == PANIC_QUEUE:
         msg = MSG_BAD_QUEUE() % (a, os_str, prog_path)
-    elif panic == PANIC_FWALL:
-        if a:
-            msg = MSG_BAD_FWALL() % T('It is likely that you are using ZoneAlarm on Vista.<br>')
-        else:
-            msg = MSG_BAD_FWALL() % "<br>"
     elif panic == PANIC_SQLITE:
         msg = MSG_SQLITE()
     elif panic == PANIC_HOST:
@@ -212,11 +165,8 @@ def panic_message(panic, a=None, b=None):
     return url
 
 
-def panic_fwall(vista):
-    launch_a_browser(panic_message(PANIC_FWALL, vista))
-
-
 def panic_port(host, port):
+    show_error_dialog("\n%s:\n  %s" % (T('Fatal error'), T('Unable to bind to port %s on %s. Some other software uses the port or SABnzbd is already running.') % (port, host)))
     launch_a_browser(panic_message(PANIC_PORT, host, port))
 
 
@@ -224,11 +174,6 @@ def panic_host(host, port):
     launch_a_browser(panic_message(PANIC_HOST, host, port))
 
 
-def panic_xport(host, port):
-    launch_a_browser(panic_message(PANIC_XPORT, host, port))
-    logging.error(T('You have no permission to use port %s'), port)
-
-
 def panic_queue(name):
     launch_a_browser(panic_message(PANIC_QUEUE, name, 0))
 
@@ -241,13 +186,8 @@ def panic_sqlite(name):
     launch_a_browser(panic_message(PANIC_SQLITE, name, 0))
 
 
-def panic_old_queue():
-    msg = MSG_OLD_QUEUE()
-    return MSG_BAD_NEWS() % (sabnzbd.MY_NAME, sabnzbd.__version__, sabnzbd.MY_NAME, sabnzbd.__version__, msg, '')
-
-
 def panic(reason, remedy=""):
-    print "\n%s:\n  %s\n%s" % (T('Fatal error'), reason, remedy)
+    show_error_dialog("\n%s:\n  %s\n%s" % (T('Fatal error'), reason, remedy))
     launch_a_browser(panic_message(PANIC_OTHER, reason, remedy))
 
 
@@ -279,6 +219,15 @@ def launch_a_browser(url, force=False):
         logging.info("Traceback: ", exc_info=True)
 
 
+def show_error_dialog(msg):
+    """ Show a pop-up when program cannot start
+        Windows-only, otherwise only print to console
+    """
+    if sabnzbd.WIN32:
+        ctypes.windll.user32.MessageBoxW(0, unicoder(msg), T('Fatal error'), 0)
+    print msg
+
+
 def error_page_401(status, message, traceback, version):
     """ Custom handler for 401 error """
     title = T('Access denied')
@@ -303,10 +252,10 @@ def error_page_404(status, message, traceback, version):
     <head>
       <script type="text/javascript">
       <!--
-      location.href = location.protocol + '//' + location.hostname + (location.port ? ':' + location.port : '') + '/sabnzbd/' ;
+      location.href = location.protocol + '//' + location.hostname + (location.port ? ':' + location.port : '') + '%s' ;
       //-->
       </script>
     </head>
     <body><br/></body>
 </html>
-'''
+''' % cfg.url_base()
