@@ -26,18 +26,23 @@ import time
 import logging
 import copy
 import socket
-try:
-    socket.ssl
-    _HAVE_SSL = True
-except:
-    _HAVE_SSL = False
+import Queue
+import collections
 from threading import RLock, Thread
-
 import sabnzbd
 from sabnzbd.decorators import synchronized
-from sabnzbd.utils.ordered import OrderedSetQueue
 import sabnzbd.cfg as cfg
 
+# A queue which ignores duplicates but maintains ordering
+class OrderedSetQueue(Queue.Queue):
+    def _init(self, maxsize):
+        self.maxsize = maxsize
+        self.queue = collections.OrderedDict()
+    def _put(self, item):
+        self.queue[item] = None
+    def _get(self):
+        return self.queue.popitem()[0]
+
 _RATING_URL = "/releaseRatings/releaseRatings.php"
 RATING_LOCK = RLock()
 
@@ -128,9 +133,6 @@ class Rating(Thread):
             self.ratings = {}
             self.nzo_indexer_map = {}
         Thread.__init__(self)
-        if not _HAVE_SSL:
-            logging.warning(T('Ratings server requires secure connection'))
-            self.stop()
 
     def stop(self):
         self.shutdown = True
@@ -231,7 +233,7 @@ class Rating(Thread):
 
     @synchronized(RATING_LOCK)
     def update_auto_flag(self, nzo_id, flag, flag_detail=None):
-        if not flag or not cfg.rating_enable() or not cfg.rating_feedback() or (nzo_id not in self.nzo_indexer_map):
+        if not flag or not cfg.rating_enable() or (nzo_id not in self.nzo_indexer_map):
             return
         logging.debug('Updating auto flag (%s: %s)', nzo_id, flag)
         indexer_id = self.nzo_indexer_map[nzo_id]
@@ -269,8 +271,6 @@ class Rating(Thread):
         api_key = cfg.rating_api_key()
         rating_host = cfg.rating_host()
         rating_url = _RATING_URL
-        if not api_key:
-            return True
 
         requests = []
         _headers = {'User-agent': 'SABnzbd+/%s' % sabnzbd.version.__version__, 'Content-type': 'application/x-www-form-urlencoded'}
@@ -281,8 +281,15 @@ class Rating(Thread):
             # Is it an URL or just a HOST?
             if host_parsed.path and host_parsed.path != '/':
                 rating_url = host_parsed.path + '?' + host_parsed.query if host_parsed.query else host_parsed.path
+
         if not rating_host:
+            _warn('%s: %s' % (T('Cannot send, missing required data'), T('Server address')))
             return True
+
+        if not api_key:
+            _warn('%s [%s]: %s - %s' % (T('Cannot send, missing required data'), rating_host, T('API Key'), T('This key provides identity to indexer. Check your profile on the indexer\'s website.')))
+            return True
+
         if rating.changed & Rating.CHANGED_USER_VIDEO:
             requests.append({'m': 'r', 'r': 'videoQuality', 'rn': rating.user_video})
         if rating.changed & Rating.CHANGED_USER_AUDIO:
