@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -78,8 +78,8 @@ class StatusIcon(Thread):
     # run this every updatefreq ms
     def run(self):
         self.sabpaused, bytes_left, bpsnow, time_left = api.fast_queue()
-        mb_left = to_units(bytes_left, dec_limit=1)
-        speed = to_units(bpsnow, dec_limit=1)
+        mb_left = to_units(bytes_left)
+        speed = to_units(bpsnow)
 
         if self.sabpaused:
             self.tooltip = T('Paused')
@@ -165,8 +165,7 @@ class StatusIcon(Thread):
 
     def restart(self, icon):
         self.hover_text = T('Restart')
-        sabnzbd.halt()
-        cherrypy.engine.restart()
+        sabnzbd.trigger_restart()
 
     def shutdown(self, icon):
         self.hover_text = T('Shutdown')
