@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -36,7 +36,6 @@ from sabnzbd.constants import LOW_PRIORITY, NORMAL_PRIORITY, HIGH_PRIORITY
 
 __SCHED = None  # Global pointer to Scheduler instance
 
-RSSTASK_MINUTE = random.randint(0, 59)
 SCHEDULE_GUARD_FLAG = False
 PP_PAUSE_EVENT = False
 
@@ -70,10 +69,15 @@ def init():
     for schedule in cfg.schedules():
         arguments = []
         argument_list = None
+
         try:
-            m, h, d, action_name = schedule.split()
+            enabled, m, h, d, action_name = schedule.split()
         except:
-            m, h, d, action_name, argument_list = schedule.split(None, 4)
+            try:
+                enabled, m, h, d, action_name, argument_list = schedule.split(None, 5)
+            except:
+                continue  # Bad schedule, ignore
+
         if argument_list:
             arguments = argument_list.split()
 
@@ -148,14 +152,22 @@ def init():
         elif action_name == 'resume_all_high':
             action = sabnzbd.nzbqueue.NzbQueue.do.resume_on_prio
             arguments = [HIGH_PRIORITY]
+        elif action_name == 'pause_cat':
+            action = sabnzbd.nzbqueue.NzbQueue.do.pause_on_cat
+            arguments = [argument_list]
+        elif action_name == 'resume_cat':
+            action = sabnzbd.nzbqueue.NzbQueue.do.resume_on_cat
+            arguments = [argument_list]
         else:
             logging.warning(T('Unknown action: %s'), action_name)
             continue
 
-        logging.debug("scheduling %s(%s) on days %s at %02d:%02d", action_name, arguments, d, h, m)
-
-        __SCHED.add_daytime_task(action, action_name, d, None, (h, m),
-                             kronos.method.sequential, arguments, None)
+        if enabled == '1':
+            logging.debug("Scheduling %s(%s) on days %s at %02d:%02d", action_name, arguments, d, h, m)
+            __SCHED.add_daytime_task(action, action_name, d, None, (h, m),
+                                 kronos.method.sequential, arguments, None)
+        else:
+            logging.debug("Skipping %s(%s) on days %s at %02d:%02d", action_name, arguments, d, h, m)
 
     # Set Guardian interval to 30 seconds
     __SCHED.add_interval_task(sched_guardian, "Guardian", 15, 30,
@@ -187,6 +199,11 @@ def init():
         __SCHED.add_daytime_task(action, 'quota_reset', range(1, 8), None, (hour, minute),
                                  kronos.method.sequential, [], None)
 
+    if sabnzbd.misc.int_conv(cfg.history_retention()) > 0:
+        logging.info('Setting schedule for midnight auto history-purge')
+        __SCHED.add_daytime_task(sabnzbd.database.midnight_history_purge, 'midnight_history_purge', range(1, 8), None, (0, 0),
+                                kronos.method.sequential, [], None)
+
     logging.info('Setting schedule for midnight BPS reset')
     __SCHED.add_daytime_task(sabnzbd.bpsmeter.midnight_action, 'midnight_bps', range(1, 8), None, (0, 0),
                              kronos.method.sequential, [], None)
@@ -259,10 +276,11 @@ def sort_schedules(all_events, now=None):
     for schedule in cfg.schedules():
         parms = None
         try:
-            m, h, dd, action, parms = schedule.split(None, 4)
+            # Note: the last parameter can have spaces (category name)!
+            enabled, m, h, dd, action, parms = schedule.split(None, 5)
         except:
             try:
-                m, h, dd, action = schedule.split(None, 3)
+                enabled, m, h, dd, action = schedule.split(None, 4)
             except:
                 continue  # Bad schedule, ignore
         action = action.strip()
@@ -277,7 +295,7 @@ def sort_schedules(all_events, now=None):
                 # Expired event will occur again after a week
                 dif = dif + week_min
 
-            events.append((dif, action, parms, schedule))
+            events.append((dif, action, parms, schedule, enabled))
             if not all_events:
                 break
 
@@ -302,6 +320,11 @@ def analyse(was_paused=False, priority=None):
     for ev in sort_schedules(all_events=True):
         if priority is None:
             logging.debug('Schedule check result = %s', ev)
+
+        # Skip if disabled
+        if ev[4] == '0':
+            continue
+
         action = ev[1]
         try:
             value = ev[2]
