@@ -1,6 +1,6 @@
 #!/usr/bin/python -OO
 # -*- coding: UTF-8 -*-
-# Copyright 2012-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2012-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -73,6 +73,8 @@ SKIN_TEXT = {
     'sch-resume_all_high': TT('Resume high prioirty jobs'), #: Config->Scheduler
     'sch-enable_quota'   : TT('Enable quota management'), #: Config->Scheduler
     'sch-disable_quota'  : TT('Disable quota management'), #: Config->Scheduler
+    'sch-pause_cat'      : TT('Pause jobs with category'), #: Config->Scheduler
+    'sch-resume_cat'     : TT('Resume jobs with category'), #: Config->Scheduler
 
     'prowl-off'          : TT('Off'), #: Prowl priority
     'prowl-very-low'     : TT('Very Low'), #: Prowl priority
@@ -84,7 +86,6 @@ SKIN_TEXT = {
     'pushover-off'       : TT('Off'), #: Prowl priority
     'pushover-low'       : TT('Low'), #: Prowl priority
     'pushover-high'      : TT('High'), #: Prowl priority
-    'pushover-confirm'   : TT('Confirm'), #: Prowl priority
 
 # General texts
     'default' : TT('Default'), #: Default value, used in dropdown menus
@@ -103,6 +104,18 @@ SKIN_TEXT = {
     'week' : TT('week'),
     'month' : TT('Month'),
     'year' : TT('Year'),
+    'January': TT('January'),
+    'February': TT('February'),
+    'March': TT('March'),
+    'April': TT('April'),
+    'May': TT('May'),
+    'June': TT('June'),
+    'July': TT('July'),
+    'August': TT('August'),
+    'September': TT('September'),
+    'October': TT('October'),
+    'November': TT('November'),
+    'December': TT('December'),
     'monday' : TT('Monday'),
     'tuesday' : TT('Tuesday'),
     'wednesday' : TT('Wednesday'),
@@ -140,7 +153,6 @@ SKIN_TEXT = {
 # General template elements
     'signOn' : TT('The automatic usenet download tool'), #: SABnzbd's theme line
     'button-save' : TT('Save'), #: "Save" button
-    'queued' : TT('Queued'), #: "Queued" used to show amount of jobs
     'confirm' : TT('Are you sure?'), #: Used in confirmation popups
     'delFiles' : TT('Delete all downloaded files?'),  #: Used in confirmation popups
 
@@ -155,6 +167,7 @@ SKIN_TEXT = {
     'menu-forums' : TT('Forum'), #: Main menu item
     'menu-irc' : TT('IRC'), #: Main menu item
     'menu-issues' : TT('Issues'), #: Main menu item
+    'menu-donate' : TT('Support the project, Donate!'), #: Main menu item
     'cmenu-general' : TT('General'), #: Main menu item
     'cmenu-folders' : TT('Folders'), #: Main menu item
     'cmenu-switches' : TT('Switches'), #: Main menu item
@@ -166,13 +179,13 @@ SKIN_TEXT = {
     'cmenu-cat' : TT('Categories'), #: Main menu item
     'cmenu-sorting' : TT('Sorting'), #: Main menu item
     'cmenu-special' : TT('Special'), #: Main menu item
+    'cmenu-search' : TT('Search'), #: Main menu item
 
 # Footer
     'ft-download' : TT('Download Dir'), # Used in Footer
     'ft-paused' : TT('PAUSED'), # Used in Footer
     'ft-buffer@2' : TT('Cached %s articles (%s)'), # Used in Footer
     'ft-sysload' : TT('Sysload'), # Used in Footer
-    'ft-warning' : TT('WARNINGS'), # Used in Footer
     'ft-newRelease@1' : TT('New release %s available at'), # Used in Footer
 
 # Main page
@@ -203,8 +216,6 @@ SKIN_TEXT = {
     'enterURL' : TT('Enter URL'), #: Add NZB Dialog
 
 # Queue page
-    'link-hideFiles' : TT('Hide files'), #: Queue page button
-    'link-showFiles' : TT('Show files'), #: Queue page button
     'onQueueFinish' : TT('On queue finish'),  #: Queue page selection menu
     'shutdownPc' : TT('Shutdown PC'), #: Queue page end-of-queue action
     'standbyPc' : TT('Standby PC'), #: Queue page end-of-queue action
@@ -267,7 +278,6 @@ SKIN_TEXT = {
     'log-info' : TT('+ Info'), #: Status page logging selection value
     'log-debug' : TT('+ Debug'), #: Status page logging selection value
     'connections' : TT('Connections'), #: Status page tab header
-    'emailResult' : TT('Email Test Result'), #: Status page, title for email test result
     'lastWarnings' : TT('Latest Warnings'), #: Status page, table header
     'clearWarnings' : TT('clear'),  #: Status page button
     'server-blocked' : TT('Unblock'), #: Status page button
@@ -298,8 +308,9 @@ SKIN_TEXT = {
 # Configuration
     'confgFile' : TT('Config File'),
     'cache' : TT('Used cache'), #: Main config page, how much cache is in use
-    'explain-Restart' : TT('This will restart SABnzbd.<br />Use it when you think the program has a stability problem.<br />Downloading will be paused before the restart and resume afterwards.') + TT('<br />If authentication is enabled, you will need to login again.'),
+    'explain-Restart' : TT('This will restart SABnzbd.<br />Use it when you think the program has a stability problem.<br />Downloading will be paused before the restart and resume afterwards.'),
     'explain-needNewLogin' : TT('<br />If authentication is enabled, you will need to login again.'),
+    'button-advanced' : TT('Advanced'),
     'button-restart' : TT('Restart'),
     'explain-orphans' : TT('There are orphaned jobs in the download folder.<br />You can choose to delete them (including files) or send them back to the queue.'),
     'button-repair' : TT('Repair'),
@@ -309,9 +320,14 @@ SKIN_TEXT = {
     #'explain-Shutdown' : TT('This will end the SABnzbd process. <br />You will be unable to access SABnzbd and no downloading will take place until the service is started again.'),
     'opt-enable_unzip' : TT('Enable Unzip'),
     'opt-enable_7zip' : TT('Enable 7zip'),
+    'opt-multicore-par2' : TT('Multicore Par2'),
+    'explain-nosslcontext' : TT('Secure (SSL) connections from SABnzbd to newsservers and HTTPS websites will be encrypted, however, validating a server\'s identity using its certificates is not possible. Python 2.7.9 or above, OpenSSL 1.0.2 or above and up-to-date local CA certificates are required.'),
+    'explain-getpar2mt': TT('Speed up repairs by installing multicore Par2, it is available for many platforms.'),
     'version' : TT('Version'),
     'uptime' : TT('Uptime'),
     'backup' : TT('Backup'), #: Indicates that server is Backup server in Status page
+    'readwiki': TT('Read the Wiki Help on this!'),
+    'restarting-sab': TT('Restarting SABnzbd...'),
 
 # Config->General
     'restartRequired' : TT('Changes will require a SABnzbd restart!'),
@@ -322,13 +338,12 @@ SKIN_TEXT = {
     'explain-port' : TT('Port SABnzbd should listen on.'),
     'opt-web_dir' : TT('Web Interface'),
     'explain-web_dir' : TT('Choose a skin.'),
-    'opt-web_dir2' : TT('Secondary Web Interface'),
-    'explain-web_dir2' : TT('Activate an alternative skin.'),
     'opt-web_username' : TT('SABnzbd Username'),
     'explain-web_username' : TT('Optional authentication username.'),
     'opt-web_password' : TT('SABnzbd Password'),
     'explain-web_password' : TT('Optional authentication password.'),
-    'httpsSupport' : TT('HTTPS Support'),
+    'checkSafety' : TT('If the SABnzbd Host or Port is exposed to the internet, your current settings allow full external access to the SABnzbd interface.'),
+    'security' : TT('Security'),
     'opt-enable_https' : TT('Enable HTTPS'),
     'opt-notInstalled' : TT('not installed'),
     'explain-enable_https' : TT('Enable accessing the interface from a HTTPS address.'),
@@ -351,6 +366,13 @@ SKIN_TEXT = {
     'explain-cache_limitstr' : TT('Cache articles in memory to reduce disk access.<br /><i>In bytes, optionally follow with K,M,G. For example: "64M" or "128M"</i>'),
     'opt-cleanup_list' : TT('Cleanup List'),
     'explain-cleanup_list' : TT('List of file extensions that should be deleted after download.<br />For example: <b>nfo</b> or <b>nfo, sfv</b>'),
+    'opt-history_retention' : TT('History Retention'),
+    'explain-history_retention' : TT('Automatically delete completed jobs from History. Beware that Duplicate Detection and some external tools rely on History information.'),
+    'history_retention-all' : TT('Keep all jobs'),
+    'history_retention-number' : TT('Keep maximum number of completed jobs'),
+    'history_retention-days' : TT('Keep completed jobs maximum number of days'),
+    'history_retention-none' : TT('Do not keep any completed jobs'),
+    'history_retention-limit': TT('Jobs'),
     'button-saveChanges' : TT('Save Changes'),
     'button-restoreDefaults' : TT('Restore Defaults'),
     'explain-restoreDefaults' : TT('Reset'),
@@ -362,13 +384,11 @@ SKIN_TEXT = {
     'opt-nzbkey' : TT('NZB Key'),
     'explain-nzbkey' : TT('This key will allow 3rd party programs to add NZBs to SABnzbd.'),
     'button-apikey' : TT('Generate New Key'),
-    'opt-disableApikey' : TT('Disable API-key'),
-    'explain-disableApikey' : TT('Do not require the API key.'),
     'explain-qr-code' : TT('API Key QR Code'), #: Explanation for QR code of APIKEY
     'opt-local_ranges' : TT('List of local network ranges'),
     'explain-local_ranges' : TT('All local network addresses start with these prefixes (often "192.168.1.")'),
     'opt-inet_exposure' : TT('External internet access'),
-    'explain-inet_exposure' : TT('You can set access rights for systems outside your local network'),
+    'explain-inet_exposure' : TT('You can set access rights for systems outside your local network. Requires List of local network ranges to be defined.'),
     'inet-local' : TT('No access'), # Selection value for external access
     'inet-nzb' : TT('Add NZB files '), # Selection value for external access
     'inet-api' : TT('API (no Config)'), # Selection value for external access
@@ -393,8 +413,8 @@ SKIN_TEXT = {
     'explain-dirscan_dir' : TT('Folder to monitor for .nzb files.<br /><i>Also scans .zip .rar and .tar.gz archives for .nzb files.</i>'),
     'opt-dirscan_speed' : TT('Watched Folder Scan Speed'),
     'explain-dirscan_speed' : TT('Number of seconds between scans for .nzb files.'),
-    'opt-script_dir' : TT('Post-Processing Scripts Folder'),
-    'explain-script_dir' : TT('Folder containing user scripts for post-processing.'),
+    'opt-script_dir' : TT('Scripts Folder'),
+    'explain-script_dir' : TT('Folder containing user scripts.'),
     'opt-email_dir' : TT('Email Templates Folder'),
     'explain-email_dir' : TT('Folder containing user-defined email templates.'),
     'opt-password_file' : TT('Password file'),
@@ -410,16 +430,12 @@ SKIN_TEXT = {
     'base-folder' : TT('Default Base Folder'),
 
 # Config->Switches
-    'opt-quick_check' : TT('Enable Quick Check'),
-    'explain-quick_check' : TT('Skip par2 checking when files are 100% valid.'),
     'opt-enable_all_par' : TT('Download all par2 files'),
-    'explain-enable_all_par' : TT('This prevents multiple repair runs. QuickCheck on: download all par2 files when needed. QuickCheck off: always download all par2 files.'),
+    'explain-enable_all_par' : TT('This prevents multiple repair runs by downloading all par2 files when needed.'),
     'opt-enable_recursive' : TT('Enable recursive unpacking'),
     'explain-enable_recursive' : TT('Unpack archives (rar, zip, 7z) within archives.'),
     'opt-flat_unpack' : TT('Ignore any folders inside archives'),
     'explain-flat_unpack' : TT('All files will go into a single folder.'),
-    'opt-overwrite_files' : TT('When unpacking, overwrite existing files'),
-    'explain-overwrite_files' : TT('This will overwrite existing files instead of creating an alternative name.'),
     'opt-top_only' : TT('Only Get Articles for Top of Queue'),
     'explain-top_only' : TT('Enable for less memory usage. Disable to prevent slow jobs from blocking the queue.'),
     'opt-safe_postproc' : TT('Post-Process Only Verified Jobs'),
@@ -427,12 +443,16 @@ SKIN_TEXT = {
     'opt-pause_on_pwrar' : TT('Action when encrypted RAR is downloaded'),
     'explain-pause_on_pwrar' : TT('In case of "Pause", you\'ll need to set a password and resume the job.'),
     'opt-no_dupes' : TT('Detect Duplicate Downloads'),
-    'explain-no_dupes' : TT('Detect identical NZB files (based on NZB content)'),
+    'explain-no_dupes' : TT('Detect identical NZB files (based on items in your History or files in .nzb Backup Folder)'),
     'opt-no_series_dupes' : TT('Detect duplicate episodes in series'),
-    'explain-no_series_dupes' : TT('Detect identical episodes in series (based on "name/season/episode")'),
+    'explain-no_series_dupes' : TT('Detect identical episodes in series (based on "name/season/episode" of items in your History)'),
+    'opt-series_propercheck' : TT('Allow proper releases'),
+    'explain-series_propercheck' : TT('Bypass series duplicate detection if PROPER, REAL or REPACK is detected in the download name'),
     'nodupes-off' : TT('Off'), #: Three way switch for duplicates
-    'nodupes-ignore' : TT('Discard'), #: Three way switch for duplicates
-    'nodupes-pause' : TT('Pause'), #: Three way switch for duplicates
+    'nodupes-ignore' : TT('Discard'), #: Four way switch for duplicates
+    'nodupes-pause' : TT('Pause'), #: Four way switch for duplicates
+    'nodupes-fail' : TT('Fail job (move to History)'), #: Four way switch for duplicates
+    'nodupes-tag' : TT('Tag job'), #: Four way switch for duplicates
     'abort' : TT('Abort'), #: Three way switch for encrypted posts
     'opt-action_on_unwanted_extensions' : TT('Action when unwanted extension detected'),
     'explain-action_on_unwanted_extensions' : TT('Action when an unwanted extension is detected in RAR files'),
@@ -440,20 +460,16 @@ SKIN_TEXT = {
     'explain-unwanted_extensions' : TT('List all unwanted extensions. For example: <b>exe</b> or <b>exe, com</b>'),
     'opt-sfv_check' : TT('Enable SFV-based checks'),
     'explain-sfv_check' : TT('Do an extra verification based on SFV files.'),
-    'opt-unpack_check' : TT('Check result of unpacking'),
-    'explain-unpack_check' : TT('Check result of unpacking (needs to be off for some file systems).'),
     'opt-script_can_fail' : TT('User script can flag job as failed'),
     'explain-script_can_fail' : TT('When the user script returns a non-zero exit code, the job will be flagged as failed.'),
     'opt-new_nzb_on_failure' : TT('On failure, try alternative NZB'),
     'explain-new_nzb_on_failure' : TT('Some servers provide an alternative NZB when a download fails.'),
     'opt-enable_meta' : TT('Use tags from indexer'),
-    'explain-enable_meta' : TT('Use tags from indexer for title, season, episode, etc. Otherwise all naming is derived from the NZB name.'),
+    'explain-enable_meta' : TT('When sorting, use tags from indexer for title, season, episode, etc. Otherwise all naming is derived from the NZB name.'),
     'opt-folder_rename' : TT('Enable folder rename'),
     'explain-folder_rename' : TT('Use temporary names during post processing. Disable when your system doesn\'t handle that properly.'),
     'opt-pre_script' : TT('Pre-queue user script'),
     'explain-pre_script' : TT('Used before an NZB enters the queue.'),
-    'opt-par2_multicore' : TT('Enable MultiCore Par2'),
-    'explain-par2_multicore' : TT('Read the Wiki Help on this!'),
     'opt-par_option' : TT('Extra PAR2 Parameters'),
     'explain-par_option' : TT('Read the Wiki Help on this!'),
     'opt-nice' : TT('Nice Parameters'),
@@ -464,6 +480,8 @@ SKIN_TEXT = {
     'explain-auto_disconnect' : TT('Disconnect from Usenet server(s) when queue is empty or paused.'),
     'opt-auto_sort' : TT('Sort by Age'),
     'explain-auto_sort' : TT('Automatically sort items by (average) age.'),
+    'opt-direct_unpack' : TT('Direct Unpack'),
+    'explain-direct_unpack' : TT('Jobs will start unpacking during the downloading to reduce post-processing time. Only works for jobs that do not need repair.'),
     'opt-propagation_delay' : TT('Propagation delay'),
     'explain-propagation_delay' : TT('Posts will be paused untill they are at least this age. Setting job priority to Force will skip the delay.'),
     'opt-check_new_rel' : TT('Check for New Release'),
@@ -473,8 +491,6 @@ SKIN_TEXT = {
     'explain-replace_spaces' : TT('Replace spaces with underscores in folder names.'),
     'opt-replace_dots' : TT('Replace dots in Foldername'),
     'explain-replace_dots' : TT('Replace dots with spaces in folder names.'),
-    'opt-replace_illegal' : TT('Replace Illegal Characters in Folder Names'),
-    'explain-replace_illegal' : TT('Replace illegal characters in folder names by equivalents (otherwise remove).'),
     'opt-sanitize_safe' : TT('Make Windows compatible'),
     'explain-sanitize_safe' : TT('For servers: make sure names are compatible with Windows.'),
     'opt-auto_browser' : TT('Launch Browser on Startup'),
@@ -483,11 +499,9 @@ SKIN_TEXT = {
     'explain-pause_on_post_processing' : TT('Pauses downloading at the start of post processing and resumes when finished.'),
     'opt-ignore_samples' : TT('Ignore Samples'),
     'explain-ignore_samples' : TT('Filter out sample files (e.g. video samples).'),
-    'igsam-off' : TT('Off'),
     'igsam-del' : TT('Delete after download'),
     'opt-enable_https_verification' : TT('HTTPS certificate verification'),
     'explain-enable_https_verification' : TT('Verify certificates when connecting to indexers and RSS-sources using HTTPS.'),
-    'swtag-general' : TT('General'),
     'swtag-server' : TT('Server'),
     'swtag-queue' : TT('Queue'),
     'swtag-pp' : TT('Post processing'),
@@ -504,19 +518,16 @@ SKIN_TEXT = {
     'explain-quota_period' : TT('Does the quota get reset each day, week or month?'),
     'opt-pre_check' : TT('Check before download'),
     'explain-pre_check' : TT('Try to predict successful completion before actual download (slower!)'),
+    'opt-ssl_ciphers' : TT('SSL Ciphers'),
+    'explain-ssl_ciphers' : TT('Increase performance by forcing a lower SSL encryption strength.'),
     'opt-max_art_tries' : TT('Maximum retries'),
     'explain-max_art_tries' : TT('Maximum number of retries per server'),
-    'opt-max_art_opt' : TT('Only for optional servers'),
-    'explain-max_art_opt' : TT('Apply maximum retries only to optional servers'),
     'opt-fail_hopeless_jobs' : TT('Abort jobs that cannot be completed'),
     'explain-fail_hopeless_jobs' : TT('When during download it becomes clear that too much data is missing, abort the job'),
-    'opt-rating_enable' : TT('Enable OZnzb Integration'),
-    'explain-rating_enable' : TT('Enhanced functionality including ratings and extra status information is available when connected to OZnzb indexer.'),
+    'opt-rating_enable' : TT('Enable Indexer Integration'),
+    'explain-rating_enable' : TT('Indexers can supply rating information when a job is added and SABnzbd can report to the indexer if a job couldn\'t be completed.'),
     'opt-rating_api_key' : TT('API Key'),
-    'opt-rating_host' : TT('Server address'),
-    'explain-rating_api_key' : TT('This key provides identity to indexer. Refer to https://www.oznzb.com/profile.'),
-    'opt-rating_feedback' : TT('Automatic Feedback'),
-    'explain-rating_feedback' : TT('Send automatically calculated validation results for downloads to indexer.'),
+    'explain-rating_api_key' : TT('This key provides identity to indexer. Check your profile on the indexer\'s website.'),
     'opt-rating_filter_enable' : TT('Enable Filtering'),
     'explain-rating_filter_enable' : TT('Action downloads according to filtering rules.'),
     'opt-rating_filter_abort_if' : TT('Abort If'),
@@ -546,12 +557,17 @@ SKIN_TEXT = {
     'srv-connections' : TT('Connections'), #: Server: amount of connections
     'srv-retention' : TT('Retention time'), #: Server's retention time in days
     'srv-ssl' : TT('SSL'), #: Server SSL tickbox
+    'explain-ssl' : TT('Secure connection to server'), #: Server SSL tickbox
+    'opt-ssl_verify' : TT('Certificate verification'),
+    'explain-ssl_verify' : TT('Minimal: when SSL is enabled, verify the identity of the server using its certificates. Strict: verify and enforce matching hostname.'),
+    'ssl_verify-disabled' : TT('Disabled'),
+    'ssl_verify-normal' : TT('Minimal'),
+    'ssl_verify-strict' : TT('Strict'),
     'srv-priority' : TT('Priority'), #: Server priority
     'explain-svrprio' : TT('0 is highest priority, 100 is the lowest priority'), #: Explain server priority
     'srv-optional' : TT('Optional'), #: Server optional tickbox
+    'explain-optional' : TT('For unreliable servers, will be ignored longer in case of failures'), #: Explain server optional tickbox
     'srv-enable' : TT('Enable'), #: Enable server tickbox
-    'srv-ssl_type' : TT('SSL type'),
-    'srv-explain-ssl_type' : TT('Use the top value, unless your Usenet provider requires otherwise!'),
     'button-addServer' : TT('Add Server'), #: Button: Add server
     'button-delServer' : TT('Remove Server'), #: Button: Remove server
     'button-testServer' : TT('Test Server'), #: Button: Test server
@@ -580,12 +596,12 @@ SKIN_TEXT = {
 # Config->RSS
     'explain-RSS' : TT('The checkbox next to the feed name should be ticked for the feed to be enabled and be automatically checked for new items.<br />When a feed is added, it will only pick up new items and not anything already in the RSS feed unless you press "Force Download".'),
     'feed' : TT('Feed'), #: Config->RSS, tab header
+    'addMultipleFeeds' : TT('Seperate multiple URLs by a comma'), #: Config->RSS, placeholder (cannot be too long)
     'button-preFeed' : TT('Read Feed'),#: Config->RSS button
     'button-forceFeed' : TT('Force Download'),#: Config->RSS button
     'rss-order' : TT('Order'), #: Config->RSS table column header
     'rss-type' : TT('Type'), #: Config->RSS table column header
     'rss-filter' : TT('Filter'), #: Config->RSS table column header
-    'rss-skip' : TT('Skip'), #: Config->RSS table column header
     'rss-accept' : TT('Accept'), #: Config->RSS filter-type selection menu
     'rss-reject' : TT('Reject'), #: Config->RSS filter-type selection menu
     'rss-must' : TT('Requires'), #: Config->RSS filter-type selection menu
@@ -593,9 +609,11 @@ SKIN_TEXT = {
     'rss-atleast' : TT('At least'), #: Config->RSS filter-type selection menu
     'rss-atmost' : TT('At most'), #: Config->RSS filter-type selection menu
     'rss-from' : TT('From SxxEyy'), #: Config->RSS filter-type selection menu "From Season/Episode"
+    'rss-from-show' : TT('From Show SxxEyy'), #: Config->RSS filter-type selection menu "From Show Season/Episode"
     'rss-matched' : TT('Matched'), #: Config->RSS section header
     'rss-notMatched' : TT('Not Matched'), #: Config->RSS section header
     'rss-done' : TT('Downloaded'), #: Config->RSS section header
+    'rss-added' : TT('Added NZB'), #: Config->RSS after adding to queue
     'link-download' : TT('Download'), #: Config->RSS button "download item"
     'button-rssNow' : TT('Read All Feeds Now'), #: Config->RSS button
 
@@ -648,6 +666,10 @@ SKIN_TEXT = {
     'explain-pushover_userkey' : TT('User Key (required)'), #: Pushover settings
     'opt-pushover_device' : TT('Device(s)'), #: Pushover settings
     'explain-pushover_device' : TT('Device(s) to which message should be sent'), #: Pushover settings
+    'opt-pushover_emergency_retry' : TT('Emergency retry'), #: Pushover settings
+    'explain-pushover_emergency_retry' : TT('How often (in seconds) the same notification will be sent'), #: Pushover settings
+    'opt-pushover_emergency_expire' : TT('Emergency expire'), #: Pushover settings
+    'explain-pushover_emergency_expire' : TT('How many seconds your notification will continue to be retried'), #: Pushover settings
     'section-Pushbullet' : TT('Pushbullet'), #: Header for Pushbullet notification section
     'opt-pushbullet_enable' : TT('Enable Pushbullet notifications'), #: Pushbullet settings
     'explain-pushbullet_enable' : TT('Requires a Pushbullet account'), #: Pushbulletsettings
@@ -664,11 +686,11 @@ SKIN_TEXT = {
     'explain-nscript_parameters' : TT('Read the Wiki Help on this!'), #: Notification Script settings
 
 # Config->Cat
-    'explain-catTags' : TT('Use the "Groups / Indexer tags" column to map groups and tags to your categories.<br/>Wildcards are supported. Use commas to separate terms.'),
+    'explain-catTags' : TT('Indexers can supply a category inside the NZB which SABnzbd will try to match to the categories defined below. Additionally, you can add terms to "Indexer Categories / Groups" to match more categories. Use commas to separate terms. Wildcards in the terms are supported. <br>More information can be found on the Wiki.'),
     'explain-catTags2' : TT('Ending the path with an asterisk * will prevent creation of job folders.'),
     'explain-relFolder' : TT('Relative folders are based on'),
     'catFolderPath' : TT('Folder/Path'),
-    'catTags' : TT('Groups / Indexer tags'),
+    'catTags' : TT('Indexer Categories / Groups'),
     'button-delCat' : TT('X'), #: Small delete button
 
 # Config->Sorting
@@ -676,9 +698,10 @@ SKIN_TEXT = {
     'opt-tvsort' : TT('Enable TV Sorting'),
     'sort-legenda' : TT('Pattern Key'),
     'button-clear' : TT('Clear'),
+    'button-evalFeed' : TT('Apply filters'),
     'presetSort' : TT('Presets'),
     'example' : TT('Example'),
-    'movieSort' : TT('Generic Sorting'),
+    'movieSort' : TT('Movie Sorting'),
     'opt-movieSort' : TT('Enable Movie Sorting'),
     'opt-movieExtra' : TT('Keep loose downloads in extra folders'),
     'affectedCat' : TT('Affected Categories'),
@@ -689,6 +712,7 @@ SKIN_TEXT = {
     'button-SeasonS01E05' : TT('S01E05 Season Folder'),
     'button-Ep1x05' : TT('1x05 Episode Folder'),
     'button-EpS01E05' : TT('S01E05 Episode Folder'),
+    'button-FileLikeFolder' : TT('Job Name as Filename'),
     'sort-title' : TT('Title'),
     'movie-sp-name' : TT('Movie Name'),
     'movie-dot-name' : TT('Movie.Name'),
@@ -708,12 +732,11 @@ SKIN_TEXT = {
     'partNumber' : TT('Part Number'),
     'decade' : TT('Decade'),
     'orgFilename' : TT('Original Filename'),
-    'orgDirname' : TT('Original Foldername'),
+    'orgJobname' : TT('Original Job Name'),
     'lowercase' : TT('Lower Case'),
     'TEXT' : TT('TEXT'),
     'text' : TT('text'),
     'sort-File' : TT('file'),
-    'sort-Folder' : TT('folder'),
     'sortString' : TT('Sort String'),
     'multiPartLabel' : TT('Multi-part label'),
     'button-inFolders' : TT('In folders'),
@@ -785,6 +808,7 @@ SKIN_TEXT = {
     'Glitter-filtered' : TT('FILTERED'),
     'Glitter-waitSec' : TT('WAIT %s sec'),
     'Glitter-error' : TT('ERROR:'),
+    'Glitter-fetch' : TT('Fetch'),
     'Glitter-interfaceOptions' : TT('Web Interface'),
     'Glitter-interfaceRefresh' : TT('Refresh rate'),
     'Glitter-useGlobalOptions' : TT('Use global interface settings'),
@@ -794,9 +818,9 @@ SKIN_TEXT = {
     'Glitter-showExtraQueueColumn' : TT('Extra queue column'),
     'Glitter-showExtraHistoryColumn' : TT('Extra history column'),
     'Glitter-page' : TT('page'),
-    'Glitter-everything' : TT('Everything'),
     'Glitter-loading' : TT('Loading'),
     'Glitter-articles' : TT('articles'),
+    'Glitter-rename' : TT('Rename'),
     'Glitter-repairQueue' : TT('Queue repair'),
     'Glitter-showActiveConnections' : TT('Show active connections'),
     'Glitter-unblockServer' : TT('Unblock'),
@@ -816,6 +840,8 @@ SKIN_TEXT = {
     'Glitter-noSelect' : TT('Nothing selected!'),
     'Glitter-removeSelected' : TT('Remove all selected files'),
     'Glitter-toggleCompletedFiles' : TT('Hide/show completed files'),
+    'Glitter-top' : TT('Top'),
+    'Glitter-bottom' : TT('Bottom'),
     'Glitter-retryJob' : TT('Retry'),
     'Glitter-more' : TT('More'),
     'Glitter-scriptLog' : TT('View Script Log'),
@@ -825,7 +851,7 @@ SKIN_TEXT = {
     'Glitter-confirmClear1Download' : TT('Are you sure?'),
     'Glitter-updateAvailable' : TT('Update Available!'),
     'Glitter-noLocalStorage' : TT('LocalStorage (cookies) are disabled in your browser, interface settings will be lost after you close the browser!'), #: Don't translate LocalStorage
-    'Glitter-glitterTips' : TT('Glitter has some (new) features you might like!'), 
+    'Glitter-glitterTips' : TT('Glitter has some (new) features you might like!'),
     'Glitter-custom' : TT('Custom'),
     'Glitter-displayCompact' : TT('Compact layout'),
     'Glitter-displayTabbed' : TT('Tabbed layout <br/>(separate queue and history)'),
@@ -835,6 +861,8 @@ SKIN_TEXT = {
     'Glitter-pausePrompt': TT('How long or untill when do you want to pause? (in English!)'),
     'Glitter-pausePromptFail': TT('Sorry, we could not interpret that. Try again.'),
     'Glitter-pauseFor' : TT('Pause for...'),
+    'Glitter-refresh' : TT('Refresh'),
+    'Glitter-logText' : TT('All usernames, passwords and API-keys are automatically removed from the log and the included copy of your settings.'),
     'Glitter-sortAgeAsc' : TT('Sort by Age <small>Oldest&rarr;Newest</small>'),
     'Glitter-sortAgeDesc' : TT('Sort by Age <small>Newest&rarr;Oldest</small>'),
     'Glitter-sortNameAsc' : TT('Sort by Name <small>A&rarr;Z</small>'),
@@ -962,11 +990,9 @@ SKIN_TEXT = {
     'wizard-server-text' :  TT('Click to test the entered details.'),
     'wizard-example' :  TT('E.g.'), #: Abbreviation for "for example"
     'wizard-button-testServer' :  TT('Test Server'), #: Wizard step
-    'wizard-restarting' :  TT('Restarting SABnzbd...'), #: Wizard step
     'wizard-complete' :  TT('Setup is now complete!'), #: Wizard step
     'wizard-tip1' :  TT('SABnzbd will now be running in the background.'), #: Wizard tip
     'wizard-tip2' :  TT('Closing any browser windows/tabs will NOT close SABnzbd.'), #: Wizard tip
-    'wizard-tip3' :  TT('After SABnzbd has finished restarting you will be able to access it at the following location: %s'), #: Wizard tip
     'wizard-tip4' :  TT('It is recommended you right click and bookmark this location and use this bookmark to access SABnzbd when it is running in the background.'), #: Wizard tip
     'wizard-tip-wiki' :  TT('Further help can be found on our'), #: Will be appended with a wiki-link, adjust word order accordingly
     'wizard-goto' :  TT('Go to SABnzbd'), #: Wizard step
@@ -978,5 +1004,7 @@ SKIN_TEXT = {
 SABnzbd comes with ABSOLUTELY NO WARRANTY.
 This is free software, and you are welcome to redistribute it under certain conditions.
 It is licensed under the GNU GENERAL PUBLIC LICENSE Version 2 or (at your option) any later version.
-''')
+'''),
+    'wizard-ad-1': TT('In order to download from usenet you will require access to a provider. Your ISP may provide you with access, however a premium provider is recommended.'),
+    'wizard-ad-2': TT('Don\'t have a usenet provider? We recommend trying %s.'),
     }
