@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -26,10 +26,11 @@ import re
 import logging
 import Queue
 import urllib2
+from httplib import IncompleteRead
 from threading import Thread
 
 import sabnzbd
-from sabnzbd.constants import FUTURE_Q_FOLDER, Status
+from sabnzbd.constants import DEF_TIMEOUT, MAX_URL_RETRIES, FUTURE_Q_FOLDER, Status
 from sabnzbd.encoding import unicoder
 import sabnzbd.misc as misc
 import sabnzbd.dirscanner as dirscanner
@@ -40,8 +41,8 @@ import sabnzbd.notifier as notifier
 
 
 _BAD_GZ_HOSTS = ('.zip', 'nzbsa.co.za', 'newshost.za.net')
-_RARTING_FIELDS = ('x-rating-id', 'x-rating-url', 'x-rating-host', 'x-rating-video', 'x-rating-videocnt', 'x-rating-audio', 'x-rating-audiocnt', 
-                    'x-rating-voteup', 'x-rating-votedown', 'x-rating-spam', 'x-rating-confirmed-spam', 'x-rating-passworded', 'x-rating-confirmed-passworded')
+_RARTING_FIELDS = ('x-rating-id', 'x-rating-url', 'x-rating-host', 'x-rating-video', 'x-rating-videocnt', 'x-rating-audio', 'x-rating-audiocnt',
+    'x-rating-voteup', 'x-rating-votedown', 'x-rating-spam', 'x-rating-confirmed-spam', 'x-rating-passworded', 'x-rating-confirmed-passworded')
 
 
 class URLGrabber(Thread):
@@ -58,8 +59,17 @@ class URLGrabber(Thread):
 
     def add(self, url, future_nzo, when=None):
         """ Add an URL to the URLGrabber queue, 'when' is seconds from now """
-        if when and future_nzo:
-            future_nzo.wait = time.time() + when
+        if future_nzo and when:
+            # Always increase counter
+            future_nzo.url_tries += 1
+
+            # Too many tries? Cancel
+            if future_nzo.url_tries > MAX_URL_RETRIES:
+                bad_fetch(future_nzo, url, T('Maximum retries'))
+                return
+
+            future_nzo.url_wait = time.time() + when
+
         self.queue.put((url, future_nzo))
 
     def stop(self):
@@ -72,19 +82,24 @@ class URLGrabber(Thread):
         self.shutdown = False
 
         while not self.shutdown:
-            # Don't pound the website!
-            time.sleep(5.0)
-
             (url, future_nzo) = self.queue.get()
 
             if not url:
                 # stop signal, go test self.shutdown
                 continue
-            if future_nzo and future_nzo.wait and future_nzo.wait > time.time():
-                # Re-queue when too early and still active
 
-                self.add(url, future_nzo)
-                continue
+            if future_nzo:
+                # Re-queue when too early and still active
+                if future_nzo.url_wait and future_nzo.url_wait > time.time():
+                    self.add(url, future_nzo)
+                    time.sleep(1.0)
+                    continue
+                # Paused
+                if future_nzo.status == Status.PAUSED:
+                    self.add(url, future_nzo)
+                    time.sleep(1.0)
+                    continue
+
             url = url.replace(' ', '')
 
             try:
@@ -98,11 +113,6 @@ class URLGrabber(Thread):
                         logging.debug('Dropping URL %s, job entry missing', url)
                         continue
 
-                logging.info('Grabbing URL %s', url)
-                req = urllib2.Request(url)
-                req.add_header('User-Agent', 'SABnzbd+/%s' % sabnzbd.version.__version__)
-                if not any(item in url for item in _BAD_GZ_HOSTS):
-                    req.add_header('Accept-encoding', 'gzip')
                 filename = None
                 category = None
                 gzipped = False
@@ -110,15 +120,18 @@ class URLGrabber(Thread):
                 wait = 0
                 retry = True
                 fn = None
+
+                logging.info('Grabbing URL %s', url)
                 try:
-                    fn = urllib2.urlopen(req)
-                except:
+                    fn = _build_request(url)
+                except Exception, e:
                     # Cannot list exceptions here, because of unpredictability over platforms
                     error0 = str(sys.exc_info()[0]).lower()
                     error1 = str(sys.exc_info()[1]).lower()
                     logging.debug('Error "%s" trying to get the url %s', error1, url)
                     if 'certificate_verify_failed' in error1 or 'certificateerror' in error0:
                         msg = T('Server %s uses an untrusted HTTPS certificate') % ''
+                        msg += ' - https://sabnzbd.org/certificate-errors'
                         retry = False
                     elif 'nodename nor servname provided' in error1:
                         msg = T('Server name does not resolve')
@@ -129,6 +142,9 @@ class URLGrabber(Thread):
                     elif '404' in error1:
                         msg = T('File not on server')
                         retry = False
+                    elif hasattr(e, 'headers') and 'retry-after' in e.headers:
+                        # Catch if the server send retry (e.headers is case-INsensitive)
+                        wait = misc.int_conv(e.headers['retry-after'])
 
                 new_url = dereferring(url, fn)
                 if new_url:
@@ -165,7 +181,6 @@ class URLGrabber(Thread):
                         elif item == 'x-dnzb-password':
                             nzo_info['password'] = value
                         elif item == 'retry-after':
-                            # For NZBFinder
                             wait = misc.int_conv(value)
 
                         # Rating fields
@@ -181,7 +196,7 @@ class URLGrabber(Thread):
                     retry = True
                     fn = None
                 elif retry:
-                    fn, msg, retry, wait, data = _analyse(fn, url)
+                    fn, msg, retry, wait, data = _analyse(fn, url, future_nzo)
 
                 if not fn:
                     if retry:
@@ -209,13 +224,18 @@ class URLGrabber(Thread):
                 if gzipped:
                     filename += '.gz'
                 if not data:
-                    data = fn.read()
+                    try:
+                        data = fn.read()
+                    except (IncompleteRead, IOError):
+                        bad_fetch(future_nzo, url, T('Server could not complete request'))
+                        fn.close()
+                        continue
                 fn.close()
 
                 if '<nzb' in data and misc.get_ext(filename) != '.nzb':
                     filename += '.nzb'
 
-                # Sanatize filename first
+                # Sanitize filename first (also removing forbidden Windows-names)
                 filename = misc.sanitize_filename(filename)
 
                 # Write data to temp file
@@ -267,26 +287,50 @@ class URLGrabber(Thread):
                 logging.debug("URLGRABBER Traceback: ", exc_info=True)
 
 
-def _analyse(fn, url):
+def _build_request(url):
+    # Detect basic auth
+    # Adapted from python-feedparser
+    urltype, rest = urllib2.splittype(url)
+    realhost, rest = urllib2.splithost(rest)
+    if realhost:
+        user_passwd, realhost = urllib2.splituser(realhost)
+        if user_passwd:
+            url = '%s://%s%s' % (urltype, realhost, rest)
+
+    # Start request
+    req = urllib2.Request(url)
+
+    # Add headers
+    req.add_header('User-Agent', 'SABnzbd+/%s' % sabnzbd.version.__version__)
+    if not any(item in url for item in _BAD_GZ_HOSTS):
+        req.add_header('Accept-encoding', 'gzip')
+    if user_passwd:
+        req.add_header('Authorization', 'Basic ' + user_passwd.encode('base64').strip())
+    return urllib2.urlopen(req)
+
+
+def _analyse(fn, url, future_nzo):
     """ Analyze response of indexer
         returns fn|None, error-message|None, retry, wait-seconds, data
     """
     data = None
     if not fn or fn.code != 200:
-        logging.debug('No usable response from indexer, retry after 60 sec')
         if fn:
             msg = fn.msg
         else:
             msg = ''
-        return None, msg, True, 60, data
+
+        # Increasing wait-time in steps for standard errors
+        when = DEF_TIMEOUT * (future_nzo.url_tries + 1)
+        logging.debug('No usable response from indexer, retry after %s sec', when)
+        return None, msg, True, when, data
 
     # Check for an error response
     if not fn or fn.msg != 'OK':
-        logging.debug('Received nothing from indexer, retry after 60 sec')
-        return None, fn.msg, True, 60, data
-
-    if '.oznzb.com' in url and 'login?' in fn.url:
-        return None, T('Unauthorized access'), False, 0, data
+        # Increasing wait-time in steps for standard errors
+        when = DEF_TIMEOUT * (future_nzo.url_tries + 1)
+        logging.debug('Received nothing from indexer, retry after %s sec', when)
+        return None, fn.msg, True, when, data
 
     return fn, fn.msg, False, 0, data
 
@@ -332,7 +376,7 @@ def bad_fetch(nzo, url, msg='', content=False):
 
     nzo.fail_msg = msg
 
-    notifier.send_notification(T('URL Fetching failed; %s') % '', '%s\n%s' % (msg, url), 'other')
+    notifier.send_notification(T('URL Fetching failed; %s') % '', '%s\n%s' % (msg, url), 'other', nzo.cat)
     if cfg.email_endjob() > 0:
         emailer.badfetch_mail(msg, url)
 
