@@ -1,5 +1,5 @@
 #!/usr/bin/python -OO
-# Copyright 2008-2015 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2008-2017 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,7 +24,7 @@ import sys
 import select
 
 from sabnzbd.newswrapper import NewsWrapper
-from sabnzbd.downloader import Server, clues_login, clues_too_many
+from sabnzbd.downloader import Server, clues_login, clues_too_many, nntp_to_msg
 from sabnzbd.config import get_servers
 from sabnzbd.misc import int_conv
 
@@ -41,7 +41,7 @@ def test_nntp_server_dict(kwargs):
     if not connections:
         return False, T('There are no connections set. Please set at least one connection.')
     ssl = int_conv(kwargs.get('ssl', 0))
-    ssl_type = kwargs.get('ssl_type')
+    ssl_verify = int_conv(kwargs.get('ssl_verify', 1))
     port = int_conv(kwargs.get('port', 0))
     if not port:
         if ssl:
@@ -50,10 +50,10 @@ def test_nntp_server_dict(kwargs):
             port = 119
 
     return test_nntp_server(host, port, server, username=username,
-                        password=password, ssl=ssl, ssl_type=ssl_type)
+                        password=password, ssl=ssl, ssl_verify=ssl_verify)
 
 
-def test_nntp_server(host, port, server=None, username=None, password=None, ssl=None, ssl_type=None):
+def test_nntp_server(host, port, server=None, username=None, password=None, ssl=None, ssl_verify=1):
     """ Will connect (blocking) to the nttp server and report back any errors """
     timeout = 4.0
     if '*' in password and not password.strip('*'):
@@ -62,7 +62,7 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
             servers = get_servers()
             got_pass = False
             for server in servers:
-                if host in server:
+                if host in servers[server].host():
                     srv = servers[server]
                     password = srv.password()
                     got_pass = True
@@ -74,7 +74,7 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
         if not got_pass:
             return False, T('Password masked in ******, please re-enter')
     try:
-        s = Server(-1, '', host, port, timeout, 0, 0, ssl, ssl_type, False, username, password)
+        s = Server(-1, '', host, port, timeout, 0, 0, ssl, ssl_verify, False, username, password)
     except:
         return False, T('Invalid server details')
 
@@ -82,20 +82,25 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
         nw = NewsWrapper(s, -1, block=True)
         nw.init_connect(None)
         while not nw.connected:
-            nw.lines = []
+            nw.clear_data()
             nw.recv_chunk(block=True)
             #more ssl related: handle 1/n-1 splitting to prevent Rizzo/Duong-Beast
             read_sockets, _, _ = select.select([nw.nntp.sock], [], [], 0.1)
             if read_sockets:
                 nw.recv_chunk(block=True)
-            nw.finish_connect(nw.lines[0][:3])
+            nw.finish_connect(nw.status_code)
 
     except socket.timeout, e:
         if port != 119 and not ssl:
             return False, T('Timed out: Try enabling SSL or connecting on a different port.')
         else:
             return False, T('Timed out')
+
     except socket.error, e:
+        # Trying SSL on non-SSL port?
+        if 'unknown protocol' in str(e).lower():
+            return False, T('Unknown SSL protocol: Try disabling SSL or connecting on a different port.')
+
         return False, unicode(e)
 
     except TypeError, e:
@@ -111,32 +116,25 @@ def test_nntp_server(host, port, server=None, username=None, password=None, ssl=
     if not username or not password:
         nw.nntp.sock.sendall('ARTICLE <test@home>\r\n')
         try:
-            nw.lines = []
+            nw.clear_data()
             nw.recv_chunk(block=True)
         except:
             return False, unicode(sys.exc_info()[1])
 
-    # Could do with making a function for return codes to be used by downloader
-    try:
-        code = nw.lines[0][:3]
-    except IndexError:
-        code = ''
-        nw.lines.append('')
-
-    if code == '480':
+    if nw.status_code == '480':
         return False, T('Server requires username and password.')
 
-    elif code == '100' or code.startswith('2') or code.startswith('4'):
+    elif nw.status_code == '100' or nw.status_code.startswith(('2', '4')):
         return True, T('Connection Successful!')
 
-    elif code == '502' or clues_login(nw.lines[0]):
+    elif nw.status_code == '502' or clues_login(nntp_to_msg(nw.data)):
         return False, T('Authentication failed, check username/password.')
 
     elif clues_too_many(nw.lines[0]):
         return False, T('Too many connections, please pause downloading or try again later')
 
     else:
-        return False, T('Could not determine connection result (%s)') % nw.lines[0]
+        return False, T('Could not determine connection result (%s)') % nntp_to_msg(nw.data)
 
     # Close the connection
     nw.terminate(quit=True)
