@@ -4,14 +4,14 @@
 # http://www.brunningonline.net/simon/blog/archives/SysTrayIcon.py.html
 # modified on 2011-10-04 by Jan Schejbal to support threading and preload icons
 #  override doUpdates to perform actions inside the icon thread
+#  override click to perform actions when left-clicking the icon
 
 import os
-import sys
-
 import pywintypes
 import win32api
 import win32con
 import win32gui_struct
+import timer
 try:
     import winxpgui as win32gui
 except ImportError:
@@ -47,6 +47,7 @@ class SysTrayIconThread(Thread):
         self.menu_actions_by_id = dict(self.menu_actions_by_id)
         del self._next_action_id
 
+        self.click_timer = None
         self.default_menu_index = (default_menu_index or 0)
         self.window_class_name = window_class_name or "SysTrayIconPy"
 
@@ -91,7 +92,7 @@ class SysTrayIconThread(Thread):
             sleep(0.100)
         win32gui.Shell_NotifyIcon(win32gui.NIM_DELETE, (self.hwnd, 0))
 
-    # override this
+    # Override this
     def doUpdates(self):
         pass
 
@@ -177,12 +178,19 @@ class SysTrayIconThread(Thread):
         win32gui.PostQuitMessage(0)  # Terminate the app.
 
     def notify(self, hwnd, msg, wparam, lparam):
+        # Double click is actually 1 single click followed
+        # by a double-click event, no way to differentiate
+        # So we need a timed callback to cancel
         if lparam == win32con.WM_LBUTTONDBLCLK:
             self.execute_menu_option(self.default_menu_index + self.FIRST_ID)
+            self.stop_click_timer()
         elif lparam == win32con.WM_RBUTTONUP:
             self.show_menu()
-        elif lparam == win32con.WM_LBUTTONUP:
-            pass
+        elif lparam == win32con.WM_LBUTTONDOWN:
+            # Wrapper of win32api, timeout is in ms
+            # We need to wait at least untill what user has defined as double click
+            self.stop_click_timer()
+            self.click_timer = timer.set_timer(win32gui.GetDoubleClickTime(), self.click)
         return True
 
     def show_menu(self):
@@ -204,7 +212,18 @@ class SysTrayIconThread(Thread):
             win32gui.PostMessage(self.hwnd, win32con.WM_NULL, 0, 0)
         except pywintypes.error:
             # Weird PyWin/win32gui bug, just ignore it for now
-            logging.debug('win32gui problem, cannot show SysTray menu')
+            pass
+
+    # Override this for left-click action
+    # Need to call the stop-timer in that function!
+    def click(self, *args):
+        pass
+
+    def stop_click_timer(self):
+        # Stop the timer
+        if self.click_timer:
+            timer.kill_timer(self.click_timer)
+            self.click_timer = None
 
     def create_menu(self, menu, menu_options):
         for option_text, option_icon, option_action, option_id in menu_options[::-1]:
